
#include <math.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qmotifplusstyle.h>
#include <qprinter.h>
#include <stdlib.h>

/** see ../widgets dir */
#include "qsmatrix.h"
#include "qsplotview.h"
#include "qsaxes3d.h"
#include "qssurface.h"
#include "qsfigure.h"

class Surface1 : public QSMatrix {
     
 public:
    Surface1() {}
    ~Surface1() {}
    
    int cols() const { return 100; }
    int rows() const { return 100; }
    
    double value( int row, int col ) { return sin( sqrt((row/8-50)*(row/8-50)+(col/8-20)*(col/8-20)) ); }
    };



int main( int argc, char ** argv ) {
    QApplication a( argc, argv );
    QApplication::setStyle( new QMotifPlusStyle() );

    QMainWindow *mw = new QMainWindow();

    QSWorkbook *doc = new QSWorkbook( mw );
    doc->clear();
    doc->pageAdd( new QSPage() );
    doc->setPrinter( new QPrinter() );
    QSPlotView *view = new QSPlotView( mw );
    view->setWorkbook( doc );
    mw->setCentralWidget( view );

    QSAxes3D *axes  = new QSAxes3D();
    doc->page(0)->objects()->add( axes->shadowObject() );
    
    // axes displayed when full page is off
    view->selection()->set( axes->shadowObject() );
    //view->setFullPage( false );

    view->bindSlider( QSPlotView::HorizontalSlider, "azimuth", 0, 359 );
    view->bindSlider( QSPlotView::VerticalSlider,   "elevation", -90, 90  );

    QSSurface *s1 = new QSSurface( axes ); axes->plotAdd( s1 );
    s1->setMatrix( QSSurface::ZData, new Surface1() );
    s1->defaultAxis(QSAxis::XAxisType)->setRange(10.0,100.0);

    mw->setCaption( "surface demo" );
    mw->show();
    a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
    return a.exec();
}
