/***************************************************************************
                                  ksarrowpanel.cpp
                             -------------------
    begin                : Mon Jun 5 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksarrowpanel.h"
#include "../widgets/qsdrvqt.h"

KSArrowPanel::KSArrowPanel(QWidget *parent, const char *name)
: KSAttrPanel(parent,name)
 {
  initDialog();

  //
  //  Uwaga !
  //
  //colpanel->setEnabled( false );
  //

  QSGArrow a;
  a.style = QSGArrow::None;    new KSListBoxArrow( plist, a, i18n("None") );
  a.style = QSGArrow::Arrow;   new KSListBoxArrow( plist, a, i18n("Arrow") );
  a.style = QSGArrow::FArrow;  new KSListBoxArrow( plist, a, i18n("Filled Arrow") );
  a.style = QSGArrow::NArrow;  new KSListBoxArrow( plist, a, i18n("Narrow Arrow") );
  a.style = QSGArrow::RArrow;  new KSListBoxArrow( plist, a, i18n("Arrow") );
  a.style = QSGArrow::RFArrow; new KSListBoxArrow( plist, a, i18n("Filled Arrow") );
  a.style = QSGArrow::RNArrow; new KSListBoxArrow( plist, a, i18n("Narrow Arrow") );

  a.style = QSGArrow::Rect;    new KSListBoxArrow( plist, a, i18n("Rectangle") );
  a.style = QSGArrow::Diamond; new KSListBoxArrow( plist, a, i18n("Diamond") );
  a.style = QSGArrow::Circle;  new KSListBoxArrow( plist, a, i18n("Circle") );
  a.style = QSGArrow::Line;    new KSListBoxArrow( plist, a, i18n("Line") );
  a.style = QSGArrow::FDiagLine; new KSListBoxArrow( plist, a, i18n("FDiag Line") );
  a.style = QSGArrow::BDiagLine; new KSListBoxArrow( plist, a, i18n("BDiag Line") );

  plist->setHScrollBarMode( QScrollView::AlwaysOff );
  plist->setCurrentItem(0);
  sizes->setValue(5);
  connect( plist, SIGNAL(highlighted(int)), this, SLOT(changePanelArrow(int)) );
  connect( sizes, SIGNAL(valueChanged(int)), this, SLOT(changePanelArrow(int)) );
 }

//-------------------------------------------------------------//

KSArrowPanel::~KSArrowPanel()
 {
 }

//-------------------------------------------------------------//

void KSArrowPanel::setPanelArrow( const QSGArrow& new_arrow )
 {
  if ( a != new_arrow ) {
         int i;
         a = new_arrow;
         disconnect( plist, SIGNAL(highlighted(int)), this, SLOT(changePanelArrow(int)) );
         disconnect( sizes, SIGNAL(valueChanged(int)), this, SLOT(changePanelArrow(int)) );
         sizes->setValue( a.size );
         for ( i=0; i<(int )plist->count(); i++ ) {
                KSListBoxArrow *it = dynamic_cast<KSListBoxArrow*>(plist->item(i));
                if ( it && it->ar.style == a.style ) {
                         plist->setCurrentItem(i);
                         break;
                        }
               }
         connect( plist, SIGNAL(highlighted(int)), this, SLOT(changePanelArrow(int)) );
         connect( sizes, SIGNAL(valueChanged(int)), this, SLOT(changePanelArrow(int)) );
         emit panelArrowChanged();
        }
 }

//-------------------------------------------------------------//

void KSArrowPanel::changePanelArrow(int)	
 {
  KSListBoxArrow *it = dynamic_cast<KSListBoxArrow*>(plist->item(plist->currentItem()));
  if ( it ) {
         QSGArrow new_a = a;
         new_a.style = it->ar.style;
         new_a.size  = sizes->value();
         setPanelArrow( new_a );
        }
 }


//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

#define ARROWBOX_SIZE_W  20
#define ARROWBOX_SIZE_H  15
#define ARROWBOX_KSPACING_W  5
#define ARROWBOX_KSPACING_H  5

KSListBoxArrow::KSListBoxArrow( QListBox *parent, const QSGArrow &init_a, const QString& name )
:QListBoxItem(parent), ar(init_a)
 {
  setText( name );
 }

//-------------------------------------------------------------//

KSListBoxArrow::~KSListBoxArrow()
 {
 }

//-------------------------------------------------------------//

int KSListBoxArrow::height( const QListBox *lb ) const
 {
  return QMAX( ARROWBOX_SIZE_H+2*ARROWBOX_KSPACING_H, lb->fontMetrics().lineSpacing()+1 );
 }

//-------------------------------------------------------------//

int KSListBoxArrow::width( const QListBox *lb ) const
 {
  return lb->fontMetrics().width(text()) + ARROWBOX_SIZE_W + ARROWBOX_KSPACING_W*3;
 }

//-------------------------------------------------------------//

void KSListBoxArrow::paint( QPainter *p )
 {
  int w = p->fontMetrics().width(text()) + ARROWBOX_SIZE_W + 3*ARROWBOX_KSPACING_W;
  int h = QMAX( ARROWBOX_SIZE_H+2*ARROWBOX_KSPACING_H, p->fontMetrics().lineSpacing()+1 );

  QRect r( ARROWBOX_KSPACING_W,
           (h-ARROWBOX_SIZE_H)/2,
           ARROWBOX_SIZE_W,
           ARROWBOX_SIZE_H );

  QSDrvQt drv;
  QSGLine l; 
  l.color = QSDrvQt::toQSGColor(p->pen().color());
  QSGArrow temp_a = ar; 
  temp_a.size = 5;
  drv.setDC(p,72,false);
  drv.startDrawing();
  drv.setLine(l);
  drv.drawLineEnd(QSPt2f(r.center().x(),r.center().y()),0,temp_a);
  drv.stopDrawing();
  p->drawText( ARROWBOX_SIZE_W+2*ARROWBOX_KSPACING_W,
               0, w, h, Qt::AlignLeft | Qt::AlignVCenter, text() );
 }

