/***************************************************************************
                                 ksattrbtn.cpp
                             -------------------
    begin                : Fri Jan 7 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksattrbtn.h"
#include "../widgets/qsdrvqt.h"
#include "ksattrpanel.h"
#include "ksfillpanel.h"
#include "ksfontpanel.h"
#include "kslinepanel.h"
#include "kspointpanel.h"
#include "ksarrowpanel.h"
#include <qpainter.h>

#define BTN_HSPACE	5
#define BTN_VSPACE	3

KSAttrBtn::KSAttrBtn(QWidget *parent, const char *name )
: QPushButton(parent,name)
 {
  apanel = NULL;
  setDefault( false );
  setAutoDefault( false );
  connect( this, SIGNAL(clicked()), this, SLOT(showAttrPanel()) );
 }

//-----------------------------------------------------------//

KSAttrBtn::~KSAttrBtn()
 {
  delete apanel;
 }

//-----------------------------------------------------------//

void KSAttrBtn::setAttrPanel( KSAttrPanel *panel )
 {
  delete apanel;
  apanel = panel;
  if ( apanel ) {
        apanel->move( 5, 5 );
        apanel->show();
        connect( apanel, SIGNAL(acceptValue()), this, SLOT(acceptValue()) );
        }
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSFillBtn::KSFillBtn( QWidget *parent, const char *name)
:KSAttrBtn(parent,name)
 {
 }

//-----------------------------------------------------------//

KSFillBtn::~KSFillBtn()
 {
 }

//-----------------------------------------------------------//

void KSFillBtn::setDialogFill( const QSGFill &new_f )
 {
  f = new_f;
  emit attrChanged();
  update();
 }

//-----------------------------------------------------------//

void KSFillBtn::drawButtonLabel( QPainter *p )
 {
  QRect r = rect();
  r.setCoords( r.left()+BTN_HSPACE,
               r.top()+BTN_VSPACE,
               r.right()-BTN_HSPACE,
               r.bottom()-BTN_VSPACE );

  if ( f.style == QSGFill::Transparent ) {
        QFont fnt = font(); fnt.setPointSize( 10 );
        p->setFont( fnt );
        p->setPen( white );
        p->fillRect( r, black );
        p->drawText( 0,
                     0,
                     size().width(),
                     size().height(),
                     AlignHCenter |
                     AlignVCenter,
                     i18n("Transparent") );
      }  else {
        p->setPen( black );
        QSDrvQt drv; // we use only a canvas driver so NULL is allowed
        drv.setDC( p, 72, false );
        drv.startDrawing();
        drv.setFill( f );
        drv.drawRect( QSPt2f(r.x(),r.y()), QSPt2f(r.right(),r.bottom()) );
        drv.stopDrawing();
      }
 }

//-----------------------------------------------------------//

void KSFillBtn::showAttrPanel()
 {
  KSFillPanel *fpanel = new KSFillPanel( parentWidget()->parentWidget() );
  fpanel->setPanelFill( f );
  setAttrPanel( fpanel );
 }

//-----------------------------------------------------------//

void KSFillBtn::acceptValue()
 {
  KSFillPanel *fpanel = static_cast<KSFillPanel*>( (KSAttrPanel *)apanel );
  if ( fpanel && fpanel->result() ) setDialogFill( fpanel->panelFill() );
  delete apanel; apanel = NULL;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSFontBtn::KSFontBtn( QWidget *parent, const char *name)
:KSAttrBtn(parent,name)
 {
 }

//-----------------------------------------------------------//

KSFontBtn::~KSFontBtn()
 {
 }

//-----------------------------------------------------------//

void KSFontBtn::setDialogFont( const QSGFont &new_f )
 {
  f = new_f;
  emit attrChanged();
  update();
 }

//-----------------------------------------------------------//

void KSFontBtn::drawButtonLabel( QPainter * p )
 {
  QFont c = QSDrvQt::toQFont( f ); c.setPointSize( 10 );

  p->setPen( black );
  p->setFont( c );
  p->drawText( 0, 0,
               size().width(),
               size().height(),
               AlignHCenter |
               AlignVCenter,
               c.family() );
 }

//-----------------------------------------------------------//

void KSFontBtn::showAttrPanel()
 {
  KSFontPanel *fpanel = new KSFontPanel( parentWidget()->parentWidget() );
  fpanel->setPanelFont( f );
  setAttrPanel( fpanel );
 }

//-----------------------------------------------------------//

void KSFontBtn::acceptValue()
 {
  KSFontPanel *fpanel = static_cast<KSFontPanel*>( (KSAttrPanel *)apanel );
  if ( fpanel && fpanel->result() ) setDialogFont( fpanel->panelFont() );
  delete apanel; apanel = NULL;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSLineBtn::KSLineBtn( QWidget *parent, const char *name)
:KSAttrBtn(parent,name)
 {
 }

//-----------------------------------------------------------//

KSLineBtn::~KSLineBtn()
 {
 }

//-----------------------------------------------------------//

void KSLineBtn::setDialogLine( const QSGLine &new_l )
 {
  l = new_l;
  emit attrChanged();
  update();
 }

//-----------------------------------------------------------//

void KSLineBtn::drawButtonLabel( QPainter * p )
 {
  QRect r = rect();
  r.setCoords( r.left()+BTN_HSPACE,
               r.top()+BTN_VSPACE,
               r.right()-BTN_HSPACE,
               r.bottom()-BTN_VSPACE );

  p->setPen( black );
  p->setBrush( white );
  p->drawRect( r );

  p->setClipRect( r );
  if ( l.style != QSGLine::Invisible ) {

                p->setPen( QSDrvQt::toQPen(l) );
                p->drawLine( r.left()  + 4, r.center().y(),
                             r.right() - 4, r.center().y() );
               } else {

                QFont f = font();f.setPointSize( 10 );
                p->setFont( f );
                p->setPen( black );
                p->drawText( 0, 0,
                             size().width(),
                             size().height(),
                             AlignHCenter |
                             AlignVCenter,
                             i18n("Invisible") );
               }

  p->setClipping( false );
 }

//-----------------------------------------------------------//

void KSLineBtn::showAttrPanel()
 {
  KSLinePanel *lpanel = new KSLinePanel( parentWidget()->parentWidget() );
  lpanel->setPanelLine( l );
  setAttrPanel( lpanel );
 }

//-----------------------------------------------------------//

void KSLineBtn::acceptValue()
 {
  KSLinePanel *lpanel = static_cast<KSLinePanel*>( (KSAttrPanel *)apanel );
  if ( lpanel && lpanel->result() ) setDialogLine( lpanel->panelLine() );
  delete apanel; apanel = NULL;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSPointBtn::KSPointBtn(QWidget *parent, const char *name)
:KSAttrBtn(parent,name)
 {
 }

//-----------------------------------------------------------//

KSPointBtn::~KSPointBtn()
 {
 }

//-----------------------------------------------------------//

void KSPointBtn::setDialogPoint( const QSGPoint &new_p )
 {
  p = new_p;
  attrChanged();
  update();
 }

//-----------------------------------------------------------//

void KSPointBtn::drawButtonLabel( QPainter *paint )
 {
  QRect r = rect();

  r.setCoords( r.left()+BTN_HSPACE,
               r.top()+BTN_VSPACE,
               r.right()-BTN_HSPACE,
               r.bottom()-BTN_VSPACE );

  paint->setPen( black );
  paint->setBrush( white );
  paint->drawRect( r );

  paint->setClipRect( r );
  if ( p.style != QSGPoint::Invisible ) {
         QSPt2f mid( r.center().x(), r.center().y() );
         QSDrvQt drv; // we use only canvas driver, so NULL is allowed
         drv.setDC( paint, 72, false );
         drv.startDrawing();
         drv.drawPoint( mid, p );
         drv.stopDrawing();
        } else {
         QFont f = font();f.setPointSize( 10 );
         paint->setFont( f );
         paint->setPen( black );
         paint->drawText( 0, 0,
                          size().width(),
                          size().height(),
                          AlignHCenter |
                          AlignVCenter,
                          i18n("Invisible") );
        }
 }

//-----------------------------------------------------------//

void KSPointBtn::showAttrPanel()
 {
  KSPointPanel *ppanel = new KSPointPanel( parentWidget()->parentWidget() );
  ppanel->setPanelPoint( p );
  setAttrPanel( ppanel );
 }

//-----------------------------------------------------------//

void KSPointBtn::acceptValue()
 {
  KSPointPanel *ppanel = static_cast<KSPointPanel*>( (KSAttrPanel *)apanel );
  if ( ppanel && ppanel->result() ) setDialogPoint( ppanel->panelPoint() );
  delete apanel; apanel = NULL;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSArrowBtn::KSArrowBtn(QWidget *parent, bool init_end1, const char *name)
:KSAttrBtn(parent,name)
 {
  end1 = init_end1;
 }

//-----------------------------------------------------------//

KSArrowBtn::~KSArrowBtn()
 {
 }

//-----------------------------------------------------------//

void KSArrowBtn::setDialogArrow( const QSGArrow &new_a )
 {
  a = new_a;
  attrChanged();
  update();
 }

//-----------------------------------------------------------//

void KSArrowBtn::drawButtonLabel( QPainter *paint )
 {
  QRect r = rect();

  r.setCoords( r.left()+BTN_HSPACE,
               r.top()+BTN_VSPACE,
               r.right()-BTN_HSPACE,
               r.bottom()-BTN_VSPACE );

  paint->setPen( black );
  paint->setBrush( white );
  paint->drawRect( r );

  paint->setClipRect( r );
  if ( a.style != QSGArrow::None ) {
         QSPt2f mid( r.center().x(), r.center().y() );
         QSDrvQt drv;   // we use only a canvas driver so NULL is allowed
	 QSGLine l;
         drv.setDC( paint, 72, false );
         drv.startDrawing();
	 drv.setLine( l );
         drv.drawLineEnd( mid, end1?180.0:0.0, a );
         drv.stopDrawing();
        } else {
         QFont f = font();f.setPointSize( 10 );
         paint->setFont( f );
         paint->setPen( black );
         paint->drawText( r,
                          AlignHCenter |
                          AlignVCenter,
                          i18n("None") );
        }
 }


//-----------------------------------------------------------//

void KSArrowBtn::showAttrPanel()
 {
  KSArrowPanel *apanel = new KSArrowPanel( parentWidget()->parentWidget() );
  apanel->setPanelArrow( a );
  setAttrPanel( apanel );
 }

//-----------------------------------------------------------//

void KSArrowBtn::acceptValue()
 {
  KSArrowPanel *arpanel = static_cast<KSArrowPanel*>( (KSAttrPanel *)apanel );
  if ( arpanel && arpanel->result() ) setDialogArrow( arpanel->panelArrow() );
  delete apanel; apanel = NULL;
 }

