/***************************************************************************
                                 ksaxispanel.cpp                             
                             -------------------                                         
    begin                : 01-January-2000
    copyright            : (C) 2000 by Kamil Dobkowski                         
    email                : kamildobk@poczta.onet.pl                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "ksaxispanel.h"
#include "ksattrbtn.h"
#include <qvalidator.h>
#include <qtoolbutton.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <assert.h>
#include <kiconloader.h>
#include "../widgets/qsaxis.h"
#include "../part/kmatplotfactory.h"
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonAxis::KSPanelButtonAxis( QWidget *parent, KSWorkbook *workbook, QSAxis *axis )
:KSPanelButton( parent, workbook, BAxes )
 {
  m_axis = axis;

  QString icon_name;
  QString tool_tip;
  switch( m_axis->type() ) {
	case QSAxis::XAxisType: icon_name = "panel_x"; tool_tip = tr("X axis : ")+axis->title(); break;
	case QSAxis::YAxisType: icon_name = "panel_y"; tool_tip = tr("Y axis : ")+axis->title();  break;
	case QSAxis::ZAxisType: icon_name = "panel_z"; tool_tip = tr("Z axis : ")+axis->title();  break;
	case QSAxis::VAxisType: icon_name = "panel_v"; tool_tip = tr("V axis : ")+axis->title();  break;
	}
  setPixmap(BarIcon(icon_name,KMatplotFactory::global()));
  QToolTip::add( this, tool_tip );
  QWhatsThis::add( this, tool_tip );
 }

//-----------------------------------------------------------//

KSPanelButtonAxis::~KSPanelButtonAxis()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonAxis::createPanel( QWidget *panelParent )
 {
  return new KSAxisPanel( panelParent, m_workbook, m_axis );
 }

//-----------------------------------------------------------//

bool KSPanelButtonAxis::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::AxisCategory &&
       object->isAxesShadow() &&
       m_axis == object->parentAxes()->axis(element) ) return true;
  return false;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

#define KS_MAX_DOUBLE   1e300

KSAxisPanel::KSAxisPanel(QWidget *parent, KSWorkbook *workbook, QSAxis *axis, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  m_workbook = workbook;
  m_axis = axis;

  ascalel->insertItem(i18n("Linear"), 0 );
  ascalel->insertItem(i18n("Log"),    1 );

  alene->setValidator( new QDoubleValidator( -1000.0, 1000.0, 3, alene ) );
  amine->setValidator( new QDoubleValidator( -KS_MAX_DOUBLE, KS_MAX_DOUBLE, 16, amine ) );
  amaxe->setValidator( new QDoubleValidator( -KS_MAX_DOUBLE, KS_MAX_DOUBLE, 16, amaxe ) );
  abasee->setValidator( new QDoubleValidator( 0.0, KS_MAX_DOUBLE, 16, abasee ) );

  lfont = new KSFontBtn( this ); lfont->setGeometry( fontp->geometry() );
  m_arrow1 = new KSArrowBtn( this, true ); m_arrow1->setGeometry( arrow1Place->geometry() );
  m_arrow2 = new KSArrowBtn( this, false  ); m_arrow2->setGeometry( arrow2Place->geometry() );
  m_line = new KSLineBtn( this ); m_line->setGeometry( linePlace->geometry() );

  QWhatsThis::add( amine, i18n("Minimum value on the axis\n\n"
                          "If both Maximum and Minimum values are set to zero "
                          "the axis range automatically fits to data. ") );
  QWhatsThis::add( amaxe, i18n("Maximum value on the axis\n\n"
                          "If both Maximum and Minimum values are set to zero "
                          "the axis range automatically fits to data. ") );
  QWhatsThis::add( alene, i18n("Position of the axis\n\n") );
  QWhatsThis::add( ascalel, i18n("Axis scale\n\nTwo modes are available LINEAR and LOG. ") );
  QWhatsThis::add( abasee,  i18n("Base value\n\nWhen a scale kind is LINEAR "
                            "this value acts as a base step for auto-generated labels. "
                            "If a scale kind is LOG, the value is used as a base of "
                            "logarithm, but always must be > 1.0. ") );
  QWhatsThis::add( lfont, i18n("Title font\n\nFont used to print the axis title.") );
  QWhatsThis::add( labele, i18n("Axis title\n\nWrite a title string here. ") );

  watchProperties();
 }

//-----------------------------------------------------------//

KSAxisPanel::~KSAxisPanel()
 {
 }

//-----------------------------------------------------------//

void KSAxisPanel::readProperties()
 {
  QString s;

  amine->setText( s.setNum( m_axis->min(), 'g', 8 ) );
  amaxe->setText( s.setNum( m_axis->max(), 'g', 8 ) );
  alene->setText( s.setNum( m_axis->position(), 'g', 8 ) );
  abasee->setText( s.setNum( m_axis->scaleBase(), 'g', 8 ) );
  ascalel->setCurrentItem( m_axis->scaleType() == QSAxis::LinearScale ? 0 : 1 );
  labele->setText( m_axis->title() );
  lfont->setDialogFont( m_axis->font(QSAxis::TitleFont) );
  m_arrow1->setDialogArrow( m_axis->arrow1() );
  m_arrow2->setDialogArrow( m_axis->arrow2() );
  m_line->setDialogLine( m_axis->line(QSAxis::AxisLine) );
  areversed->setChecked( m_axis->reversed() );
  visible->setChecked( m_axis->visible() );
  scrollable->setChecked( m_axis->scrollable() );
  opposite->setChecked( m_axis->oppositePosition() );
  defaultPosition->setChecked( m_axis->defaultPosition() );
  titlePosPara->setText( QString::number( m_axis->titlePosition(), 'g', 3 ) );
  titlePosPerp->setText( QString::number( m_axis->titleDistance(), 'g', 3 ) );
 }

//-----------------------------------------------------------//

void KSAxisPanel::applyProperties()
 {
  KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties( m_axis );
  cmd->setProperty( "min", amine->text().toDouble() );
  cmd->setProperty( "max", amaxe->text().toDouble() );
  cmd->setProperty( "position", alene->text().toDouble() );
  cmd->setProperty( "scaleType", ascalel->currentItem()==1 ? QSAxis::LogScale : QSAxis::LinearScale );
  cmd->setProperty( "scaleBase", abasee->text().toDouble() );
  cmd->setProperty( "title", labele->text() );
  cmd->setFont( QSAxis::TitleFont, lfont->dialogFont() );
  cmd->setProperty( "reversed", areversed->isChecked() );
  cmd->setProperty( "arrow1", toQString(m_arrow1->dialogArrow()) );
  cmd->setProperty( "arrow2", toQString(m_arrow2->dialogArrow()) );
  cmd->setLine( QSAxis::AxisLine, m_line->dialogLine() );
  cmd->setProperty( "visible", visible->isChecked() );
  cmd->setProperty( "scrollable", scrollable->isChecked() );
  cmd->setProperty( "oppositePosition", opposite->isChecked() );
  cmd->setProperty( "defaultPosition", defaultPosition->isChecked() );
  cmd->setProperty( "titlePosition", titlePosPara->text().toDouble() );
  cmd->setProperty( "titleDistance", titlePosPerp->text().toDouble() );
  m_workbook->execute( cmd );
 }

/*
  m_axis->setRange( amine->text().toDouble(), amaxe->text().toDouble() );
  m_axis->setPosition( alene->text().toDouble() );
  m_axis->setScale( ascalel->currentItem()==1 ? QSAxis::LogScale : QSAxis::LinearScale,
                        abasee->text().toDouble() );
  m_axis->setTitle( labele->text() );
  m_axis->setFont( QSAxis::TitleFont, lfont->dialogFont() );
  m_axis->setReversed( areversed->isChecked() );
  m_axis->setArrow1( m_arrow1->dialogArrow() );
  m_axis->setArrow2( m_arrow2->dialogArrow() );
  m_axis->setLine( QSAxis::AxisLine, m_line->dialogLine() );
  m_axis->setVisible( visible->isChecked() );
  m_axis->setScrollable( scrollable->isChecked() );
  m_axis->setOppositePosition( opposite->isChecked() );
  m_axis->setDefaultPosition( defaultPosition->isChecked() );
*/













































