/***************************************************************************
                                  ksclabelpanel.cpp
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksclabelpanel.h"
#include "ksattrbtn.h"
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonCLabel::KSPanelButtonCLabel( QWidget *parent, KSWorkbook *workbook, QSCObject *o )
:KSPanelButton( parent, workbook, BObject, "panel_label", tr("Label") )
 {
  m_object = o;
 }

//-----------------------------------------------------------//

KSPanelButtonCLabel::~KSPanelButtonCLabel()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonCLabel::createPanel( QWidget *panelParent )
 {
  return new KSCLabelPanel( panelParent, m_workbook, m_object );
 }


//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//


KSCLabelPanel::KSCLabelPanel(QWidget *parent, KSWorkbook *workbook, QSCObject *init_o, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  m_workbook = workbook;
  btn_frame = new KSLineBtn( this ); btn_frame->setGeometry( framep->geometry() );
  btn_font  = new KSFontBtn( this ); btn_font->setGeometry( fontp->geometry() );
  btn_fill  = new KSFillBtn( this ); btn_fill->setGeometry( fillp->geometry() );
  btn_shadowfill = new KSFillBtn( this ); btn_shadowfill->setGeometry( shadowfillp->geometry() );

  o = init_o;
  watchProperties();
 }

//-------------------------------------------------------------//

KSCLabelPanel::~KSCLabelPanel()
 {
 }

//-------------------------------------------------------------//

void KSCLabelPanel::readProperties()
 {
  btn_frame->setDialogLine( toQSGLine(o->property("frame").toString()) );
  btn_font->setDialogFont( toQSGFont(o->property("font").toString()) );
  btn_fill->setDialogFill( toQSGFill(o->property("fill").toString()) );
  btn_shadowfill->setDialogFill( toQSGFill(o->property("shadowFill").toString()) );
  shadowdx->setValue( o->property("shadowPosX").toInt() );
  shadowdy->setValue( o->property("shadowPosY").toInt() );
  setPosAlign( o->property("textAlign").toInt() );
 }

//-------------------------------------------------------------//

void KSCLabelPanel::applyProperties()
 {
   KSCmdSetProperties *cmd = new KSCmdSetProperties(o);
   cmd->setProperty( "frame", toQString(btn_frame->dialogLine()) );
   cmd->setProperty( "font", toQString(btn_font->dialogFont()) );
   cmd->setProperty( "fill", toQString(btn_fill->dialogFill()) );
   cmd->setProperty( "shadowFill", toQString(btn_shadowfill->dialogFill()) );
   cmd->setProperty( "textAlign", posAlign() );
   cmd->setProperty( "shadowPosX", shadowdx->value() );
   cmd->setProperty( "shadowPosY", shadowdy->value() );
   m_workbook->execute( cmd );
 }

/*
  *line = btn_frame->dialogLine();
  *font = btn_font->dialogFont();
  *fill = btn_fill->dialogFill();
  *sfill = btn_shadowfill->dialogFill();
  *align = posAlign();
  spos->x = shadowdx->value();
  spos->y = shadowdy->value();
   o->changed();
*/
//-------------------------------------------------------------//

void KSCLabelPanel::setPosAlign( int a )
 {
  if ( a & AlignTop ) {
         if ( a & AlignLeft    ) _tl->setChecked( true );
         if ( a & AlignHCenter ) _tc->setChecked( true );
         if ( a & AlignRight   ) _tr->setChecked( true );
        }
  else
  if ( a & AlignVCenter ) {
         if ( a & AlignLeft    ) _cl->setChecked( true );
         if ( a & AlignHCenter ) _cc->setChecked( true );
         if ( a & AlignRight   ) _cr->setChecked( true );
  }
  else
  if ( a & AlignBottom ) {
         if ( a & AlignLeft    ) _bl->setChecked( true );
         if ( a & AlignHCenter ) _bc->setChecked( true );
         if ( a & AlignRight   ) _br->setChecked( true );
  }
 }

//-------------------------------------------------------------//

int KSCLabelPanel::posAlign()
 {
  int result = 0;

  if ( _tl->isChecked() ) result =  AlignTop | AlignLeft ;
  if ( _tc->isChecked() ) result =  AlignTop | AlignHCenter ;
  if ( _tr->isChecked() ) result =  AlignTop | AlignRight ;

  if ( _cl->isChecked() ) result =  AlignVCenter | AlignLeft ;
  if ( _cc->isChecked() ) result =  AlignVCenter | AlignHCenter ;
  if ( _cr->isChecked() ) result =  AlignVCenter | AlignRight ;

  if ( _bl->isChecked() ) result =  AlignBottom | AlignLeft ;
  if ( _bc->isChecked() ) result =  AlignBottom | AlignHCenter ;
  if ( _br->isChecked() ) result =  AlignBottom | AlignRight ;

  return result;
 }

