/***************************************************************************
                                 kscolorpanel.cpp
                             -------------------
    begin                : Fri Jan 7 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kscolorpanel.h"
#include <kcolordlg.h>

//-------------------------------------------------------------//

KSColorPanel::KSColorPanel(QWidget *parent, const char *name)
: QWidget(parent,name)
 {
  initDialog();
  new KSListBoxColor( clist, white, i18n("Custom") );
  new KSListBoxColor( clist, black, i18n("Black")  );

  new KSListBoxColor( clist, white, i18n("White") );
  new KSListBoxColor( clist, darkGray,  i18n("Dark Gray") );
  new KSListBoxColor( clist, gray,      i18n("Gray") );
  new KSListBoxColor( clist, lightGray, i18n("Light Gray") );
  new KSListBoxColor( clist, red,   i18n("Red") );
  new KSListBoxColor( clist, green, i18n("Green") );
  new KSListBoxColor( clist, blue,  i18n("Blue") );
  new KSListBoxColor( clist, cyan,  i18n("Cyan") );
  new KSListBoxColor( clist, magenta, i18n("Magenta") );
  new KSListBoxColor( clist, yellow, i18n("Yellow") );
  new KSListBoxColor( clist, darkRed, i18n("Dark Red") );
  new KSListBoxColor( clist, darkGreen, i18n("DarkGren") );
  new KSListBoxColor( clist, darkBlue, i18n("Dark Blue") );
  new KSListBoxColor( clist, darkCyan, i18n("Dark Cyan") );
  new KSListBoxColor( clist, darkMagenta, i18n("Dark Magenta") );
  new KSListBoxColor( clist, darkYellow, i18n("Dark Yellow") );

  c = black; clist->setCurrentItem(1);
  clist->setHScrollBarMode( QScrollView::AlwaysOff );
  a = 255; alphas->setValue( a );
  connect( customb, SIGNAL(clicked()), this, SLOT(showColorDialog()) );
  connect( clist, SIGNAL(highlighted(int)), this, SLOT(changePanelColor(int)) );
  connect( alphas, SIGNAL(valueChanged(int)), this, SLOT(changePanelAlpha(int)) );
 }

//-------------------------------------------------------------//

KSColorPanel::~KSColorPanel()
 {
 }

//-------------------------------------------------------------//

void KSColorPanel::setPanelColor( const QSGColor& new_color )
 {
  QColor color( new_color.r, new_color.g, new_color.b );

  if ( c != color ) {

         int i=1;

         for ( i=1; i<(int )clist->count(); i++ ) {
                KSListBoxColor *it = dynamic_cast<KSListBoxColor*>(clist->item(i));
                if ( it && it->c == color ) {
                         c = color; clist->setCurrentItem(i);
                         break;
                        }
               }

         // not ont the list
         if ( i == (int )clist->count() ) {
                dynamic_cast<KSListBoxColor*>(clist->item(0))->c = c = color;
                clist->setCurrentItem( 0 );
                clist->triggerUpdate(TRUE);
              }
        }

  unsigned char alpha = new_color.a;
  if ( alpha != a ) {
         a = alpha;
         alphas->setValue( a );
        }
 }

//-------------------------------------------------------------//

QSGColor KSColorPanel::panelColor() const
 {
  return QSGColor( c.red(), c.green(), c.blue(), a );
 }

//-------------------------------------------------------------//

void KSColorPanel::showColorDialog()
 {
  QColor color = c;
  if ( KColorDialog::getColor( color, this ) ) setPanelColor( QSGColor(color.red(),color.green(),color.blue(),a) );
 }

//-------------------------------------------------------------//

void KSColorPanel::changePanelColor( int )
 {
  KSListBoxColor *it = dynamic_cast<KSListBoxColor*>(clist->item(clist->currentItem()));
  if ( it ) c = it->c;
 }

//-------------------------------------------------------------//

void KSColorPanel::changePanelAlpha( int value )
 {
  if ( alphas->text().length() > 0 )  a = (unsigned char )value;
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

#define COLORBOX_SIZE_W  15
#define COLORBOX_SIZE_H  10
#define COLORBOX_SPACING_W  5
#define COLORBOX_SPACING_H  5

KSListBoxColor::KSListBoxColor( QListBox *parent, const QColor &color, const QString& name )
:QListBoxItem(parent), c(color)
 {
  setText( name );
 }

//-------------------------------------------------------------//

KSListBoxColor::~KSListBoxColor()
 {
 }

//-------------------------------------------------------------//

int KSListBoxColor::height( const QListBox *lb ) const
 {
  QFontMetrics fm = lb->fontMetrics();
  return QMAX( COLORBOX_SIZE_H+2*COLORBOX_SPACING_H, fm.lineSpacing()+1 );
 }

//-------------------------------------------------------------//

int KSListBoxColor::width( const QListBox *lb ) const
 {
  QFontMetrics fm = lb->fontMetrics();
  return fm.width(text()) + COLORBOX_SIZE_W + COLORBOX_SPACING_W*3;
 }

//-------------------------------------------------------------//

void KSListBoxColor::paint( QPainter *p )
 {
  QFontMetrics fm = p->fontMetrics();
  int w = fm.width(text()) + COLORBOX_SIZE_W + 3*COLORBOX_SPACING_W;
  int h = QMAX( COLORBOX_SIZE_H+2*COLORBOX_SPACING_H, fm.lineSpacing()+1 );

  QRect r( COLORBOX_SPACING_W,
           (h-COLORBOX_SIZE_H)/2,
           COLORBOX_SIZE_W,
           COLORBOX_SIZE_H );

  p->fillRect( r, c );
  p->drawText( COLORBOX_SIZE_W+2*COLORBOX_SPACING_W,
               0,
               w,
               h,
               Qt::AlignLeft | Qt::AlignVCenter,
               text() );
 }

