/***************************************************************************
                                  kscontourpanel.cpp
                             -------------------
    begin                : Mon Mar 19 2001
    copyright            : (C) 2001 by Kamil Dobkowski
    email                : kamildobk@poczta.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kscontourpanel.h"
#include "../widgets/qscontour.h"
#include "ksattrbtn.h"
#include <qwhatsthis.h>
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonContour::KSPanelButtonContour( QWidget *parent, KSWorkbook *workbook, QSContour *contour )
:KSPanelButton( parent, workbook, BDatasets, "contour", tr("Contour: ")+contour->title() )
 {
  m_contour = contour;
 }

//-----------------------------------------------------------//

KSPanelButtonContour::~KSPanelButtonContour()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonContour::createPanel( QWidget *panelParent )
 {
  return new KSContourPanel( panelParent, m_workbook, m_contour );
 }

//-----------------------------------------------------------//

bool KSPanelButtonContour::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::DatasetCategory &&
       object->isAxesShadow() &&
       object->parentAxes()->plot(element) == m_contour ) return true;
  return false;
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSContourPanel::KSContourPanel(QWidget *parent, KSWorkbook *workbook, QSContour *init_contour, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  contour = init_contour;
  m_workbook = workbook;
  gline = new KSLineBtn( this ); gline->setGeometry( glinew->geometry() );
  point = new KSPointBtn( this ); point->setGeometry( pointw->geometry() );
  m_x_select = new KSAxisSelect( xaxisc, contour->parentAxes(), QSAxis::XAxisType );
  m_y_select = new KSAxisSelect( yaxisc, contour->parentAxes(), QSAxis::YAxisType );
  m_v_select = new KSAxisSelect( vaxisc, contour->parentAxes(), QSAxis::VAxisType );
  QWhatsThis::add( titlee, i18n("Series title\n\nIt is displayed on the legend.") );
  watchProperties();
}

//-------------------------------------------------------------//

KSContourPanel::~KSContourPanel()
{
}

//-------------------------------------------------------------//

void KSContourPanel::readProperties()
{
 m_x_select->readProperties( contour->defaultAxis(QSAxis::XAxisType) );
 m_y_select->readProperties( contour->defaultAxis(QSAxis::YAxisType) );
 m_v_select->readProperties( contour->defaultAxis(QSAxis::VAxisType) );
 fillsc->setChecked( contour->contourFills() );
 linesc->setChecked( contour->contourLines() );
 labelsc->setChecked( contour->contourLabels() );
 legendc->setChecked( contour->legendItemVisible() );
 //gformate->setText( contour->contourGridLabelsFormat() );
 point->setDialogPoint( contour->point(QSContour::PointMark) );
 gline->setDialogLine( contour->line(QSContour::Grid) );
 titlee->setText( contour->title() );
}

//-------------------------------------------------------------//

void KSContourPanel::applyProperties()
{
 KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties(contour);

 cmd->setProperty( "title", titlee->text() );
 cmd->setProperty( "contourFills", fillsc->isChecked() );
 cmd->setProperty( "contourLines", linesc->isChecked() );
 cmd->setProperty( "contourLabels", labelsc->isChecked() );
 cmd->setProperty( "legendItemVisible", legendc->isChecked() );
 cmd->setLine( QSContour::Grid, gline->dialogLine() );
 cmd->setPoint( QSContour::PointMark, point->dialogPoint() );

 cmd->setProperty( "defaultXAxis", contour->parentAxes()->axisIndex(m_x_select->selectedAxis()) );
 cmd->setProperty( "defaultYAxis", contour->parentAxes()->axisIndex(m_y_select->selectedAxis()) );
 cmd->setProperty( "defaultVAxis", contour->parentAxes()->axisIndex(m_v_select->selectedAxis()) );

 m_workbook->execute( cmd );
}

//-------------------------------------------------------------//

/*
 contour->setDefaultAxis( QSAxis::XAxisType, m_x_select->selectedAxis() );
 contour->setDefaultAxis( QSAxis::YAxisType, m_y_select->selectedAxis() );
 contour->setDefaultAxis( QSAxis::VAxisType, m_v_select->selectedAxis() );
 contour->setContourFills( fillsc->isChecked() );
 contour->setContourLines( linesc->isChecked() );
 contour->setContourLabels( labelsc->isChecked() );
 contour->setLegendItemVisible( legendc->isChecked() );
 //contour->setContourGridLabelsFormat( gformate->text() );
 contour->setLine( QSContour::Grid, gline->dialogLine() );
 contour->setPoint( QSContour::PointMark, point->dialogPoint() );
*/
/*
int KSContourPanel::axis_number( QSAxes::AxisType t, int type_axis_number )
 {
  int number = 0;
  for ( int i=0; i<contour->parentAxes()->axesCount(); i++ )
  	 if ( contour->parentAxes()->axisType(i) == t )
  	 	 if ( number++ == type_axis_number ) return i;
  return -1;  	 	
 }

//-------------------------------------------------------------//

int KSContourPanel::type_axis_number( QSAxes::AxisType t, int axis_number )
 {
  int result = -1;
  for ( int i=0; i<=axis_number; i++ )
  	 if ( contour->parentAxes()->axisType(i) == t ) result++;
  return result;
 }

int xnr = 0;
  int ynr = 0;
  int vnr = 0;
  for( int i=0; i<contour->parentAxes()->axesCount(); i++ ) {
  	 QSAxes::AxisType type = contour->parentAxes()->axisType(i);
  	 if ( type == QSAxes::XAxisType ) xaxisc->insertItem( QString(tr("x"))+QString::number(++xnr) );
  	 if ( type == QSAxes::YAxisType ) yaxisc->insertItem( QString(tr("y"))+QString::number(++ynr) );
  	 if ( type == QSAxes::VAxisType ) vaxisc->insertItem( QString(tr("v"))+QString::number(++vnr) );
  	}

   QWhatsThis::add( yaxisc, i18n("Y axis\n\nAxis, which the current dataset is bound to. The first Y axis is "
                           "always placed at the left side of the plot and it is a default one. "
                           "The second Y axis is placed at the right and by default has hidden "
                           "labels") );
  QWhatsThis::add( xaxisc, i18n("X axis\n\nAxis, which the current dataset is bound to. The first X axis is "
                           "always placed at the bottom side and it is a default one. "
                           "The second X axis is placed at the top and by default has hidden "
                           "labels") );
  QWhatsThis::add( vaxisc, i18n("V axis\n\nAxis, which the current dataset is bound to.  "
				"The V-Axis plays a special role. Using it you can set a visible range of displayed values. Out of range values are"
				"clipped. All values below the minimum are displayed as black pixels and all values above maximum are"
				"displayed as white pixels. You can even set log scale or turn on an autoscale on the V-axis. But before"
				"changing the parameters of the V-axis remember to turn off 'Raw mode'") );

	int axis_number( QSAxes::AxisType t, int type_axis_number );
	int type_axis_number( QSAxes::AxisType t, int axis_number );
*/


