/***************************************************************************
                                  kscpospanel.cpp
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kscpospanel.h"
#include "../widgets/qsaxis.h"
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonCPos::KSPanelButtonCPos( QWidget *parent,
					KSWorkbook *workbook, QSCObject *o,
					const QCString& posX, const QCString& posY, const QCString& posZ,
					const QCString& coordX, const QCString& coordY, const QCString& coordZ )
:KSPanelButton( parent, workbook, BObject, "panel_pos1", tr("Position") )
 {
  m_object = o;
  m_pos_x = posX;
  m_pos_y = posY;
  m_pos_z = posZ;
  m_coord_x = coordX;
  m_coord_y = coordY;
  m_coord_z = coordZ;
 }

//-----------------------------------------------------------//

KSPanelButtonCPos::~KSPanelButtonCPos()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonCPos::createPanel( QWidget *panelParent )
 {
  return new KSCPosPanel( panelParent, m_workbook, m_object, m_pos_x, m_pos_y, m_pos_z, m_coord_x, m_coord_y, m_coord_z );
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSCPosPanel::KSCPosPanel(QWidget *parent, KSWorkbook *workbook, QSCObject *init_o,
			const QCString& posX, const QCString& posY, const QCString& posZ,
			const QCString& coordX, const QCString& coordY, const QCString& coordZ,
			const char *name )
: KSPanel(parent,name)
 {
  initDialog();
  o = init_o;
  m_workbook = workbook;
  m_pos_x = posX;
  m_pos_y = posY;
  m_pos_z = posZ;
  m_coord_x = coordX;
  m_coord_y = coordY;
  m_coord_z = coordZ;

  m_x_select = new KSAxisSelect( xAxis, o->parentAxes(), QSAxis::XAxisType );
  m_y_select = new KSAxisSelect( yAxis, o->parentAxes(), QSAxis::YAxisType );
  m_z_select = new KSAxisSelect( zAxis, o->parentAxes(), QSAxis::ZAxisType );

  watchProperties();
 }

//-------------------------------------------------------------//

KSCPosPanel::~KSCPosPanel()
 {
 }


//-------------------------------------------------------------//

void KSCPosPanel::readProperties() 
 {
  m_x_select->readProperties( o->defaultAxis(QSAxis::XAxisType) );
  m_y_select->readProperties( o->defaultAxis(QSAxis::YAxisType) );
  m_z_select->readProperties( o->defaultAxis(QSAxis::ZAxisType) );

  xaxisc->setCurrentItem( o->property(m_coord_x).toInt() );
  yaxisc->setCurrentItem( o->property(m_coord_y).toInt() );
  zaxisc->setCurrentItem( o->property(m_coord_z).toInt() );

  xpos->setText( QString::number(o->property(m_pos_x).toDouble(),'g',9) );
  ypos->setText( QString::number(o->property(m_pos_y).toDouble(),'g',9) );
  zpos->setText( QString::number(o->property(m_pos_z).toDouble(),'g',9) );
 }


/*
  xaxisc->setCurrentItem( *xaxis );
  yaxisc->setCurrentItem( *yaxis );
  zaxisc->setCurrentItem( *zaxis );

  xpos->setText( QString::number(pos->x,'g',9) );
  ypos->setText( QString::number(pos->y,'g',9) );
  zpos->setText( QString::number(pos->z,'g',9) );
*/

//-------------------------------------------------------------//

void KSCPosPanel::applyProperties() 
 {
  KSCmdSetProperties *cmd = new KSCmdSetProperties(o);

  if ( o->parentAxes() ) {
  	cmd->setProperty( "defaultXAxis", o->parentAxes()->axisIndex(m_x_select->selectedAxis()) );
  	cmd->setProperty( "defaultYAxis", o->parentAxes()->axisIndex(m_y_select->selectedAxis()) );
  	cmd->setProperty( "defaultZAxis", o->parentAxes()->axisIndex(m_z_select->selectedAxis()) );
	}

  int old_xCoord = o->property(m_coord_x).toInt();
  int old_yCoord = o->property(m_coord_y).toInt();
  int old_zCoord = o->property(m_coord_z).toInt();

  int new_xCoord = xaxisc->currentItem();
  int new_yCoord = yaxisc->currentItem();
  int new_zCoord = zaxisc->currentItem();

  if ( old_xCoord != new_xCoord ||
       old_yCoord != new_yCoord ||
       old_zCoord != new_zCoord  ) {
	double dpi = o->parentAxes() ? o->parentAxes()->run_dpi() : 72.0;
        QSPt3f p(xpos->text().toDouble(),
                 ypos->text().toDouble(),
                 zpos->text().toDouble());
	QSPt3f cp = o->mixedToCanvas( p, old_xCoord, old_yCoord, old_zCoord, dpi );	
        cmd->setProperty( m_coord_x, new_xCoord );
        cmd->setProperty( m_coord_y, new_yCoord );
        cmd->setProperty( m_coord_z, new_zCoord );
	p = o->canvasToMixed( cp, new_xCoord, new_yCoord, new_zCoord, dpi );	
        xpos->setText( QString::number(p.x,'g',9) );
        ypos->setText( QString::number(p.y,'g',9) );
        zpos->setText( QString::number(p.z,'g',9) );
       }

  cmd->setProperty( m_pos_x, xpos->text().toDouble() );
  cmd->setProperty( m_pos_y, ypos->text().toDouble() );
  cmd->setProperty( m_pos_z, zpos->text().toDouble() );

  m_workbook->execute( cmd );
 }

  /*
  o->setDefaultAxis( QSAxis::XAxisType, m_x_select->selectedAxis() );
  o->setDefaultAxis( QSAxis::YAxisType, m_y_select->selectedAxis() );
  o->setDefaultAxis( QSAxis::ZAxisType, m_z_select->selectedAxis() );

  int old_xCoord = o->property(m_coord_x).toInt();
  int old_yCoord = o->property(m_coord_y).toInt();
  int old_zCoord = o->property(m_coord_z).toInt();

  int new_xCoord = xaxisc->currentItem();
  int new_yCoord = yaxisc->currentItem();
  int new_zCoord = zaxisc->currentItem();

  if ( old_xCoord != new_xCoord ||
       old_yCoord != new_yCoord ||
       old_zCoord != new_zCoord  ) {

	double dpi = o->parentAxes() ? o->parentAxes()->run_dpi() : 72.0;
	
        QSPt3f p(xpos->text().toDouble(),
                 ypos->text().toDouble(),
                 zpos->text().toDouble());
	QSPt3f cp = o->mixedToCanvas( p, *xaxis, *yaxis, *zaxis, dpi );	
        *xaxis = new_xCoord;
        *yaxis = new_yCoord;
        *zaxis = new_zCoord;	
	p = o->canvasToMixed( cp, *xaxis, *yaxis, *zaxis, dpi );	
        xpos->setText( QString::number(p.x,'g',9) );
        ypos->setText( QString::number(p.y,'g',9) );
        zpos->setText( QString::number(p.z,'g',9) );
       }

  *pos = QSPt3f(xpos->text().toDouble(),
                ypos->text().toDouble(),
                zpos->text().toDouble());

  o->changed();
*/

//-------------------------------------------------------------//

