/***************************************************************************
                                 kscubepanel.cpp
                             -------------------
    begin                : Mon Jan 10 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kscubepanel.h"
#include "ksattrbtn.h"
#include <qvalidator.h>
#include <qstring.h>
#include <qwhatsthis.h>
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonCube::KSPanelButtonCube( QWidget *parent, KSWorkbook *workbook, QSAxes3D *axes )
:KSPanelButton( parent, workbook, BGeneral, "panel_box", tr("Cube") )
 {
  m_axes = axes;
 }

//-----------------------------------------------------------//

KSPanelButtonCube::~KSPanelButtonCube()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonCube::createPanel( QWidget *panelParent )
 {
  return new KSCubePanel( panelParent, m_workbook, m_axes );
 }

//-----------------------------------------------------------//

bool KSPanelButtonCube::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::GeneralCategory &&
       object->isAxesShadow() &&
       element == 0 ) return true;
  return false;
 }
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSCubePanel::KSCubePanel(QWidget *parent, KSWorkbook *workbook, QSAxes3D *plot, const char *name)
 : KSPanel(parent,name)
 {
  initDialog();

  assert( plot ); p3 = plot;
  m_workbook = workbook;

  xyFill = new KSFillBtn( this ); xyFill->setGeometry( xyFillPlace->geometry() );
  xzFill = new KSFillBtn( this ); xzFill->setGeometry( xzFillPlace->geometry() );
  yzFill = new KSFillBtn( this ); yzFill->setGeometry( yzFillPlace->geometry() );

  xyLine = new KSLineBtn( this ); xyLine->setGeometry( xyLinePlace->geometry() );
  xzLine = new KSLineBtn( this ); xzLine->setGeometry( xzLinePlace->geometry() );
  yzLine = new KSLineBtn( this ); yzLine->setGeometry( yzLinePlace->geometry() );

  xyThick->setValidator( new QDoubleValidator( 0.0, 1.0, 16, xyThick ) );
  xzThick->setValidator( new QDoubleValidator( 0.0, 1.0, 16, xzThick ) );
  yzThick->setValidator( new QDoubleValidator( 0.0, 1.0, 16, yzThick ) );

  //QWhatsThis::add( rcol,  i18n("Fill style\n\nFill style used to paint rear walls of the axis box"));
  //QWhatsThis::add( bcol,  i18n("Fill style\n\nFill styte used to paint the bottom wall of the axis box"));
  //QWhatsThis::add( rline, i18n("Line style\n\nLine style used to draw edges of the rear walls") );
  //QWhatsThis::add( bline, i18n("Line style\n\nLine style used to draw edges of the bottom wall" ));
  //QWhatsThis::add( rthicke, i18n("Rear wall thickness\n\nThis value is proportional to X and Y lengths.") );
  //QWhatsThis::add( bthicke, i18n("Bottom wall thickness\n\nThis value is proportional to a length of the Z axis.") );

  watchProperties();
 }

//-------------------------------------------------------------//

KSCubePanel::~KSCubePanel()
 {
 }

//-------------------------------------------------------------//

void KSCubePanel::readProperties()
 {
  QString s;

  xyThick->setText( s.setNum( p3->xyWallThickness(), 'g', 16 ) );
  xzThick->setText( s.setNum( p3->xzWallThickness(), 'g', 16 ) );
  yzThick->setText( s.setNum( p3->yzWallThickness(), 'g', 16 ) );

  xEdge->setText( s.setNum( p3->xEdgeLength(), 'g', 16 ) );
  yEdge->setText( s.setNum( p3->yEdgeLength(), 'g', 16 ) );
  zEdge->setText( s.setNum( p3->zEdgeLength(), 'g', 16 ) );

  xyFill->setDialogFill( p3->fill( QSAxes3D::XYWallFill ) );
  xzFill->setDialogFill( p3->fill( QSAxes3D::XZWallFill ) );
  yzFill->setDialogFill( p3->fill( QSAxes3D::YZWallFill ) );

  xyLine->setDialogLine( p3->line( QSAxes3D::XYWallLine ) );
  xzLine->setDialogLine( p3->line( QSAxes3D::XZWallLine ) );
  yzLine->setDialogLine( p3->line( QSAxes3D::YZWallLine ) );

 }

//-------------------------------------------------------------//

void KSCubePanel::applyProperties()
 {
  KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties(p3);
  cmd->setProperty( "xyWallThickness", xyThick->text().toDouble() );
  cmd->setProperty( "xzWallThickness", xzThick->text().toDouble() );
  cmd->setProperty( "yzWallThickness", yzThick->text().toDouble() );

  cmd->setProperty( "xEdgeLength", xEdge->text().toDouble() );
  cmd->setProperty( "yEdgeLength", yEdge->text().toDouble() );
  cmd->setProperty( "zEdgeLength", zEdge->text().toDouble() );

  cmd->setFill( QSAxes3D::XYWallFill, xyFill->dialogFill() );
  cmd->setFill( QSAxes3D::XZWallFill, xzFill->dialogFill() );
  cmd->setFill( QSAxes3D::YZWallFill, yzFill->dialogFill() );

  cmd->setLine( QSAxes3D::XYWallLine, xyLine->dialogLine() );
  cmd->setLine( QSAxes3D::XZWallLine, xzLine->dialogLine() );
  cmd->setLine( QSAxes3D::YZWallLine, yzLine->dialogLine() );

  m_workbook->execute( cmd );
 }	

/*
  p3->setWallThickness( xyThick->text().toDouble(),
			xzThick->text().toDouble(),
			yzThick->text().toDouble() );

  p3->setEdgeLength( xEdge->text().toDouble(),
		     yEdge->text().toDouble(),
		     zEdge->text().toDouble() );

  p3->setFill( QSAxes3D::XYWallFill, xyFill->dialogFill() );
  p3->setFill( QSAxes3D::XZWallFill, xzFill->dialogFill() );
  p3->setFill( QSAxes3D::YZWallFill, yzFill->dialogFill() );

  p3->setLine( QSAxes3D::XYWallLine, xyLine->dialogLine() );
  p3->setLine( QSAxes3D::XZWallLine, xzLine->dialogLine() );
  p3->setLine( QSAxes3D::YZWallLine, yzLine->dialogLine() );
*/

