/***************************************************************************
                                 ksfigurepanel.cpp
                             -------------------
    begin                : Tue Jan 11 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksfigurepanel.h"
#include "../widgets/qsfigure.h"
#include "../widgets/qsaxis.h"
#include "../part/kscommands.h"
#include <qwhatsthis.h>

///-----------------------------------------------------------//

KSPanelButtonFigure::KSPanelButtonFigure( QWidget *parent, KSWorkbook *workbook, QSFigure *figure )
:KSPanelButton( parent, workbook, BDatasets, "figure", tr("Figure")+figure->title() )
 {
  m_figure = figure;
 }

//-----------------------------------------------------------//

KSPanelButtonFigure::~KSPanelButtonFigure()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonFigure::createPanel( QWidget *panelParent )
 {
  return new KSFigurePanel( panelParent, m_workbook, m_figure );
 }

//-----------------------------------------------------------//

bool KSPanelButtonFigure::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::DatasetCategory &&
       object->isAxesShadow() &&
       object->parentAxes()->plot(element) == m_figure ) return true;
  return false;
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSFigurePanel::KSFigurePanel(QWidget *parent, KSWorkbook *workbook, QSFigure *init_figure, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  figure = init_figure;
  m_workbook = workbook;
  m_x_select = new KSAxisSelect( xaxisc, figure->parentAxes(), QSAxis::XAxisType );
  m_y_select = new KSAxisSelect( yaxisc, figure->parentAxes(), QSAxis::YAxisType );
  m_z_select = new KSAxisSelect( zaxisc, figure->parentAxes(), QSAxis::ZAxisType );
  m_v_select = new KSAxisSelect( vaxisc, figure->parentAxes(), QSAxis::VAxisType );
  QWhatsThis::add( accuracys, i18n("VCR - Vertex comparing accuracy\n\nThis value is used"
	                     " in shading calculations by drivers which use"
	                     " normals to vertices, such as OpenGL. A normal"
	                     " to a vertex is calculated as an average of all normals"
	                     " to polygons, which share this vertex. To find if two"
	                     " polygons share a vertex all vertices in those polygons"
	                     " are compared. To eliminate round-off errors, before comparing,"
	                     " all vertices in the plot are mapped to <0,1> box and next all"
	                     " coordinates are rounded to VCR places after the point.") );


 watchProperties();
}

//-------------------------------------------------------------//

KSFigurePanel::~KSFigurePanel()
 {
 }

//-------------------------------------------------------------//

void KSFigurePanel::readProperties()
 {
  m_x_select->readProperties( figure->defaultAxis(QSAxis::XAxisType)  );
  m_y_select->readProperties( figure->defaultAxis(QSAxis::YAxisType)  );
  m_z_select->readProperties( figure->defaultAxis(QSAxis::ZAxisType)  );
  m_v_select->readProperties( figure->defaultAxis(QSAxis::VAxisType)  );
  legendc->setChecked( figure->legendItemVisible() );
  accuracys->setValue( figure->vertexCompareAccuracy() );
  titlee->setText( figure->title() );
 }

//-------------------------------------------------------------//

void KSFigurePanel::applyProperties()
 {
  KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties(figure);

  cmd->setProperty( "legendItemVisible", legendc->isChecked() );
  cmd->setProperty( "vertexCompareAccuracy", accuracys->value() );
  cmd->setProperty( "title", titlee->text() );
  cmd->setProperty( "defaultXAxis", figure->parentAxes()->axisIndex(m_x_select->selectedAxis()) );
  cmd->setProperty( "defaultYAxis", figure->parentAxes()->axisIndex(m_y_select->selectedAxis()) );
  cmd->setProperty( "defaultZAxis", figure->parentAxes()->axisIndex(m_z_select->selectedAxis()) );
  cmd->setProperty( "defaultVAxis", figure->parentAxes()->axisIndex(m_v_select->selectedAxis()) );

  m_workbook->execute( cmd );
 }

/*
  figure->setDefaultAxis( QSAxis::XAxisType, m_x_select->selectedAxis() );
  figure->setDefaultAxis( QSAxis::YAxisType, m_y_select->selectedAxis() );
  figure->setDefaultAxis( QSAxis::ZAxisType, m_z_select->selectedAxis() );
  figure->setDefaultAxis( QSAxis::VAxisType, m_v_select->selectedAxis() );
  figure->setLegendItemVisible( legendc->isChecked() );
  figure->setVertexCompareAccuracy( accuracys->value() );
  figure->setTitle( titlee->text() );
*/

//-------------------------------------------------------------//

/*

  int xnr = 0;
  int ynr = 0;
  int znr = 0;
  for( int i=0; i<figure->parentAxes()->axesCount(); i++ ) {
  	 QSAxes::AxisType type = figure->parentAxes()->axisType(i);
  	 if ( type == QSAxes::XAxisType ) xaxisc->insertItem( QString(tr("x"))+QString::number(++xnr) );
  	 if ( type == QSAxes::YAxisType ) yaxisc->insertItem( QString(tr("y"))+QString::number(++ynr) );
  	 if ( type == QSAxes::ZAxisType ) zaxisc->insertItem( QString(tr("z"))+QString::number(++znr) );
  	}

  QWhatsThis::add( yaxisc, i18n("Y axis\n\nAxis, which the current dataset is bound to.") );
  QWhatsThis::add( xaxisc, i18n("X axis\n\nAxis, which the current dataset is bound to. "
                           "labels") );
  QWhatsThis::add( zaxisc, i18n("Z axis\n\nAxis, which the current dataset is bound to. ") );
  xaxisc->setCurrentItem( type_axis_number(QSAxes::XAxisType,figure->defaultAxis(QSAxes::XAxisType)) );
  yaxisc->setCurrentItem( type_axis_number(QSAxes::YAxisType,figure->defaultAxis(QSAxes::YAxisType)) );
  zaxisc->setCurrentItem( type_axis_number(QSAxes::ZAxisType,figure->defaultAxis(QSAxes::ZAxisType)) );

  figure->setDefaultAxis( QSAxes::XAxisType, axis_number(QSAxes::XAxisType,xaxisc->currentItem()) );
  figure->setDefaultAxis( QSAxes::YAxisType, axis_number(QSAxes::YAxisType,yaxisc->currentItem()) );
  figure->setDefaultAxis( QSAxes::ZAxisType, axis_number(QSAxes::ZAxisType,zaxisc->currentItem()) );

int KSFigurePanel::axis_number( QSAxes::AxisType t, int type_axis_number )
 {
  int number = 0;
  for ( int i=0; i<figure->parentAxes()->axesCount(); i++ )
  	 if ( figure->parentAxes()->axisType(i) == t )
  	 	 if ( number++ == type_axis_number ) return i;
  return -1;  	 	
 }

//-------------------------------------------------------------//

int KSFigurePanel::type_axis_number( QSAxes::AxisType t, int axis_number )
 {
  int result = -1;
  for ( int i=0; i<=axis_number; i++ )
  	 if ( figure->parentAxes()->axisType(i) == t ) result++;
  return result;
 }

	
	int axis_number( QSAxes::AxisType t, int type_axis_number );
	int type_axis_number( QSAxes::AxisType t, int axis_number );
*/

