/***************************************************************************
                                 ksfillpanel.cpp
                             -------------------
    begin                : Fri Jan 7 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksfillpanel.h"
#include "../widgets/qsdrvqt.h"

KSFillPanel::KSFillPanel(QWidget *parent, const char *name)
: KSAttrPanel(parent,name)
{
 initDialog();
 flist->clear();
 new KSListBoxFill( flist, QBrush( black, Qt::SolidPattern ), i18n("Solid") );
 new KSListBoxFill( flist, QBrush( white, Qt::NoBrush ),  i18n("Transparent") ) ;
 new KSListBoxFill( flist, QBrush( black, Qt::HorPattern ), i18n("Horizontal Lines") );
 new KSListBoxFill( flist, QBrush( black, Qt::VerPattern ), i18n("Vertical Lines") );
 new KSListBoxFill( flist, QBrush( black, Qt::CrossPattern ), i18n("Cross Lines") );
 new KSListBoxFill( flist, QBrush( black, Qt::BDiagPattern ), i18n("BDiag Lines") );
 new KSListBoxFill( flist, QBrush( black, Qt::FDiagPattern ), i18n("FDiag Lines") );
 new KSListBoxFill( flist, QBrush( black, Qt::DiagCrossPattern ), i18n("Diag Cross Lines") );
 new KSListBoxFill( flist, QBrush( black, Qt::Dense4Pattern ), i18n("Half Pattern") );

 b = QBrush( black, Qt::SolidPattern ); flist->setCurrentItem( 0 );

 flist->setHScrollBarMode( QScrollView::AlwaysOff );
 connect( flist, SIGNAL(highlighted(int)), this, SLOT(changePanelBrush(int)) );
}

//-----------------------------------------------------------//

KSFillPanel::~KSFillPanel()
 {
 }

//-----------------------------------------------------------//

void KSFillPanel::setPanelBrush( const QBrush& new_brush )
 {
  QBrush brush = new_brush; brush.setColor( black );

  if ( b.style() != brush.style() ) {

         int i;

         for ( i=0; i<(int )flist->count(); i++ ) {
                KSListBoxFill *it = dynamic_cast<KSListBoxFill*>(flist->item(i));
                if ( it && it->b.style() == brush.style() ) {
                         b = brush;
                         flist->setCurrentItem(i);
                         break;
                        }
               }

         // not on the list
         if ( i == (int )flist->count() ) {
                KSListBoxFill *it = dynamic_cast<KSListBoxFill*>(flist->item(0));
                if ( it ) b = it->b;
                flist->setCurrentItem(0);
                flist->triggerUpdate(TRUE);
               }

        }
 }

//-----------------------------------------------------------//

void KSFillPanel::changePanelBrush(int)
 {
  KSListBoxFill *it = dynamic_cast<KSListBoxFill*>(flist->item(flist->currentItem()));
  if ( it ) setPanelBrush( it->b );
 }

//-----------------------------------------------------------//

void KSFillPanel::setPanelFill( const QSGFill& fill )
 {
  setPanelBrush( QSDrvQt::toQBrush( fill ) );
  setPanelColor( fill.color );
 }

//-----------------------------------------------------------//

QSGFill KSFillPanel::panelFill() const
 {
  QSGFill result = QSDrvQt::toQSGFill( b );
  result.color = panelColor();
  return result;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

#define FILLBOX_SIZE_W  20
#define FILLBOX_SIZE_H  15
#define FILLBOX_KSACING_W  5
#define FILLBOX_KSACING_H  5

KSListBoxFill::KSListBoxFill( QListBox *parent, const QBrush &brush, const QString& name )
:QListBoxItem(parent), b(brush)
 {
  setText( name );

 }

//-------------------------------------------------------------//

KSListBoxFill::~KSListBoxFill()
 {
 }

//-------------------------------------------------------------//

int KSListBoxFill::height( const QListBox *lb ) const
 {
  return QMAX( FILLBOX_SIZE_H+2*FILLBOX_KSACING_H, lb->fontMetrics().lineSpacing()+1 );
 }

//-------------------------------------------------------------//

int KSListBoxFill::width( const QListBox *lb ) const
 {
  return lb->fontMetrics().width(text()) + FILLBOX_SIZE_W + FILLBOX_KSACING_W*3;
 }

//-------------------------------------------------------------//

void KSListBoxFill::paint( QPainter *p )
 {
  int w = p->fontMetrics().width(text()) + FILLBOX_SIZE_W + 3*FILLBOX_KSACING_W;
  int h = QMAX( FILLBOX_SIZE_H+2*FILLBOX_KSACING_H, p->fontMetrics().lineSpacing()+1 );

  p->setBrushOrigin( 0, listBox()->itemRect(this).y() );

  QRect r( FILLBOX_KSACING_W,
           (h-FILLBOX_SIZE_H)/2,
           FILLBOX_SIZE_W,
           FILLBOX_SIZE_H );

  QBrush br = QBrush( p->pen().color(), b.style() );
  p->fillRect( r, br );
  p->drawRect( r );
  p->drawText( FILLBOX_SIZE_W+2*FILLBOX_KSACING_W,
               0,
               w,
               h,
               Qt::AlignLeft | Qt::AlignVCenter,
               text() );
 }
