/***************************************************************************
                                 ksfontpanel.cpp
                             -------------------
    begin                : Fri Jan 7 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksfontpanel.h"
#include "../widgets/qsdrvqt.h"
#include <kfontdialog.h>

KSFontPanel::KSFontPanel(QWidget *parent, const char *name)
: KSAttrPanel(parent,name)
{
  initDialog();
  QFont df( "helvetica", 10 );
  new KSListBoxFont( flist, df, i18n("Custom Style ...") );
  new KSListBoxFont( flist, df, "Helvetica" );
  f = df; flist->setCurrentItem(1);
  df.setItalic( TRUE );
  new KSListBoxFont( flist, df, "Helvetica Italic" );
  df.setItalic( FALSE );
  df.setPointSize( 8 );
  new KSListBoxFont( flist, df, "Helvetica Small" );
  df.setBold( TRUE );
  df.setPointSize(12);
  new KSListBoxFont( flist, df, "Helvetica Large" );

  df.setFamily("Times");
  df.setPointSize(10);
  df.setBold( FALSE );
  new KSListBoxFont( flist, df, "Times" );
  df.setItalic( TRUE );
  new KSListBoxFont( flist, df, "Times Italic" );
  df.setItalic( FALSE );
  df.setPointSize( 8 );
  new KSListBoxFont( flist, df, "Times Small" );


  df.setFamily("Courier");
  df.setPointSize(10);
  df.setBold( FALSE );
  new KSListBoxFont( flist, df, "Courier" );

  flist->setHScrollBarMode( QScrollView::AlwaysOff );
  connect( customb, SIGNAL(clicked()), this, SLOT(showFontDialog()) );
  connect( flist, SIGNAL(highlighted(int)), this, SLOT(changePanelQFont(int)) );
}

//-------------------------------------------------------------//

KSFontPanel::~KSFontPanel()
 {
 }

//-------------------------------------------------------------//

void KSFontPanel::setPanelQFont( const QFont& font )
 {
   if ( f != font ) {

         int i;

         for ( i=1; i<(int )flist->count(); i++ ) {
                KSListBoxFont *it = dynamic_cast<KSListBoxFont*>(flist->item(i));
                if ( it && it->f == font ) {
                         f = font;
                         flist->setCurrentItem(i);
                         break;
                        }
               }

         // not on the list
         if ( i == (int )flist->count() ) {
                dynamic_cast<KSListBoxFont*>(flist->item(0))->f = f = font;
                flist->setCurrentItem(0);
                flist->triggerUpdate(TRUE);
               }

         f = font;
        }
 }

//-------------------------------------------------------------//

void KSFontPanel::showFontDialog()
 {
  QFont font = f;
	if ( KFontDialog::getFont(font) ) setPanelQFont( font );
 }

//-------------------------------------------------------------//

void KSFontPanel::changePanelQFont(int)	
 {
  KSListBoxFont *it = dynamic_cast<KSListBoxFont*>(flist->item(flist->currentItem()));
  if ( it ) setPanelQFont( it->f );
 }

//-----------------------------------------------------------//

void KSFontPanel::setPanelFont( const QSGFont &f )
 {
  setPanelQFont( QSDrvQt::toQFont(f) );
  setPanelColor( f.color );
 }

//-----------------------------------------------------------//

QSGFont KSFontPanel::panelFont() const
 {
  QSGFont result;
  result = QSDrvQt::toQSGFont( panelQFont(), black );
  result.color = panelColor();
  return result;
 }
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

#define FONTBOX_KSACING_W 5

KSListBoxFont::KSListBoxFont( QListBox *parent, const QFont &font, const QString& name )
:QListBoxItem(parent), f(font)
 {
  setText( name );
 }

//-------------------------------------------------------------//

KSListBoxFont::~KSListBoxFont()
 {
 }

//-------------------------------------------------------------//

int KSListBoxFont::height( const QListBox * ) const
 {
  return QFontMetrics(f).lineSpacing()+1;
 }

//-------------------------------------------------------------//

int KSListBoxFont::width( const QListBox * ) const
 {
  return QFontMetrics(f).width(text()) + FONTBOX_KSACING_W*2;
 }

//-------------------------------------------------------------//

void KSListBoxFont::paint( QPainter *paint )
 {

  int w = QFontMetrics(f).width(text()) + FONTBOX_KSACING_W*2;
  int h = QFontMetrics(f).lineSpacing()+1;
  paint->setFont( f );
  paint->drawText( FONTBOX_KSACING_W,
                   0,
                   w,
                   h,
                   Qt::AlignLeft | Qt::AlignVCenter,
                   text() );

 }

