/***************************************************************************
                                 ksglpanel.cpp
                             -------------------
    begin                : Mon Jan 10 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksglpanel.h"
#include "../part/kscommands.h"
#include <qwhatsthis.h>

//-----------------------------------------------------------//

KSPanelButtonGL::KSPanelButtonGL( QWidget *parent, KSWorkbook *workbook, QSAxes3D *axes )
:KSPanelButton( parent, workbook, BGeneral, "panel_gl", tr("OpenGL") )
 {
  m_axes = axes;
 }

//-----------------------------------------------------------//

KSPanelButtonGL::~KSPanelButtonGL()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonGL::createPanel( QWidget *panelParent )
 {
  return new KSGLPanel( panelParent, m_workbook, m_axes );
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSGLPanel::KSGLPanel(QWidget *parent, KSWorkbook *workbook, QSAxes3D *plot, const char *name)
: KSPanel(parent,name)
 {
  initDialog();

  p3 = plot;
  m_workbook = workbook;

  QWhatsThis::add( openglc, i18n("OpenGL\n\nIf checked, an OpenGL will be used to draw the plot"
                            " instead of a standard Qt driver. OpenGL supports "
                            " smooth shading, transparency and color interpolation") );
  QWhatsThis::add( alphac,  i18n("Alpha\n\nTurns on a transparency support in OpenGL") );
  QWhatsThis::add( globaltr, i18n("Global transparency\n\nThis transparency value will be added to each color.") );
  QWhatsThis::add( shadewallsc, i18n("Shading\n\nTurns on/off shading of the axis box. ") );
  QWhatsThis::add( shadewallsc, i18n("Auto color\n\n") );
  QWhatsThis::add( shadewallsc, i18n("Lightness\n\n") );

#ifndef HAVE_GL
  openglc->setEnabled( false );
  shadewallsc->setEnabled( false );
  alphac->setEnabled( false );
  autostrokec->setEnabled( false );
  globaltr->setEnabled( false );
  strokel->setEnabled( false );
  setEnabled( false );
#endif

  watchProperties();
 }

//-------------------------------------------------------------//

KSGLPanel::~KSGLPanel()
 {
 }

//-------------------------------------------------------------//

void KSGLPanel::readProperties()
 {
#ifdef HAVE_GL
  openglc->setChecked(p3->openGL());
  alphac->setChecked(p3->glTransparency());
  shadewallsc->setChecked(p3->glShadeWalls());
  autostrokec->setChecked(p3->glMeshAutoStroke());
  globaltr->setValue(p3->glGlobalTransparency());
  strokel->setValue(p3->glAutoStrokeLightness());
#endif
 }

//-------------------------------------------------------------//

void KSGLPanel::applyProperties()
 {
#ifdef HAVE_GL
  KSCmdSetProperties *cmd = new KSCmdSetProperties(p3);
  cmd->setProperty( "openGL", openglc->isChecked() );
  cmd->setProperty( "glTransparency", alphac->isChecked() );
  cmd->setProperty( "glShadeWalls", shadewallsc->isChecked() );
  cmd->setProperty( "glMeshAutoStroke", autostrokec->isChecked() );
  cmd->setProperty( "glGlobalTransparency", globaltr->value() );
  cmd->setProperty( "glAutoStrokeLightness", strokel->value() );
  m_workbook->execute( cmd );
#endif
 }

//-------------------------------------------------------------//

  /*
  p3->setOpenGL( openglc->isChecked() );
  p3->glSetTransparency( alphac->isChecked() );
  p3->glSetShadeWalls( shadewallsc->isChecked() );
  p3->glSetMeshAutoStroke( autostrokec->isChecked() );
  p3->glSetGlobalTransparency( globaltr->value() );
  p3->glSetAutoStrokeLightness( strokel->value() );
  */


