/***************************************************************************
                                 ksgradientganel.cpp
                             -------------------
    begin                : May 2001
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksgradientpanel.h"
#include "ksattrbtn.h"
#include "../widgets/qsimage.h"
#include "../widgets/qscontour.h"
#include "../widgets/qssurface.h"
#include "../widgets/qsfigure.h"
#include "../part/kscommands.h"
#include <qpainter.h>
#include <qwhatsthis.h>

//-----------------------------------------------------------//

KSPanelButtonGradient::KSPanelButtonGradient( QWidget *parent, KSWorkbook *workbook, QSPlot *plot )
:KSPanelButton( parent, workbook, BDatasets, "panel_gradient", tr("Gradient: ")+plot->title() )
 {
  m_plot = plot;
 }

//-----------------------------------------------------------//

KSPanelButtonGradient::~KSPanelButtonGradient()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonGradient::createPanel( QWidget *panelParent )
 {
  return new KSGradientPanel( panelParent, m_workbook, m_plot );
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSGradientPanel::KSGradientPanel(QWidget *parent, KSWorkbook *workbook, QSPlot *init_plot, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  m_plot = init_plot;
  m_workbook = workbook;
 }


//-------------------------------------------------------------//

KSGradientPanel::~KSGradientPanel()
 {
 }

//-------------------------------------------------------------//

void KSGradientPanel::readProperties()
 {
  QSGGradient g = m_plot->gradient();

 	type->setCurrentItem( g.type() );
        gradient_type_changed(g.type());

  	col1->setDialogFill( g.fill(0) );
  	col2->setDialogFill( g.fill(1) );
  	col3->setDialogFill( g.fill(2) );
  	col4->setDialogFill( g.fill(3) );
  	col5->setDialogFill( g.fill(4) );
	
  preview();
 }

//-------------------------------------------------------------//

void KSGradientPanel::applyProperties()
 {
  KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties( m_plot );
  cmd->setProperty( "gradient", toQString( QSGGradient( col1->dialogFill(),
					      col2->dialogFill(),
					      col3->dialogFill(),
					      col4->dialogFill(),
					      col5->dialogFill(),
					      (QSGGradient::Type )type->currentItem() )) );
  m_workbook->execute( cmd );
  preview();
 }


//-------------------------------------------------------------//

bool KSGradientPanel::eventFilter( QObject *o, QEvent *e )
 {
  QSGFill fill;

  if ( o == gradientw ) {
  if (  e->type() == QEvent::Paint ) {
	QPainter p( gradientw );
        QSGGradient g = m_plot->gradient();
        int w = gradientw->size().width();
        int h = gradientw->size().height();
        for ( int i=0; i<h; i++ ) {
                g.fill( double(h-i) / h, fill );
                p.setPen( QColor( fill.color.r, fill.color.g, fill.color.b ) );
                p.drawLine( 0, i, w-1, i );
               }
         return TRUE;
        }
     }

  return KSPanel::eventFilter(o,e);
 }

//-------------------------------------------------------------//

void KSGradientPanel::preview()
 {
  gradientw->update();
 }

//-------------------------------------------------------------//

void  KSGradientPanel::initDialog()
 {
  this->resize(130,300);
  this->setMinimumSize(0,0);

  type = new QComboBox(this,"GradientType");
  type->setGeometry(5,150,120,20);
  type->insertItem(tr("Default"));
  type->insertItem(tr("Gray"));
  type->insertItem(tr("Strange"));
  type->insertItem(tr("Two colors"));
  type->insertItem(tr("Five colors"));

  col5w= new QWidget(this,"Mesh5Place");
  col5w->setGeometry(5,180,55,20);
  col5w->setMinimumSize(0,0);
  col4w= new QWidget(this,"Mesh4Place");
  col4w->setGeometry(5,200,55,20);
  col4w->setMinimumSize(0,0);
  col3w= new QWidget(this,"Mesh3Place");
  col3w->setGeometry(5,220,55,20);
  col3w->setMinimumSize(0,0);
  col2w= new QWidget(this,"Mesh2Place");
  col2w->setGeometry(5,240,55,20);
  col2w->setMinimumSize(0,0);
  col1w= new QWidget(this,"Mesh1Place");
  col1w->setGeometry(5,260,55,20);
  col1w->setMinimumSize(0,0);

  gradientw= new QWidget(this,"GradientWidget");
  gradientw->setGeometry(65,185,60,85);
  gradientw->setMinimumSize(0,0);

  col1 = new KSFillBtn( this ); col1->setGeometry( col1w->geometry() );
  col2 = new KSFillBtn( this ); col2->setGeometry( col2w->geometry() );
  col3 = new KSFillBtn( this ); col3->setGeometry( col3w->geometry() );
  col4 = new KSFillBtn( this ); col4->setGeometry( col4w->geometry() );
  col5 = new KSFillBtn( this ); col5->setGeometry( col5w->geometry() );

  connect( type, SIGNAL(activated(int)), this, SLOT(gradient_type_changed(int)) );
  //connect( col1, SIGNAL(dialogFillChanged()), this, SLOT(preview()) );
  //connect( col2, SIGNAL(dialogFillChanged()), this, SLOT(preview()) );
  //connect( col3, SIGNAL(dialogFillChanged()), this, SLOT(preview()) );
  //connect( col4, SIGNAL(dialogFillChanged()), this, SLOT(preview()) );
  //connect( col5, SIGNAL(dialogFillChanged()), this, SLOT(preview()) );

  gradientw->setBackgroundMode( NoBackground );
  gradientw->installEventFilter( this );

  QWhatsThis::add( col1,   i18n("Default color for points with lowest Z values.") );
  QWhatsThis::add( col2,   i18n("Default color for points with low Z values.") );
  QWhatsThis::add( col3,   i18n("Default color for points with middle Z values.") );
  QWhatsThis::add( col4,   i18n("Default color for points with great Z values.") );
  QWhatsThis::add( col5,   i18n("Default color for points with greates Z values.") );
  QWhatsThis::add( gradientw, i18n("Gradient preview") );

  watchProperties();
}

//-------------------------------------------------------------//

void KSGradientPanel::gradient_type_changed(int )
 {
  QSGGradient g = QSGGradient( (QSGGradient::Type )type->currentItem() );
  col1->show();
  col2->show();
  col3->show();
  col4->show();
  col5->show();

  	col1->setDialogFill( g.fill(0) );
  	col2->setDialogFill( g.fill(1) );
  	col3->setDialogFill( g.fill(2) );
  	col4->setDialogFill( g.fill(3) );
  	col5->setDialogFill( g.fill(4) );

  if ( type->currentItem() == QSGGradient::GTwoColors ) {
	col2->hide();
	col3->hide();
	col4->hide();
	}
  else
  if ( type->currentItem() == QSGGradient::GFiveColors ) {
	}
  else {
        col1->hide();
	col2->hide();
	col3->hide();
	col4->hide();
	col5->hide();
       }

  preview();
 }
