/***************************************************************************
                                 ksimagepanel.cpp
                             -------------------
    begin                : Tue Jan 11 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksimagepanel.h"
#include "../widgets/qsimage.h"
#include "../widgets/qsaxis.h"
#include "../part/kscommands.h"
#include <qwhatsthis.h>

//-----------------------------------------------------------//

KSPanelButtonImage::KSPanelButtonImage( QWidget *parent, KSWorkbook *workbook, QSImage *image )
:KSPanelButton( parent, workbook, BDatasets, "pixmap", tr("Image: ")+image->title() )
 {
  m_image = image;
 }

//-----------------------------------------------------------//

KSPanelButtonImage::~KSPanelButtonImage()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonImage::createPanel( QWidget *panelParent )
 {
  return new KSImagePanel( panelParent, m_workbook, m_image );
 }

//-----------------------------------------------------------//

bool KSPanelButtonImage::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::DatasetCategory &&
       object->isAxesShadow() &&
       object->parentAxes()->plot(element) == m_image ) return true;
  return false;
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSImagePanel::KSImagePanel(QWidget *parent, KSWorkbook *workbook, QSImage *init_image, const char *name)
: KSPanel(parent,name)
 {
  initDialog();

  m_workbook = workbook;
  gradientc= new QCheckBox(this,"GradientCheck");
  gradientc->setGeometry(15,245,100,25);
  gradientc->setMinimumSize(0,0);
  gradientc->setText(i18n("Gradient"));

  image = init_image;
  m_x_select = new KSAxisSelect( xaxisc, image->parentAxes(), QSAxis::XAxisType );
  m_y_select = new KSAxisSelect( yaxisc, image->parentAxes(), QSAxis::YAxisType );
  m_v_select = new KSAxisSelect( vaxisc, image->parentAxes(), QSAxis::VAxisType );


  QWhatsThis::add( titlee, i18n("Series title\n\nIt is displayed on the legend.") );
  QWhatsThis::add( rawc, i18n("Raw mode\n\nWhen raw mode is on, the V axis has set fixed range <0,255> and"
                        " linear scale. It requires much less calculations, so repaints"
                        " are much faster.") );
 watchProperties();
}

//-------------------------------------------------------------//

KSImagePanel::~KSImagePanel()
{
}

//-------------------------------------------------------------//

void KSImagePanel::readProperties()
{
 m_x_select->readProperties( image->defaultAxis(QSAxis::XAxisType)  );
 m_y_select->readProperties( image->defaultAxis(QSAxis::YAxisType)  );
 m_v_select->readProperties( image->defaultAxis(QSAxis::VAxisType)  );
 legendc->setChecked( image->legendItemVisible() );
 gradientc->setChecked( image->useGradient() );
 rawc->setChecked( image->rawMode() );
 titlee->setText( image->title() );
}

//-------------------------------------------------------------//

void KSImagePanel::applyProperties()
{
 KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties(image);
 cmd->setProperty( "defaultXAxis", image->parentAxes()->axisIndex(m_x_select->selectedAxis()) );
 cmd->setProperty( "defaultYAxis", image->parentAxes()->axisIndex(m_y_select->selectedAxis()) );
 cmd->setProperty( "defaultVAxis", image->parentAxes()->axisIndex(m_v_select->selectedAxis()) );
 cmd->setProperty( "legendItemVisible", legendc->isChecked() );
 cmd->setProperty( "rawMode", rawc->isChecked() );
 cmd->setProperty( "useGradient", gradientc->isChecked() );
 cmd->setProperty( "title", titlee->text() );
 m_workbook->execute( cmd );
}
/*
 image->setDefaultAxis( QSAxis::XAxisType, m_x_select->selectedAxis() );
 image->setDefaultAxis( QSAxis::YAxisType, m_y_select->selectedAxis() );
 image->setDefaultAxis( QSAxis::VAxisType, m_v_select->selectedAxis() );
 image->setLegendItemVisible( legendc->isChecked() );
 image->setRawMode( rawc->isChecked() );
 image->setUseGradient( gradientc->isChecked() );
 image->setTitle( titlee->text() );
*/
//-------------------------------------------------------------//

/*
 image->setDefaultAxis( QSAxes::XAxisType, axis_number(QSAxes::XAxisType,xaxisc->currentItem()) );
 image->setDefaultAxis( QSAxes::YAxisType, axis_number(QSAxes::YAxisType,yaxisc->currentItem()) );
 image->setDefaultAxis( QSAxes::VAxisType, axis_number(QSAxes::VAxisType,vaxisc->currentItem()) );
  xaxisc->setCurrentItem( type_axis_number(QSAxes::XAxisType,image->defaultAxis(QSAxes::XAxisType)) );
 yaxisc->setCurrentItem( type_axis_number(QSAxes::YAxisType,image->defaultAxis(QSAxes::YAxisType)) );
 vaxisc->setCurrentItem( type_axis_number(QSAxes::VAxisType,image->defaultAxis(QSAxes::VAxisType)) );
  int xnr = 0;
  int ynr = 0;
  int vnr = 0;
  for( int i=0; i<image->parentAxes()->axesCount(); i++ ) {
  	 QSAxes::AxisType type = image->parentAxes()->axisType(i);
  	 if ( type == QSAxes::XAxisType ) xaxisc->insertItem( QString(tr("x"))+QString::number(++xnr) );
  	 if ( type == QSAxes::YAxisType ) yaxisc->insertItem( QString(tr("y"))+QString::number(++ynr) );
  	 if ( type == QSAxes::VAxisType ) vaxisc->insertItem( QString(tr("v"))+QString::number(++vnr) );
  	}

 QWhatsThis::add( yaxisc, i18n("Y axis\n\nAxis, which the current dataset is bound to. The first Y axis is "
                           "always placed at the left side of the plot and it is a default one. "
                           "The second Y axis is placed at the right and by default has hidden "
                           "labels") );
  QWhatsThis::add( xaxisc, i18n("X axis\n\nAxis, which the current dataset is bound to. The first X axis is "
                           "always placed at the bottom side and it is a default one. "
                           "The second X axis is placed at the top and by default has hidden "
                           "labels") );
  QWhatsThis::add( vaxisc, i18n("V axis\n\nAxis, which the current dataset is bound to.  "
				"The V-Axis plays a special role. Using it you can set a visible range of displayed values. Out of range values are"
				"clipped. All values below the minimum are displayed as black pixels and all values above maximum are"
				"displayed as white pixels. You can even set log scale or turn on an autoscale on the V-axis. But before"
				"changing the parameters of the V-axis remember to turn off 'Raw mode'") );


int KSImagePanel::axis_number( QSAxes::AxisType t, int type_axis_number )
 {
  int number = 0;
  for ( int i=0; i<image->parentAxes()->axesCount(); i++ )
  	 if ( image->parentAxes()->axisType(i) == t )
  	 	 if ( number++ == type_axis_number ) return i;
  return -1;  	 	
 }

//-------------------------------------------------------------//

int KSImagePanel::type_axis_number( QSAxes::AxisType t, int axis_number )
 {
  int result = -1;
  for ( int i=0; i<=axis_number; i++ )
  	 if ( image->parentAxes()->axisType(i) == t ) result++;
  return result;
 }


	int axis_number( QSAxes::AxisType t, int type_axis_number );
	int type_axis_number( QSAxes::AxisType t, int axis_number );
*/

