/***************************************************************************
                                 kslightpanel.cpp
                             -------------------
    begin                : Mon Jan 10 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kslightpanel.h"
#include "../widgets/qsaxes3d.h"
#include "../part/kscommands.h"
#include <qwhatsthis.h>

//-----------------------------------------------------------//

KSPanelButtonLight::KSPanelButtonLight( QWidget *parent, KSWorkbook *workbook, QSAxes3D *axes )
:KSPanelButton( parent, workbook, BGeneral, "panel_light", tr("Light") )
 {
  m_axes = axes;
 }

//-----------------------------------------------------------//

KSPanelButtonLight::~KSPanelButtonLight()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonLight::createPanel( QWidget *panelParent )
 {
  return new KSLightPanel( panelParent, m_workbook, m_axes );
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSLightPanel::KSLightPanel(QWidget *parent, KSWorkbook *workbook, QSAxes3D *plot, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  assert( plot ); p3 = plot;
  m_workbook = workbook;
  QWhatsThis::add( lightc, i18n("Light\n\nTurns on lighting.") );
  QWhatsThis::add( lightas, i18n("Light azimuth\n\nPosition of the light source in spherical coordinates. Range is <0,359> degree. ") );
  QWhatsThis::add( lightes, i18n("Light elevation\n\nPosition of the light source in spherical coordinates. Range is <-90,90> degree.") );
  QWhatsThis::add( lightis, i18n("Light intensity\n\nIntensity of the directed light source. This parameter changes from "
                             "-50 (0%) to 50 (100%).") );
  QWhatsThis::add( lightams, i18n("Abmient light intensity\n\nIntensity of the ambient light source or simple - lightness "
                              "This parameter changes from -50 (0%) to 50 (100%).") );

  watchProperties();
 }

//-----------------------------------------------------------//

KSLightPanel::~KSLightPanel()
 {
 }

//-----------------------------------------------------------//
			
void KSLightPanel::readProperties()
 {
  lightas->setValue( p3->lightAzimuth() );
  lightes->setValue( p3->lightElevation() );
  lightis->setValue( p3->directLight() );
  lightams->setValue( p3->ambientLight() );
  lightc->setChecked( p3->light() );
 }

//-----------------------------------------------------------//

void KSLightPanel::applyProperties()
 {
  KSCmdSetProperties *cmd = new KSCmdSetProperties(p3);
  cmd->setProperty( "lightAzimuth", lightas->value() );
  cmd->setProperty( "lightElevation", lightes->value() );
  cmd->setProperty( "directLight", lightis->value() );
  cmd->setProperty( "ambientLight", lightams->value() );
  cmd->setProperty( "light", lightc->isChecked() );
  m_workbook->execute( cmd );
 }

/*
  p3->setLightAzimuth( lightas->value() );
  p3->setLightElevation( lightes->value() );
  p3->setDirectLight( lightis->value() );
  p3->setAmbientLight( lightams->value() );
  p3->setLight( lightc->isChecked() );
*/

//-----------------------------------------------------------//



