/***************************************************************************
                                 kslinepanel.cpp
                             -------------------
    begin                : Fri Jan 7 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kslinepanel.h"
#include "../widgets/qsdrvqt.h"
#include "../widgets/qsgattr.h"

//-------------------------------------------------------------//

KSLinePanel::KSLinePanel(QWidget *parent, const char *name)
: KSAttrPanel(parent,name)
 {
  initDialog();
  w = 0;
  QSGLine l;

  w = 0;

  l.style = QSGLine::Invisible; new KSListBoxLine( llist, QSDrvQt::toQPen(l), i18n("Invisible") );
  p = QSDrvQt::toQPen(l);
  l.style = QSGLine::Solid;      new KSListBoxLine( llist, QSDrvQt::toQPen(l), i18n("Solid") );
  l.style = QSGLine::Dash;       new KSListBoxLine( llist, QSDrvQt::toQPen(l), i18n("Dashed") );
  l.style = QSGLine::DashDot;    new KSListBoxLine( llist, QSDrvQt::toQPen(l), i18n("Dashed-Dotted") );
  l.style = QSGLine::Dot;        new KSListBoxLine( llist, QSDrvQt::toQPen(l), i18n("Dotted") );
  l.style = QSGLine::DashDotDot; new KSListBoxLine( llist, QSDrvQt::toQPen(l), i18n("Dashed-2Dotted") );
  llist->setHScrollBarMode( QScrollView::AlwaysOff );
  llist->setCurrentItem(0);
  connect( llist, SIGNAL(highlighted(int)), this, SLOT(changePanelQPen(int)) );
  connect( widths, SIGNAL(valueChanged(int)), this, SLOT(changePanelLineWidth(int)) );
 }

//-------------------------------------------------------------//

KSLinePanel::~KSLinePanel()
 {
 }

//-------------------------------------------------------------//

void KSLinePanel::setPanelQPen( const QPen& new_pen )
 {
  QPen pen = new_pen;
  pen.setColor( black );
  pen.setWidth( 0 );

  if ( p != pen ) {
         int i;
         for ( i=0; i<(int )llist->count(); i++ ) {
                KSListBoxLine *it = dynamic_cast<KSListBoxLine*>(llist->item(i));
                if ( it && it->p == pen ) {
                         p = pen; llist->setCurrentItem(i);
                         break;
                        }
               }

         if ( i == (int )llist->count() ) {
                KSListBoxLine *it = dynamic_cast<KSListBoxLine*>(llist->item(0));
                if ( it ) p = it->p;
                llist->setCurrentItem(0);
                llist->triggerUpdate(TRUE);
               }
        }
 }

//-------------------------------------------------------------//

void KSLinePanel::setPanelLineWidth( int width )
 {
  if ( w != width ) {
         w = width;
         widths->setValue(w);
        }
 }

//-------------------------------------------------------------//

void KSLinePanel::changePanelQPen(int)	
 {
  KSListBoxLine *it = dynamic_cast<KSListBoxLine*>(llist->item(llist->currentItem()));
  if ( it ) setPanelQPen( it->p );
 }

//-------------------------------------------------------------//

void KSLinePanel::changePanelLineWidth( int value )	
 {
  setPanelLineWidth( value );
 }

//-----------------------------------------------------------//

void KSLinePanel::setPanelLine( const QSGLine &l )
 {
  QPen p = QSDrvQt::toQPen( l );
  setPanelQPen( p );
  setPanelLineWidth( l.width );
  setPanelColor( l.color );
 }

//-----------------------------------------------------------//

QSGLine KSLinePanel::panelLine() const
 {
  QSGLine result;
  result = QSDrvQt::toQSGLine( panelQPen() );
  result.color = panelColor();
  result.width = panelLineWidth();
  return result;
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

#define LINEBOX_SIZE_W    40
#define LINEBOX_SIZE_H    15
#define LINEBOX_KSACING_W  5
#define LINEBOX_KSACING_H  5

KSListBoxLine::KSListBoxLine( QListBox *parent, const QPen &pen, const QString& name )
:QListBoxItem(parent), p(pen)
 {
  setText( name );
 }

//-------------------------------------------------------------//

KSListBoxLine::~KSListBoxLine()
 {
 }

//-------------------------------------------------------------//

int KSListBoxLine::height( const QListBox *lb ) const
 {
  return QMAX( LINEBOX_SIZE_H+2*LINEBOX_KSACING_H, lb->fontMetrics().lineSpacing()+1 );
 }

//-------------------------------------------------------------//

int KSListBoxLine::width( const QListBox *lb ) const
 {
  return lb->fontMetrics().width(text()) + LINEBOX_SIZE_W + LINEBOX_KSACING_W*3;
 }

//-------------------------------------------------------------//

void KSListBoxLine::paint( QPainter *paint )
 {
  int w = paint->fontMetrics().width(text()) + LINEBOX_SIZE_W + 3*LINEBOX_KSACING_W;
  int h = QMAX( LINEBOX_SIZE_H+2*LINEBOX_KSACING_H, paint->fontMetrics().lineSpacing()+1 );

  QRect r( LINEBOX_KSACING_W,
           (h-LINEBOX_SIZE_H)/2,
           LINEBOX_SIZE_W,
           LINEBOX_SIZE_H );

  QPen pen = p;
  if ( listBox() && selected() ) pen.setColor( listBox()->colorGroup().highlightedText() );
  paint->drawText( LINEBOX_SIZE_W+2*LINEBOX_KSACING_W,
                   0,
                   w,
                   h,
                   Qt::AlignLeft | Qt::AlignVCenter,
                   text() );
  paint->setPen( pen );
  paint->drawLine( r.left(), r.top()+(r.bottom()-r.top())/2, r.right(), r.top()+(r.bottom()-r.top())/2 );
 }


