/***************************************************************************
                                 ksmeshpanel.cpp
                             -------------------
    begin                : Mon Jan 10 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksmeshpanel.h"
#include "ksattrbtn.h"
#include <qpainter.h>
#include <qwhatsthis.h>
#include <assert.h>
#include "../widgets/qsplot.h"
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonMesh::KSPanelButtonMesh( QWidget *parent, KSWorkbook *workbook, QSPlot3D *plot )
:KSPanelButton( parent, workbook, BDatasets, "panel_mesh", tr("Mesh: ")+plot->title() )
 {
  m_plot = plot;
 }

//-----------------------------------------------------------//

KSPanelButtonMesh::~KSPanelButtonMesh()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonMesh::createPanel( QWidget *panelParent )
 {
  return new KSMeshPanel( panelParent, m_workbook, m_plot );
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSMeshPanel::KSMeshPanel(QWidget *parent, KSWorkbook *workbook, QSPlot3D *plot, const char *name)
: KSPanel(parent,name)
 {
  assert( plot ); p3 = plot;
  m_workbook = workbook;
  initDialog();
  col1 = new KSFillBtn( this ); col1->setGeometry( col1w->geometry() );
  col2 = new KSFillBtn( this ); col2->setGeometry( col2w->geometry() );
  col3 = new KSFillBtn( this ); col3->setGeometry( col3w->geometry() );
  col4 = new KSFillBtn( this ); col4->setGeometry( col4w->geometry() );
  col5 = new KSFillBtn( this ); col5->setGeometry( col5w->geometry() );
  top = new KSFillBtn( this ); top->setGeometry( topw->geometry() );
  bot = new KSFillBtn( this ); bot->setGeometry( botw->geometry() );
  line = new KSLineBtn( this ); line->setGeometry( linew->geometry() );
  point = new KSPointBtn( this ); point->setGeometry( pointPlace->geometry() );

  type->insertItem(tr("Default"));
  type->insertItem(tr("Gray"));
  type->insertItem(tr("Strange"));
  type->insertItem(tr("Two colors"));
  type->insertItem(tr("Five colors"));
  connect( type, SIGNAL(activated(int)), this, SLOT(gradient_type_changed(int)) );

  gradientw->setBackgroundMode( NoBackground );
  gradientw->installEventFilter( this );

  QWhatsThis::add( colorc, i18n("Auto colors\n\nIf checked auto-colors are used as a fill color for a top side of the mesh."
                           " Default colors/Top is used otherwise.") );
  QWhatsThis::add( top,    i18n("Top color\n\nDefault color for a top side of the mesh. Used only if Auto color"
                           " is off.") );
  QWhatsThis::add( bot,    i18n("Bottom color\n\nDefault color for the bottom side of the mesh.") );
  QWhatsThis::add( col1,   i18n("Default color for points with lowest Z values.") );
  QWhatsThis::add( col2,   i18n("Default color for points with low Z values.") );
  QWhatsThis::add( col3,   i18n("Default color for points with middle Z values.") );
  QWhatsThis::add( col4,   i18n("Default color for points with great Z values.") );
  QWhatsThis::add( col5,   i18n("Default color for points with greates Z values.") );
  QWhatsThis::add( line,   i18n("Line style\n\nLine style for the mesh stroke.") );
  QWhatsThis::add( gradientw, i18n("Gradient preview\n\nNothing really usable :)") );

  watchProperties();
 }

//-------------------------------------------------------------//

KSMeshPanel::~KSMeshPanel()
 {
 }

//-------------------------------------------------------------//

void KSMeshPanel::readProperties()
 {
  colorc->setChecked( p3->colored() );
  divide->setChecked( p3->autoDivide() );
  topbottom->setChecked( p3->topBottom() );
  tvertex->setValue( p3->tVertex() );

  top->setDialogFill( p3->fill(QSPlot3D::TMeshFill) );
  bot->setDialogFill( p3->fill(QSPlot3D::BMeshFill) );
  line->setDialogLine( p3->line(QSPlot3D::MeshLine) );
  point->setDialogPoint( p3->point(QSPlot3D::PointMark) );

  QSGGradient g = p3->gradient();

 	type->setCurrentItem( g.type() );
        gradient_type_changed(g.type());

  	col1->setDialogFill( g.fill(0) );
  	col2->setDialogFill( g.fill(1) );
  	col3->setDialogFill( g.fill(2) );
  	col4->setDialogFill( g.fill(3) );
  	col5->setDialogFill( g.fill(4) );

  preview();
 }

//-------------------------------------------------------------//

void KSMeshPanel::applyProperties()
 {
  KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties( p3 );
  cmd->setProperty( "gradient", toQString( QSGGradient( col1->dialogFill(),
					      col2->dialogFill(),
					      col3->dialogFill(),
					      col4->dialogFill(),
					      col5->dialogFill(),
					      (QSGGradient::Type )type->currentItem())) );
  cmd->setProperty( "colored", colorc->isChecked() );
  cmd->setProperty( "autoDivide", divide->isChecked() );
  cmd->setProperty( "topBottom", topbottom->isChecked() );
  cmd->setProperty( "tVertex", tvertex->value() );
  cmd->setFill( QSPlot3D::TMeshFill, top->dialogFill() );
  cmd->setFill( QSPlot3D::BMeshFill, bot->dialogFill() );
  cmd->setLine( QSPlot3D::MeshLine, line->dialogLine() );
  cmd->setPoint( QSPlot3D::PointMark, point->dialogPoint() );
  m_workbook->execute( cmd );
  preview();
 }


//-------------------------------------------------------------//

bool KSMeshPanel::eventFilter( QObject *o, QEvent *e )
 {
  QSGFill fill;

  if ( o == gradientw ) {
  if (  e->type() == QEvent::Paint ) {
	QPainter p( gradientw );
        QSGGradient g = p3->gradient();
        int w = gradientw->size().width();
        int h = gradientw->size().height();
        for ( int i=0; i<h; i++ ) {
                g.fill( double(h-i) / h, fill );
                p.setPen( QColor( fill.color.r, fill.color.g, fill.color.b ) );
                p.drawLine( 0, i, w-1, i );
               }
         return TRUE;
        }
     }

  return KSPanel::eventFilter(o,e);
 }

//-------------------------------------------------------------//

void KSMeshPanel::preview()
 {
  gradientw->update();
 }

//-------------------------------------------------------------//

void KSMeshPanel::gradient_type_changed(int )
 {
  QSGGradient g = QSGGradient( (QSGGradient::Type )type->currentItem() );
  col1->show();
  col2->show();
  col3->show();
  col4->show();
  col5->show();

  	col1->setDialogFill( g.fill(0) );
  	col2->setDialogFill( g.fill(1) );
  	col3->setDialogFill( g.fill(2) );
  	col4->setDialogFill( g.fill(3) );
  	col5->setDialogFill( g.fill(4) );


  if ( type->currentItem() == QSGGradient::GTwoColors ) {
	col2->hide();
	col3->hide();
	col4->hide();
	}
  else
  if ( type->currentItem() == QSGGradient::GFiveColors ) {
	}
  else {
        col1->hide();
	col2->hide();
	col3->hide();
	col4->hide();
	col5->hide();
       }

  preview();
 }


