/***************************************************************************
                                 kspointpanel.cpp
                             -------------------
    begin                : Sat Jan 15 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kspointpanel.h"
#include "../widgets/qsdrvqt.h"

KSPointPanel::KSPointPanel(QWidget *parent, const char *name)
: KSAttrPanel(parent,name)
 {
  initDialog();
  QSGPoint p;
  p.fill  = QSGPoint::Transparent;
  p.style = QSGPoint::Invisible;   new KSListBoxPoint( plist, p, i18n("Invisible") );
  p.style = QSGPoint::Circle;      new KSListBoxPoint( plist, p, i18n("Circle") );
  p.style = QSGPoint::Rect;        new KSListBoxPoint( plist, p, i18n("Rectangle") );
  p.style = QSGPoint::Triangle;    new KSListBoxPoint( plist, p, i18n("Triangle") );
  p.style = QSGPoint::Diamond;     new KSListBoxPoint( plist, p, i18n("Diamond") );
  p.style = QSGPoint::Cross;       new KSListBoxPoint( plist, p, i18n("Cross") );
  p.style = QSGPoint::Plus;        new KSListBoxPoint( plist, p, i18n("Plus") );
  p.style = QSGPoint::HLine;       new KSListBoxPoint( plist, p, i18n("HLine") );
  p.style = QSGPoint::VLine;       new KSListBoxPoint( plist, p, i18n("VLine") );
  p.fill  = QSGPoint::Filled;
  p.style = QSGPoint::Circle;     new KSListBoxPoint( plist, p, i18n("Filled Circle") );
  p.style = QSGPoint::Rect;       new KSListBoxPoint( plist, p, i18n("Filled Rectangle") );
  p.style = QSGPoint::Triangle;   new KSListBoxPoint( plist, p, i18n("Filled Triangle") );
  p.style = QSGPoint::Diamond;    new KSListBoxPoint( plist, p, i18n("Filled Diamond") );
  p.fill  = QSGPoint::Opaque;
  p.style = QSGPoint::Circle;     new KSListBoxPoint( plist, p, i18n("Opaque Circle") );
  p.style = QSGPoint::Rect;       new KSListBoxPoint( plist, p, i18n("Opaque Rectangle") );
  p.style = QSGPoint::Triangle;   new KSListBoxPoint( plist, p, i18n("Opaque Triangle") );
  p.style = QSGPoint::Diamond;    new KSListBoxPoint( plist, p, i18n("Opaque Diamond") );

  plist->setHScrollBarMode( QScrollView::AlwaysOff );
  plist->setCurrentItem(0);
  sizes->setValue(9);
  connect( plist, SIGNAL(highlighted(int)), this, SLOT(changePanelPointShape(int)) );
  connect( sizes, SIGNAL(valueChanged(int)), this, SLOT(changePanelPointShape(int)) );
 }

//-------------------------------------------------------------//

KSPointPanel::~KSPointPanel()
 {
 }

//-------------------------------------------------------------//

void KSPointPanel::setPanelPointShape( const QSGPoint& new_point )
 {
  if ( p != new_point ) {
         int i;
         p = new_point;
         disconnect( plist, SIGNAL(highlighted(int)), this, SLOT(changePanelPointShape(int)) );
         disconnect( sizes, SIGNAL(valueChanged(int)), this, SLOT(changePanelPointShape(int)) );
         sizes->setValue( p.size );
         for ( i=0; i<(int )plist->count(); i++ ) {
                KSListBoxPoint *it = dynamic_cast<KSListBoxPoint*>(plist->item(i));
                if ( it && it->pt.style == p.style && it->pt.fill == p.fill ) {
                         plist->setCurrentItem(i);
                         break;
                        }
               }
         connect( plist, SIGNAL(highlighted(int)), this, SLOT(changePanelPointShape(int)) );
         connect( sizes, SIGNAL(valueChanged(int)), this, SLOT(changePanelPointShape(int)) );
        }
 }
//-------------------------------------------------------------//

void KSPointPanel::changePanelPointShape(int)	
 {
  KSListBoxPoint *it = dynamic_cast<KSListBoxPoint*>(plist->item(plist->currentItem()));
  if ( it ) {
         QSGPoint new_p = p;
         new_p.style = it->pt.style;
         new_p.fill  = it->pt.fill;
         new_p.size  = sizes->value();
         setPanelPointShape( new_p );
        }
 }

//-----------------------------------------------------------//

void KSPointPanel::setPanelPoint( const QSGPoint &p )
 {
  setPanelPointShape( p );
  setPanelColor( p.color );
 }

//-----------------------------------------------------------//

QSGPoint KSPointPanel::panelPoint() const
 {
  QSGPoint result;
  result = panelPointShape();
  result.color = panelColor();
  return result;
 }
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

#define POINTBOX_SIZE_W  20
#define POINTBOX_SIZE_H  15
#define POINTBOX_KSACING_W  5
#define POINTBOX_KSACING_H  5

KSListBoxPoint::KSListBoxPoint( QListBox *parent, const QSGPoint &init_p, const QString& name )
:QListBoxItem(parent), pt(init_p)
 {
  setText( name );
 }

//-------------------------------------------------------------//

KSListBoxPoint::~KSListBoxPoint()
 {
 }

//-------------------------------------------------------------//

int KSListBoxPoint::height( const QListBox *lb ) const
 {
  return QMAX( POINTBOX_SIZE_H+2*POINTBOX_KSACING_H, lb->fontMetrics().lineSpacing()+1 );
 }

//-------------------------------------------------------------//

int KSListBoxPoint::width( const QListBox *lb ) const
 {
  return lb->fontMetrics().width(text()) + POINTBOX_SIZE_W + POINTBOX_KSACING_W*3;
 }

//-------------------------------------------------------------//

void KSListBoxPoint::paint( QPainter *p )
 {
  int w = p->fontMetrics().width(text()) + POINTBOX_SIZE_W + 3*POINTBOX_KSACING_W;
  int h = QMAX( POINTBOX_SIZE_H+2*POINTBOX_KSACING_H, p->fontMetrics().lineSpacing()+1 );


  QRect r( POINTBOX_KSACING_W,
           (h-POINTBOX_SIZE_H)/2,
           POINTBOX_SIZE_W,
           POINTBOX_SIZE_H );

  QSGPoint pts = pt; pts.size = 9; pts.color = QSDrvQt::toQSGColor( p->pen().color() );

  QSDrvQt drv;
  drv.setDC(p,72,false);
  drv.startDrawing();
  drv.drawPoint( QSPt2f(r.center().x(), r.center().y()), pts );
  drv.stopDrawing();
  p->drawText( POINTBOX_SIZE_W+2*POINTBOX_KSACING_W,
               0,
               w,
               h,
               Qt::AlignLeft | Qt::AlignVCenter,
               text() );
 }


