/***************************************************************************
                          ksrangepanel.cpp  -  description
                             -------------------
    begin                : Sun Sep 2 2001
    copyright            : (C) 2001 by Kamil Dobkowski
    email                : kamildobk@poczta.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksrangepanel.h"
#include "../widgets/qsaxes3d.h"
#include "../widgets/qsaxes2d.h"
#include "../widgets/qsaxis.h"
#include <qpainter.h>
#include <qwhatsthis.h>
#include <kiconloader.h>
#include <qinputdialog.h>
#include "../part/kmatplotfactory.h"
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonRange::KSPanelButtonRange( QWidget *parent, KSWorkbook *workbook, QSAxes *axes )
:KSPanelButton( parent, workbook, BGeneral, "panel_position", tr("Range") )
 {
  m_axes = axes;
 }

//-----------------------------------------------------------//

KSPanelButtonRange::~KSPanelButtonRange()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonRange::createPanel( QWidget *panelParent )
 {
  return new KSRangePanel( panelParent, m_workbook, m_axes );
 }

//-----------------------------------------------------------//

bool KSPanelButtonRange::isButtonFor( QSCObject *, int, int )
 {
  /*
  if ( category == QSAxes::GeneralCategory &&
       object->isAxesShadow() &&
       element == 0 ) return true;
  */
  return false;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSRangePanel::KSRangePanel(QWidget *parent, KSWorkbook *workbook, QSAxes *axes, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  m_axes = axes;
  m_workbook = workbook;
  viewUp->setPixmap(BarIcon("up",KMatplotFactory::global()));
  viewDown->setPixmap(BarIcon("down",KMatplotFactory::global()));
  viewLeft->setPixmap(BarIcon("back",KMatplotFactory::global()));
  viewRight->setPixmap(BarIcon("forward",KMatplotFactory::global()));

  xPlus->setPixmap(BarIcon("1rightarrow",KMatplotFactory::global()));
  xMinus->setPixmap(BarIcon("1leftarrow",KMatplotFactory::global()));

  yPlus->setPixmap(BarIcon("1uparrow",KMatplotFactory::global()));
  yMinus->setPixmap(BarIcon("1downarrow",KMatplotFactory::global()));

  zPlus->setPixmap(BarIcon("1uparrow",KMatplotFactory::global()));
  zMinus->setPixmap(BarIcon("1downarrow",KMatplotFactory::global()));

  vPlus->setPixmap(BarIcon("1uparrow",KMatplotFactory::global()));
  vMinus->setPixmap(BarIcon("1downarrow",KMatplotFactory::global()));

  connect( rememberView, SIGNAL(clicked()), this, SLOT(remember_view()) );

  connect( view1, SIGNAL(clicked()), this, SLOT(set_view_1()) );
  connect( view2, SIGNAL(clicked()), this, SLOT(set_view_2()) );
  connect( view3, SIGNAL(clicked()), this, SLOT(set_view_3()) );


  connect( viewUp, SIGNAL(clicked()), this, SLOT(view_up()) );
  connect( viewDown, SIGNAL(clicked()), this, SLOT(view_down()) );
  connect( viewLeft, SIGNAL(clicked()), this, SLOT(view_left()) );
  connect( viewRight, SIGNAL(clicked()), this, SLOT(view_right()) );

  connect( xPlus, SIGNAL(clicked()), this, SLOT(x_plus()) );
  connect( yPlus, SIGNAL(clicked()), this, SLOT(y_plus()) );
  connect( zPlus, SIGNAL(clicked()), this, SLOT(z_plus()) );
  connect( vPlus, SIGNAL(clicked()), this, SLOT(v_plus()) );

  connect( xMinus, SIGNAL(clicked()), this, SLOT(x_minus()) );
  connect( yMinus, SIGNAL(clicked()), this, SLOT(y_minus()) );
  connect( zMinus, SIGNAL(clicked()), this, SLOT(z_minus()) );
  connect( vMinus, SIGNAL(clicked()), this, SLOT(v_minus()) );

  m_axes3 = dynamic_cast<QSAxes3D*>(axes);

  if ( !m_axes3 ) {
	viewLeft->setEnabled( false );
	viewRight->setEnabled( false );
	viewUp->setEnabled( false );
	viewDown->setEnabled( false );
	}
 }

//-----------------------------------------------------------//

KSRangePanel::~KSRangePanel()
 {
 }

//-----------------------------------------------------------//

void KSRangePanel::readProperties()
 {
 }

//-----------------------------------------------------------//

void KSRangePanel::applyProperties()
 {
 }

//-----------------------------------------------------------//

void KSRangePanel::x_plus()
 {
  shift_range( QSAxis::XAxisType, 0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::y_plus()
 {
  shift_range( QSAxis::YAxisType, 0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::z_plus()
 {
  shift_range( QSAxis::ZAxisType, 0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::v_plus()
 {
  shift_range( QSAxis::VAxisType, 0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::x_minus()
 {
  shift_range( QSAxis::XAxisType, -0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::y_minus()
 {
  shift_range( QSAxis::YAxisType, -0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::z_minus()
 {
  shift_range( QSAxis::ZAxisType, -0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::v_minus()
 {
  shift_range( QSAxis::VAxisType, -0.1 );
 }

//-----------------------------------------------------------//

void KSRangePanel::view_up()
 {
  if ( m_axes3 ) {
	m_axes3->setElevation( m_axes3->elevation()+5 );
	}
 }

//-----------------------------------------------------------//

void KSRangePanel::view_down()
 {
   if ( m_axes3 ) m_axes3->setElevation( m_axes3->elevation()-5 );
 }

 //-----------------------------------------------------------//

void KSRangePanel::view_left()
 {
  if ( m_axes3 ) m_axes3->setAzimuth( m_axes3->azimuth()-15 );
 }

//-----------------------------------------------------------//

void KSRangePanel::view_right()
 {
  if ( m_axes3 ) m_axes3->setAzimuth( m_axes3->azimuth()+15 );
 }

//-----------------------------------------------------------//

void KSRangePanel::shift_range( int axis_type, double shift )
 {
  KSCmdSetRanges *cmd = new KSCmdSetRanges( m_axes );
  for( int axis_nr=0; axis_nr<m_axes->axisCount(); axis_nr++ ) {
	QSAxis *axis = m_axes->axis(axis_nr);
	if ( axis->type() == axis_type && axis->scrollable() ) {
		double min = axis->worldToData(0.0+shift);
		double max = axis->worldToData(1.0+shift);
		axis->setRange( min, max );
		}
	}
  cmd->commit();
  m_workbook->execute( cmd );
 }

//		axis->setRange( min, max );
//-----------------------------------------------------------//

void KSRangePanel::set_view_1()
 {
  KSCmdSetRanges *cmd = new KSCmdSetRanges( m_axes );
  m_axes->setRememberedView( 0 );
  cmd->commit();
  m_workbook->execute( cmd );
 }

//-----------------------------------------------------------//

void KSRangePanel::set_view_2()
 {
  KSCmdSetRanges *cmd = new KSCmdSetRanges( m_axes );
  m_axes->setRememberedView( 1 );
  cmd->commit();
  m_workbook->execute( cmd );
 }

//-----------------------------------------------------------//

void KSRangePanel::set_view_3()
 {
  KSCmdSetRanges *cmd = new KSCmdSetRanges( m_axes );
  m_axes->setRememberedView( 2 );
  cmd->commit();
  m_workbook->execute( cmd );
 }

//-----------------------------------------------------------//

void KSRangePanel::remember_view()
 {
  bool ok = FALSE;
  int view_number = QInputDialog::getInteger( tr( "Please enter a number" ), "", 1, 1, 3, 1, &ok, this );
  if ( ok ) m_axes->rememberCurrentView( view_number-1 );
 }


  /*
  viewUp->setFlat( TRUE );
  viewDown->setFlat( TRUE );
  viewLeft->setFlat( TRUE );
  viewRight->setFlat( TRUE );

  xPlus->setFlat( TRUE );
  yPlus->setFlat( TRUE );
  zPlus->setFlat( TRUE );
  vPlus->setFlat( TRUE );

  xMinus->setFlat( TRUE );
  yMinus->setFlat( TRUE );
  zMinus->setFlat( TRUE );
  vMinus->setFlat( TRUE );
  */

