/***************************************************************************
                              ksseriespanel.cpp
                             -------------------
    begin                : Mon Jan 10 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksseriespanel.h"
#include "ksattrbtn.h"
#include "../part/kscommands.h"
#include <qstringlist.h>
#include <qwhatsthis.h>

//-----------------------------------------------------------//

KSPanelButtonCurve::KSPanelButtonCurve( QWidget *parent, KSWorkbook *workbook, QSCurve *curve )
:KSPanelButton( parent, workbook, BDatasets, "curve", tr("Curve: ")+curve->title() )
 {
  m_curve = curve;
 }

//-----------------------------------------------------------//

KSPanelButtonCurve::~KSPanelButtonCurve()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonCurve::createPanel( QWidget *panelParent )
 {
  return new KSSeriesPanel( panelParent, m_workbook, m_curve );
 }

//-----------------------------------------------------------//

bool KSPanelButtonCurve::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::DatasetCategory &&
       object->isAxesShadow() &&
       object->parentAxes()->plot(element) == m_curve ) return true;
  return false;
 }
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

//-------------------------------------------------------------//

KSSeriesPanel::KSSeriesPanel(QWidget *parent, KSWorkbook *workbook, QSCurve *plot, const char *name)
: KSPanel(parent,name)
 {
  initDialog();

  assert( plot ); curve = plot;
  m_workbook = workbook;
  typec->clear();
  typec->insertItem(i18n("Lines"));
  typec->insertItem(i18n("Area"));
  typec->insertItem(i18n("Ribbon"));
  typec->insertItem(i18n("Bars"));
  typec->insertItem(i18n("Vectors"));
  typec->insertItem(i18n("Flux"));
  typec->insertItem(i18n("Rectangles"));
  typec->insertItem(i18n("Ellipses"));
  typec->insertItem(i18n("Left Stairs"));
  typec->insertItem(i18n("Middle Stairs"));
  typec->insertItem(i18n("Right Stairs"));


  col   = new KSFillBtn( this ); col->setGeometry( colw->geometry() );
  line  = new KSLineBtn( this ); line->setGeometry( linew->geometry() );
  point = new KSPointBtn( this ); point->setGeometry( pointw->geometry() );
  arrow1 = new KSArrowBtn( this,true ); arrow1->setGeometry( arrow1w->geometry() );
  arrow2 = new KSArrowBtn( this ); arrow2->setGeometry( arrow2w->geometry() );
  error  = new KSLineBtn( this ); error->setGeometry( errorw->geometry() );
  linex  = new KSLineBtn( this ); linex->setGeometry( linexw->geometry() );
  liney  = new KSLineBtn( this ); liney->setGeometry( lineyw->geometry() );

  m_x_select = new KSAxisSelect( xaxisc, curve->parentAxes(), QSAxis::XAxisType );
  m_y_select = new KSAxisSelect( yaxisc, curve->parentAxes(), QSAxis::YAxisType );

  QWhatsThis::add( titlee, i18n("Series title\n\nIt is displayed on the legend.") );

  QWhatsThis::add( typec, i18n("Style\n\nPlot style for the current series.") );

  QWhatsThis::add( line, i18n("Line style\n\nLine style used to draw the current series") );
  QWhatsThis::add( col, i18n("Fill style\n\n Used to draw interiors of Polys, Rectangles, Ellipses and Bars") );
  QWhatsThis::add( point, i18n("Point style\n\nStyle of point marks used to draw data points") );
  QWhatsThis::add( error, i18n("Errorbars line style\n\nLine style used to draw errorbars. See also 'Arrows' below.") );

  QWhatsThis::add( deltapdxe, i18n("Delta % X value\n\nThis value is used when drawing Errorbars, Vectors, Rectangles, Ellipses or Polys. "
  				  "Total delta value is calculated as follows: DX[i] = DXVector[i] + FixedDeltaX + PercentDeltaX*XVector[i]/100.0;") );
  QWhatsThis::add( deltapdye, i18n("Delta % Y value\n\nThis value is used when drawing Errorbars, Vectors, Rectangles, Ellipses or Polys."
			          "Total delta value is calculated as follows: DY[i] = DYVector[i] + FixedDeltaY + PercentDeltaY*YVector[i]/100.0;") );
  QWhatsThis::add( deltadxe, i18n("Fixed Delta X value\n\nThis value is used when drawing Errorbars, Vectors, Rectangles, Ellipses or Polys."
  				  "Total delta value is calculated as follows: DX[i] = DXVector[i] + FixedDeltaX + PercentDeltaX*XVector[i]/100.0;") );
  QWhatsThis::add( deltadye, i18n("Fixed Delta Y value\n\nThis value is used when drawing Errorbars, Vectors, Rectangles, Ellipses or Polys."
			          "Total delta value is calculated as follows: DY[i] = DYVector[i] + FixedDeltaY + PercentDeltaY*YVector[i]/100.0;") );

  QWhatsThis::add( zeroxe, i18n("Zero Level X\n\nThis an X coordinate where XLines begin") );
  QWhatsThis::add( zeroye, i18n("Zero Level Y\n\nThis an Y coordinate where YLines and Bars begin") );

  QWhatsThis::add( liney, i18n("Y Lines\n\n Horizontal lines, which crosses data points on the plot") );
  QWhatsThis::add( linex, i18n("X Lines\n\n Vertical lines, which crosses data points on the plot") );

  QWhatsThis::add( arrow1, i18n("Arrow style\n\n Arrow style used to draw horizontal bars or a vector begin") );
  QWhatsThis::add( arrow2, i18n("Arrow style\n\n Arrow style used to draw vertical bars or a vector end") );


  watchProperties();
 }

//-------------------------------------------------------------//

KSSeriesPanel::~KSSeriesPanel()
 {
 }



//-------------------------------------------------------------//

void KSSeriesPanel::readProperties()
 {
  typec->setCurrentItem( curve->type() );
  m_x_select->readProperties( curve->defaultAxis(QSAxis::XAxisType)  );
  m_y_select->readProperties( curve->defaultAxis(QSAxis::YAxisType)  );

  legendc->setChecked( curve->legendItemVisible() );

  deltapdxe->setText( QString::number(curve->percentDelta().x) );
  deltapdye->setText( QString::number(curve->percentDelta().y) );

  deltadxe->setText( QString::number(curve->fixedDelta().x) );
  deltadye->setText( QString::number(curve->fixedDelta().y) );

  zeroxe->setText( QString::number(curve->zeroPoint().x) );
  zeroye->setText( QString::number(curve->zeroPoint().y) );
  error->setDialogLine( curve->line(QSCurve::ErrorLine) );
  linex->setDialogLine( curve->line(QSCurve::XLine) );
  liney->setDialogLine( curve->line(QSCurve::YLine) );
  line->setDialogLine( curve->line(QSCurve::BaseLine) );
  col->setDialogFill( curve->fill(QSCurve::BaseFill) );
  point->setDialogPoint( curve->point(QSCurve::PointMark) );
  arrow1->setDialogArrow( curve->arrow1() );
  arrow2->setDialogArrow( curve->arrow2() );
  titlee->setText( curve->title() );
 }

//-------------------------------------------------------------//

void KSSeriesPanel::applyProperties()
{
 KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties(curve);
 cmd->setProperty( "defaultXAxis", curve->parentAxes()->axisIndex(m_x_select->selectedAxis()) );
 cmd->setProperty( "defaultYAxis", curve->parentAxes()->axisIndex(m_y_select->selectedAxis()) );
 cmd->setProperty( "legendItemVisible", legendc->isChecked() );
 cmd->setProperty( "title", titlee->text() );

 cmd->setProperty( "type", typec->currentItem() );
 cmd->setProperty( "zeroLevelX", zeroxe->text().toDouble() );
 cmd->setProperty( "zeroLevelY", zeroye->text().toDouble() );
 cmd->setProperty( "fixedDX", deltadxe->text().toDouble() );
 cmd->setProperty( "fixedDY", deltadye->text().toDouble() );
 cmd->setProperty( "percentDX", deltapdxe->text().toDouble() );
 cmd->setProperty( "percentDY", deltapdye->text().toDouble() );

 cmd->setLine( QSCurve::BaseLine, line->dialogLine() );
 cmd->setLine( QSCurve::ErrorLine, error->dialogLine() );
 cmd->setLine( QSCurve::XLine, linex->dialogLine() );
 cmd->setLine( QSCurve::YLine, liney->dialogLine() );
 cmd->setFill( QSCurve::BaseFill, col->dialogFill() );
 cmd->setPoint( QSCurve::PointMark, point->dialogPoint() );

 cmd->setProperty( "arrow1", toQString(arrow1->dialogArrow()) );
 cmd->setProperty( "arrow2", toQString(arrow2->dialogArrow()) );

 m_workbook->execute( cmd );
}

//-------------------------------------------------------------//

/*
  curve->setType( (QSCurve::SeriesType )typec->currentItem() );
  curve->setDefaultAxis( QSAxis::XAxisType, m_x_select->selectedAxis() );
  curve->setDefaultAxis( QSAxis::YAxisType, m_y_select->selectedAxis() );

  curve->setLegendItemVisible( legendc->isChecked() );
  curve->setTitle( titlee->text() );
  curve->setZeroPoint( QSPt2f( zeroxe->text().toDouble(),zeroye->text().toDouble()) );
  curve->setFixedDelta( QSPt2f( deltadxe->text().toDouble(),deltadye->text().toDouble()) );
  curve->setPercentDelta( QSPt2f( deltapdxe->text().toDouble(),deltapdye->text().toDouble()) );
  curve->setLine( QSCurve::BaseLine, line->dialogLine() );
  curve->setLine( QSCurve::ErrorLine, error->dialogLine() );
  curve->setLine( QSCurve::XLine, linex->dialogLine() );
  curve->setLine( QSCurve::YLine, liney->dialogLine() );
  curve->setFill( QSCurve::BaseFill, col->dialogFill() );
  curve->setPoint( QSCurve::PointMark, point->dialogPoint() );
  curve->setArrow1( arrow1->dialogArrow() );
  curve->setArrow2( arrow2->dialogArrow() );
*/
/*
  xaxisc->setCurrentItem( type_axis_number(QSAxes::XAxisType,curve->defaultAxis(QSAxes::XAxisType)) );
  yaxisc->setCurrentItem( type_axis_number(QSAxes::YAxisType,curve->defaultAxis(QSAxes::YAxisType)) );
  curve->setDefaultAxis( QSAxes::XAxisType, axis_number(QSAxes::XAxisType,xaxisc->currentItem()) );
  curve->setDefaultAxis( QSAxes::YAxisType, axis_number(QSAxes::YAxisType,yaxisc->currentItem()) );
	int axis_number( QSAxes::AxisType t, int type_axis_number );
	int type_axis_number( QSAxes::AxisType t, int axis_number );


int KSSeriesPanel::axis_number( QSAxes::AxisType t, int type_axis_number )
 {
  int number = 0;
  for ( int i=0; i<curve->parentAxes()->axesCount(); i++ )
  	 if ( curve->parentAxes()->axisType(i) == t )
  	 	 if ( number++ == type_axis_number ) return i;
  return -1;  	 	
 }

//-------------------------------------------------------------//

int KSSeriesPanel::type_axis_number( QSAxes::AxisType t, int axis_number )
 {
  int result = -1;
  for ( int i=0; i<=axis_number; i++ )
  	 if ( curve->parentAxes()->axisType(i) == t ) result++;
  //cout << "axis nr " << axis_number << " is " << result << "axis of type" << t << endl;
  return result;
 }

  int xnr = 0;
  int ynr = 0;
  for( int i=0; i<curve->parentAxes()->axesCount(); i++ ) {
  	 QSAxes::AxisType type = curve->parentAxes()->axisType(i);
  	 if ( type == QSAxes::XAxisType ) xaxisc->insertItem( QString(tr("x"))+QString::number(++xnr) );
  	 if ( type == QSAxes::YAxisType ) yaxisc->insertItem( QString(tr("y"))+QString::number(++ynr) );
  	}
   QWhatsThis::add( yaxisc, i18n("Y axis\n\nAxis, which the current series is bound to. The first Y axis is "
                           "always placed at the left side of the plot and it is a default one. "
                           "The second Y axis is placed at the right and by default has hidden "
                           "labels") );
  QWhatsThis::add( xaxisc, i18n("X axis\n\nAxis, which the current series is bound to. The first X axis is "
                           "always placed at the bottom side and it is a default one. "
                           "The second X axis is placed at the top and by default has hidden "
                           "labels") );

*/

