/***************************************************************************
                                 kssurfacepanel.cpp
                             -------------------
    begin                : Tue Jan 11 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kssurfacepanel.h"
#include "../widgets/qssurface.h"
#include "../part/kscommands.h"
#include <qwhatsthis.h>

//-----------------------------------------------------------//

KSPanelButtonSurface::KSPanelButtonSurface( QWidget *parent, KSWorkbook *workbook, QSSurface *surface )
:KSPanelButton( parent, workbook, BDatasets, "surface", tr("Surface: ")+surface->title() )
 {
  m_surface = surface;
 }

//-----------------------------------------------------------//

KSPanelButtonSurface::~KSPanelButtonSurface()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonSurface::createPanel( QWidget *panelParent )
 {
  return new KSSurfacePanel( panelParent, m_workbook, m_surface );
 }

//-----------------------------------------------------------//

bool KSPanelButtonSurface::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::DatasetCategory &&
       object->isAxesShadow() &&
       object->parentAxes()->plot(element) == m_surface ) return true;
  return false;
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSSurfacePanel::KSSurfacePanel(QWidget *parent, KSWorkbook *workbook, QSSurface *init_surface, const char *name)
: KSPanel(parent,name)
 {
  initDialog();
  surface = init_surface;
  m_workbook = workbook;
  m_x_select = new KSAxisSelect( xaxisc, surface->parentAxes(), QSAxis::XAxisType );
  m_y_select = new KSAxisSelect( yaxisc, surface->parentAxes(), QSAxis::YAxisType );
  m_z_select = new KSAxisSelect( zaxisc, surface->parentAxes(), QSAxis::ZAxisType );
  m_v_select = new KSAxisSelect( vaxisc, surface->parentAxes(), QSAxis::VAxisType );
  watchProperties();
 }

//------------------------------------------------------------//

KSSurfacePanel::~KSSurfacePanel()
 {
 }

//------------------------------------------------------------//

void KSSurfacePanel::readProperties()
 {
  m_x_select->readProperties( surface->defaultAxis(QSAxis::XAxisType)  );
  m_y_select->readProperties( surface->defaultAxis(QSAxis::YAxisType)  );
  m_z_select->readProperties( surface->defaultAxis(QSAxis::ZAxisType)  );
  m_v_select->readProperties( surface->defaultAxis(QSAxis::VAxisType)  );
  legendc->setChecked( surface->legendItemVisible() );
  titlee->setText( surface->title() );
 }

//------------------------------------------------------------//

void KSSurfacePanel::applyProperties()
 {
  KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties(surface);

  cmd->setProperty( "legendItemVisible", legendc->isChecked() );
  cmd->setProperty( "title", titlee->text() );
  cmd->setProperty( "defaultXAxis", surface->parentAxes()->axisIndex(m_x_select->selectedAxis()) );
  cmd->setProperty( "defaultYAxis", surface->parentAxes()->axisIndex(m_y_select->selectedAxis()) );
  cmd->setProperty( "defaultZAxis", surface->parentAxes()->axisIndex(m_z_select->selectedAxis()) );
  cmd->setProperty( "defaultVAxis", surface->parentAxes()->axisIndex(m_v_select->selectedAxis()) );

  m_workbook->execute( cmd );
 }

/*
 surface->setDefaultAxis( QSAxis::XAxisType, m_x_select->selectedAxis() );
 surface->setDefaultAxis( QSAxis::YAxisType, m_y_select->selectedAxis() );
 surface->setDefaultAxis( QSAxis::ZAxisType, m_z_select->selectedAxis() );
 surface->setDefaultAxis( QSAxis::VAxisType, m_v_select->selectedAxis() );
 surface->setLegendItemVisible( legendc->isChecked() );
 surface->setTitle( titlee->text() );
*/
//-------------------------------------------------------------//

