/***************************************************************************
                                 ksticspanel.cpp                             
                             -------------------                                         
    begin                : 01-January-2000
    copyright            : (C) 2000 by Kamil Dobkowski                         
    email                : kamildobk@poczta.onet.pl                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "ksticspanel.h"

#include "ksattrbtn.h"
#include <qvalidator.h>
#include <qtoolbutton.h>
#include <qwhatsthis.h>
#include <qtooltip.h>
#include <assert.h>
#include <kiconloader.h>
#include "../widgets/qsaxis.h"
#include "../part/kmatplotfactory.h"
#include "../part/kscommands.h"

//-----------------------------------------------------------//

KSPanelButtonGrid::KSPanelButtonGrid( QWidget *parent, KSWorkbook *workbook, QSAxis *axis )
:KSPanelButton( parent, workbook, BAxes )
 {
  m_axis = axis;


  QString icon_name;
  QString tool_tip;
  switch( m_axis->type() ) {
	case QSAxis::XAxisType: icon_name = "panel_gx"; tool_tip = tr("X grid : ")+axis->title(); break;
	case QSAxis::YAxisType: icon_name = "panel_gy"; tool_tip = tr("Y grid : ")+axis->title(); break;
	case QSAxis::ZAxisType: icon_name = "panel_gz"; tool_tip = tr("Z grid : ")+axis->title(); break;
	case QSAxis::VAxisType: icon_name = "panel_gv"; tool_tip = tr("V grid : ")+axis->title(); break;
	}
  setPixmap(BarIcon(icon_name,KMatplotFactory::global()));
  QToolTip::add( this, tool_tip );
  QWhatsThis::add( this, tool_tip );
 }

//-----------------------------------------------------------//

KSPanelButtonGrid::~KSPanelButtonGrid()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonGrid::createPanel( QWidget *panelParent )
 {
  return new KSTicsPanel( panelParent, m_workbook, m_axis );
 }

//-----------------------------------------------------------//

bool KSPanelButtonGrid::isButtonFor( QSCObject *object, int category, int element )
 {
  if ( category == QSAxes::GridCategory &&
       object->isAxesShadow() &&
       object->parentAxes()->axis(element) == m_axis ) return true;
  return false;
 }

//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//
//-----------------------------------------------------------//

KSTicsPanel::KSTicsPanel(QWidget *parent, KSWorkbook *workbook, QSAxis *axis, const char *name)
 : KSPanel(parent,name)
 {
  initDialog();
  m_axis = axis;
  m_workbook = workbook;

  codes = new QPopupMenu( formate );
  codes->setFont(QFont("helvetica", 10, 50));
  codes->insertItem( i18n("value"),  Value  );
  codes->insertItem( i18n("factor"), Factor );
  codes->insertItem( i18n("base"),   Base   );
  codes->insertItem( i18n("exponent"), Exponent );

  major = new KSLineBtn( this );
  minor = new KSLineBtn( this );
  font  = new KSFontBtn( this  );
  major->setGeometry( majorw->geometry() );
  minor->setGeometry( minorw->geometry() );
  font->setGeometry( fontw->geometry() );

  formate->installEventFilter( this );


  QWhatsThis::add( adjustc,  i18n("Adjust an axis range \n\nIf turned on the axis range "
                             "is rounded to a tic step border.") );
  QWhatsThis::add( majdensitye, i18n("Major tic step(value>0) \nMajor tic density (value<0)\n\nDensity is "
                                "only an approximative value. Number of visible tics "
                                "may be slightly different. ") );
  QWhatsThis::add( mindensitye, i18n("Minor tic step \nMinor tic density \n\nDensity is "
                                "only an approximative value. Number of visible tics "
                                "may be slightly different. ") );
  QWhatsThis::add( formate, i18n("Format of axis labels \n\nThis is a text, which will be displayed "
                            "as a label text.\nTo insert a tic value into it type <TIC>. Two "
                            "additional paramaters may be added after TIC word:\n"
                            "TYPE=\"?\", where ? can be one of the following letters: \n"
                            "V - tic value. This is a default value. \n"
                            "X - exponent part of the label value. \n"
                            "F - factor part of the label value. \n"
                            "B - base part of the label value.\n"
                            "\n"
                            "FORMAT=\"?\", where ? is an ordinary C-styled \"printf\" format, "
                            "but with only eEfgG types allowed. Default value is %g. "
                            "\n\n"
                            "Example:\n"
                            "<TIC TYPE=\"x\" FORMAT=\"%.2f\"> \n"
                            "\n"
                            "Press a right mouse button over the edit field to insert a standard format.") );
  QWhatsThis::add( major, i18n("Line style\n\nLine style for major grid lines.") );
  QWhatsThis::add( minor, i18n("Line style\n\nLine style for minor grid lines.") );
  QWhatsThis::add( font, i18n("Labels font\n\nFont used to draw labels.") );
  watchProperties();
 }

  //majdensitye->setValidator( new QDoubleValidator( 0.0, 100.0, 3, majdensitye ));
  //mindensitye->setValidator( new QDoubleValidator( 0.0, 100.0, 3, mindensitye ));
//-----------------------------------------------------------//

KSTicsPanel::~KSTicsPanel()
 {
 }

//-----------------------------------------------------------//

void KSTicsPanel::applyProperties()
 {
  KSCmdSetGraphicalProperties *cmd = new KSCmdSetGraphicalProperties(m_axis);
  cmd->setProperty( "ticsFormat", formate->text() );
  cmd->setProperty( "roundRangeToTicStep", adjustc->isChecked() );
  cmd->setProperty( "majorGridStep", majdensitye->text().toDouble() );
  cmd->setProperty( "minorGridStep", mindensitye->text().toDouble() );
  cmd->setFont( QSAxis::TicsFont, font->dialogFont()  );
  cmd->setLine( QSAxis::MajorGridLine, major->dialogLine() );
  cmd->setLine( QSAxis::MinorGridLine, minor->dialogLine() );
  cmd->setProperty( "ticsOuter", outerc->isChecked() );
  cmd->setProperty( "ticsVisible", showticc->isChecked() );
  cmd->setProperty( "ticsAngle", angle->value() );
  cmd->setProperty( "ticLabelPos1", evenPos->text().toDouble() );
  cmd->setProperty( "ticLabelPos2", oddPos->text().toDouble() );
  m_workbook->execute( cmd );
 }


//-----------------------------------------------------------//

void KSTicsPanel::readProperties()
 {
  formate->setText( m_axis->ticsFormat() );
  adjustc->setChecked( m_axis->roundRangeToTicStep() );
  majdensitye->setText( QString::number( m_axis->majorGridStep(), 'g', 3 ) );
  mindensitye->setText( QString::number( m_axis->minorGridStep(), 'g', 3 ) );
  font->setDialogFont(  m_axis->font(QSAxis::TicsFont)  );
  major->setDialogLine( m_axis->line(QSAxis::MajorGridLine) );
  minor->setDialogLine( m_axis->line(QSAxis::MinorGridLine) );
  outerc->setChecked( m_axis->ticsOuter() );
  showticc->setChecked( m_axis->ticsVisible() );
  angle->setValue( m_axis->ticsAngle() );
  evenPos->setText( QString::number( m_axis->ticLabelPos1(), 'g', 3 ) );
  oddPos->setText( QString::number( m_axis->ticLabelPos2(), 'g', 3 ) );
 }


//-----------------------------------------------------------//

bool KSTicsPanel::eventFilter( QObject *o, QEvent *e )
 {
  if ( e->type() == QEvent::MouseButtonPress && o == formate ) {
         // popup menu
         QMouseEvent *p = static_cast<QMouseEvent *>(e);

         if ( p->button() == RightButton ) {

                 FormatCode code = FormatCode( codes->exec( QCursor::pos()));

                 switch( code ) {
                         case Value:         formate->insert("<TIC TYPE=\"V\" FORMAT=\"%g\">");   break;
                         case Factor:        formate->insert("<TIC TYPE=\"F\" FORMAT=\"%.2f\">"); break;
                         case Exponent:      formate->insert("<TIC TYPE=\"X\" FORMAT=\"%g\">");   break;
                         case Base:          formate->insert("<TIC TYPE=\"B\" FORMAT=\"%g\">");   break;
                        }
                 return TRUE;
                }
        }

  return KSPanel::eventFilter(o,e);
 }









































