/***************************************************************************
                                 ksviewpanel.cpp
                             -------------------
    begin                : Mon Jan 10 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksviewpanel.h"
#include "../widgets/qsaxes3d.h"
#include "../part/kscommands.h"
#include <qwhatsthis.h>

//-----------------------------------------------------------//

KSPanelButtonView::KSPanelButtonView( QWidget *parent, KSWorkbook *workbook, QSAxes3D *axes )
:KSPanelButton( parent, workbook, BGeneral, "panel_view", tr("Viewpoint") )
 {
  m_axes = axes;
 }

//-----------------------------------------------------------//

KSPanelButtonView::~KSPanelButtonView()
 {
 }

//-----------------------------------------------------------//

KSPanel *KSPanelButtonView::createPanel( QWidget *panelParent )
 {
  return new KSViewPanel( panelParent, m_workbook, m_axes );
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//

KSViewPanel::KSViewPanel(QWidget *parent, KSWorkbook *workbook, QSAxes3D *plot, const char *name)
: KSPanel(parent,name)
 {
  initDialog();

  assert( plot ); p3 = plot;
  m_workbook = workbook;

  QWhatsThis::add( perspectivec, i18n("Perspective\n\nTurns on perspective."));
  QWhatsThis::add( autoscalec, i18n("Autoscale\n\nFits an axis box to the display area."));
  QWhatsThis::add( azimuths, i18n("Azimuth\n\nPosition of the viewpoint in spherical coordinates. Range is <0,359> degree.") );
  QWhatsThis::add( elevations, i18n("Elevation\n\nPosition of the viewpoint in spherical coordinates. Range is <-90,90> degree."));
  QWhatsThis::add( distances, i18n("Distance\n\nSize of the plot. Changes from -50 (0%) to 50 (200%).") );
  QWhatsThis::add( fdistances, i18n("Focuspoint distance\n\nDistance from the focus point to the "
                                "screen. Changes from -50 (almost zero) to 50 (almost inf).")  );

  watchProperties();
 }

//-----------------------------------------------------------//

KSViewPanel::~KSViewPanel()
 {
 }

//-----------------------------------------------------------//	

void KSViewPanel::readProperties()
 {
  azimuths->setValue( p3->azimuth() );
  elevations->setValue( p3->elevation() );
  distances->setValue( p3->distance() );
  fdistances->setValue( p3->focusDistance() );

  perspectivec->setChecked( p3->perspective() );
  autoscalec->setChecked( p3->autoscale() );
 }

//-----------------------------------------------------------//

void KSViewPanel::applyProperties()
 {
  KSCmdSetProperties *cmd = new KSCmdSetProperties(p3);

  cmd->setProperty( "azimuth", azimuths->value() );
  cmd->setProperty( "elevation", elevations->value() );
  cmd->setProperty( "distance", distances->value() );
  cmd->setProperty( "focusDistance", fdistances->value() );
  cmd->setProperty( "perspective", perspectivec->isChecked() );
  cmd->setProperty( "autoscale", autoscalec->isChecked() );

  m_workbook->execute( cmd );
 }


