/***************************************************************************
                                  kmatplotshell.cpp
                             -------------------
    begin                : Sat Mar 11 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmatplotshell.h"
#include "part/kmatplotpart.h"
#include "ksdeletedlg.h"
#include "ksgeneraldialogs.h"
#include "ksworksheetdlg.h"
#include "kssocketio.h"
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <kstdaction.h>
#include <kfiledialog.h>
#include <klibloader.h>
#include <klocale.h>
#include <kdebug.h>
#include <kkeydialog.h>
#include <kmenubar.h>
#include <kedittoolbar.h>
#include <kconfig.h>
#include <kapp.h>

#include "ksexportpicturedlg.h"
#include "part/ksmatrix.h"

// see main.cpp
extern KSSocketIO *kmatplot_socket;

//-----------------------------------------------------------------------------//

KMatplotShell::KMatplotShell( const char * )
: KParts::MainWindow()
 {
  //part_id = -1;
  resize( 675, 520 );
  m_kmatplot_part = NULL;

  (void )KStdAction::open( this, SLOT(slotFileOpen()), actionCollection() );
  (void )new KAction( i18n("&Merge ..."), 0,
		      this, SLOT(slotFileMerge()), actionCollection(), "file_merge" );
  (void )KStdAction::saveAs( this, SLOT(slotFileSaveAs()), actionCollection() );
  (void )KStdAction::save( this, SLOT(slotFileSave()), actionCollection() );
  (void )KStdAction::quit( this, SLOT(close()), actionCollection() );
   m_recent_files = KStdAction::openRecent(this, SLOT(openURL(const KURL&)), actionCollection());


  (void )new KAction( i18n("Export p&icture ..."), 0,
		      this, SLOT(slotExportPicture()), actionCollection(), "export_picture" );
  m_datasets = new KAction( i18n("Datasets && &Axes ..."), 0,
		      this, SLOT(slotDatasets()), actionCollection(), "edit_datasets" );
  m_objects = new KAction( i18n("&Objects ..."), 0,
		      this, SLOT(slotObjects()), actionCollection(), "edit_objects" );
  (void )new KAction( i18n("&Data ..."), 0,
		      this, SLOT(slotWorksheet()), actionCollection(), "worksheet" );
  (void )new KAction( i18n("&I/O info ..."), 0,
		      this, SLOT(slotViewInfo()), actionCollection(), "view_info");

  KStdAction::keyBindings( this, SLOT( slotConfAccels() ), actionCollection() );
  KStdAction::configureToolbars( this, SLOT( slotEditToolbars() ), actionCollection() );
  m_show_toolbar   = KStdAction::showToolbar( this, SLOT( toggleToolBar() ), actionCollection() );
  m_show_statusbar = KStdAction::showStatusbar( this, SLOT( toggleStatusBar() ), actionCollection() );

  KLibFactory *factory = KLibLoader::self()->factory("libkmatplot");
  if ( factory ) {
     m_kmatplot_part = (KParts::ReadWritePart *)factory->create(this,"kmatplot","KParts::ReadWritePart");

    } else {
     kdFatal() << "Cant find resources:( check if KDEDIR is set correctly ) " << KLibLoader::self()->lastErrorMessage();
     ::exit(-1);
    }


  if ( kmatplot_socket ) kmatplot_socket->setPart( (KMatplotPart *)m_kmatplot_part );
  setCentralWidget( m_kmatplot_part->widget() );
  setXMLFile("kmatplot.rc");
  createGUI( m_kmatplot_part );

  connect( view(), SIGNAL(sigActiveAxesChanged()), this, SLOT(slot_active_axes_changed()) );
  connect( view(), SIGNAL(sigCurrentPageChanged()), this, SLOT(slot_curr_page_changed()) );
  slot_active_axes_changed();
  slot_curr_page_changed();
  readOptions();
 }

//-----------------------------------------------------------------------------//

KMatplotShell::~KMatplotShell()
 {
  //writeSettings();
  //if ( kmatplot_socket ) kmatplot_socket->unregisterPart( part_id );
  delete m_kmatplot_part;
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slot_active_axes_changed()
 {
  if ( view()->activeAxes() ) {
	m_datasets->setEnabled(TRUE);
	} else {
	m_datasets->setEnabled(FALSE);
	}
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slot_curr_page_changed()
 {
  if ( view()->currentPage() ) {
	m_objects->setEnabled(TRUE);
	} else {
	m_objects->setEnabled(FALSE);
	}
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotConfAccels()
{
//  KKeyDialog::configureKeys( actionCollection(), xmlFile() );
}

//-----------------------------------------------------------------------------//

void KMatplotShell::slotEditToolbars()
{
  KEditToolbar dlg( factory() );
  dlg.exec();
 //if ( dlg.exec() ) { applyMainWindowSettings( KGlobal::config(), "MainWindow" ); }
}

//-----------------------------------------------------------------------------//

void KMatplotShell::toggleStatusBar()
{
   if (m_show_statusbar->isChecked())
    statusBar()->show();
    else
    statusBar()->hide();
}

//-----------------------------------------------------------------------------//

void KMatplotShell::toggleToolBar()
{
  if (m_show_toolbar->isChecked())
    toolBar()->show();
    else
    toolBar()->hide();
}

//-----------------------------------------------------------------------------//

bool KMatplotShell::openURL( const KURL& url )
 {
  if (!url.isEmpty()) {
	m_recent_files->addURL( url );
        ((KMatplotPart *)m_kmatplot_part)->setClearFlag( true );
	return m_kmatplot_part->openURL( url );
	}
  return false;
 }

//-----------------------------------------------------------------------------//

bool KMatplotShell::mergeURL( const KURL& url )
 {
  if (!url.isEmpty()) {
	m_recent_files->addURL( url );
        ((KMatplotPart *)m_kmatplot_part)->setClearFlag( false );
	return m_kmatplot_part->openURL( url );
	}
  return false;
 }

//-----------------------------------------------------------------------------//

bool KMatplotShell::saveURL( const KURL& url )
 {
  if (!url.isEmpty()) {
	m_recent_files->addURL( url );
	return m_kmatplot_part->saveAs( url );
	}
  return false;
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotFileOpen()
 {
  KURL url = KFileDialog::getOpenURL( QString::null, 
		i18n("*.kmp|KMatplot files (*.kmp)") );
  openURL( url );
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotFileMerge()
 {
  KURL url = KFileDialog::getOpenURL( QString::null,
		i18n("*.kmp|KMatplot files (*.kmp)") );
  mergeURL( url );
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotFileSaveAs()
 {
  KURL url = KFileDialog::getSaveURL( QString::null, 
		i18n("*.kmp|KMatplot files (*.kmp)") );
  saveURL( url );
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotFileSave()
 {
  if ( m_kmatplot_part->url().isEmpty() ) slotFileSaveAs();
    else m_kmatplot_part->save();
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotExportPicture()
 {
  KSExportPictureDlg dlg( view(), "exportpicture" );
  slotSetStatusBarText(tr("Please wait. Exporting ..."));
  dlg.exec();
  slotSetStatusBarText(tr("Done."));
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotDatasets()
 {
 KSDeleteDlg d( workbook(), view()->activeAxes() );
 d.exec();
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotObjects()
 {
  KSObjectTreeDlg *o = new KSObjectTreeDlg( dynamic_cast<KMatplotPart *>(m_kmatplot_part), NULL );
  o->show();
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotWorksheet()
 {
  KSWorksheetDlg *w = new KSWorksheetDlg( workbook(), view(), this, 0 );
  w->show();
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::slotViewInfo()
 {
  QString info;
  info += i18n(" Application: " ) + QString::number(kmatplot_socket->appNumber()) + i18n(" ( see 'ksetapp' command ) \n");
  info += i18n(" Active axes: " ) + QString::number(kmatplot_socket->axesId(view()->activeAxes())) + i18n(" ( see 'ksetaxes' command ) ");
  info += "\n";
  info += i18n(" Socket name: " ) + (const char *)kmatplot_socket->socketName();
  QMessageBox::information( this, i18n("I/O Info"), info );
 }

//-----------------------------------------------------------------------------//

QSPlotView *KMatplotShell::view()
 {
  return dynamic_cast<KMatplotPart *>(m_kmatplot_part)->view();
 }

//-----------------------------------------------------------------------------//

KSWorkbook *KMatplotShell::workbook()
 {
  return dynamic_cast<KMatplotPart *>(m_kmatplot_part)->workbook();
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::readProperties( KConfig* config )
{
  config->setGroup("General");

  m_show_toolbar->setChecked(config->readBoolEntry("ShowToolBar", true));
  toggleToolBar();

  m_show_statusbar->setChecked (config->readBoolEntry ("ShowStatusBar", true));
  toggleStatusBar();


  m_recent_files->loadEntries(config);
}

//-----------------------------------------------------------------------------//

void KMatplotShell::saveProperties( KConfig* config )
{
  config->setGroup( "General" );

  config->writeEntry ("ShowToolBar", m_show_toolbar->isChecked());
  config->writeEntry ("ShowStatusBar", m_show_statusbar->isChecked());


  m_recent_files->saveEntries(config);

  config->sync();
}

//-----------------------------------------------------------------------------//

void KMatplotShell::readOptions()
 {
  KConfig *config = kapp->config();
  readProperties( config );
 }

//-----------------------------------------------------------------------------//

void KMatplotShell::saveOptions()
 {
  KConfig *config = kapp->config();
  saveProperties( config );
 }

//-----------------------------------------------------------------------------//

bool KMatplotShell::queryExit()
 {
  saveOptions();
  return TRUE;
 }

