/***************************************************************************
                                 ksexportdlg.cpp
                             -------------------
    begin                : Thu Feb 10 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksexportdlg.h"
#include "ksmatrixio.h"
#include "ksplotchannels.h"
#include <kfiledialog.h>
#include <qregexp.h>
#include <klocale.h>

QString KSExportDlg::m_file_name;

//-----------------------------------------------------------//

KSExportDlg::KSExportDlg(QSData *data, QWidget *parent, const char *name)
: QDialog(parent,name,true)
 {
  initDialog();
  setCaption(i18n("Export ASCII data"));
  delete clist;
  m_chan_list = new KSChannelList( QGroupBox_1 );
  m_chan_list->setGeometry(10,25,305,240);
  m_chan_list->addColumn( i18n(" Channel name"), 145 );
  m_chan_list->addColumn( i18n(" RowsxCols"), 75 );
  m_chan_list->addColumn( i18n(" Action "), 65 );
  m_chan_list->setDataObject( data );

  filename->setText( m_file_name );

  connect( bbrowse, SIGNAL(clicked()), this, SLOT(chooseFile()) );
  connect( bexport, SIGNAL(clicked()), this, SLOT(markChannel()) );
  connect( bok,     SIGNAL(clicked()), this, SLOT(accept()) );
  connect( bcancel, SIGNAL(clicked()), this, SLOT(reject()) );
 }

//-----------------------------------------------------------//

KSExportDlg::~KSExportDlg()
 {
 }

//-----------------------------------------------------------//

void KSExportDlg::markChannel()
 {
  KSChannelListItem *citem = m_chan_list->selected();
  if ( citem && citem->isValid() ) {
	if ( citem->text(2).isEmpty() ) citem->setText( 2, i18n("Export") );
	                           else citem->setText( 2, QString::null  );
	}
 }

//-----------------------------------------------------------//

void KSExportDlg::chooseFile()
 {
  QString result = KFileDialog::getSaveFileName(QDir::currentDirPath(),
                                        i18n("*| All files (*)"),
					this,
					i18n("Export to..."));
  if ( !result.isEmpty() ) {
        m_file_name = result;
        filename->setText( m_file_name );
       }
 }

//-----------------------------------------------------------//

void KSExportDlg::saveAll()
 {
  m_file_name = filename->text().stripWhiteSpace();
  if ( m_file_name.isEmpty() ) return;

  KSMatrixIO io( m_file_name, IO_WriteOnly );
  KSChannelListItem *citem = m_chan_list->first();
  while ( citem ) {
         if ( citem->isValid() && !citem->text(2).isEmpty() ) {
                 QString mname = QString("c") +
				 m_chan_list->channelNameFactory(citem->dataObject(),citem->channel()).lower().stripWhiteSpace()+
				 QString::number(citem->channel());
                 
		 mname.replace( QRegExp("\\s+" ), "_" );
		 mname.replace( QRegExp("\\-+" ), "_" );
		 mname.replace( QRegExp("\\.+" ), "_" );
                 QSMatrix *m = citem->dataObject()->matrix( citem->channel() );
                 if ( m ) io.saveMatrix( m, mname, "ASCII" );
                }

         // process the next item
	 citem = citem->next();
        }
 }

//-----------------------------------------------------------//

