/***************************************************************************
                               ksimportdlg.cpp
                             -------------------                                         
    begin                : 01-January-2000
    copyright            : (C) 2000 by Kamil Dobkowski                         
    email                : kamildobk@poczta.onet.pl                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "ksimportdlg.h"
#include "part/ksmatrix.h"
#include "part/kscommands.h"
#include <qprogressbar.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qpixmap.h>
#include <kfiledialog.h>
#include <klocale.h>
#include "ksplotchannels.h"
#include "ksexception.h"
#include "ksmatrixio.h"

QString KSImportDlg::m_file_name;

//-----------------------------------------------------------//

KSImportDlg::KSImportDlg( KSWorkbook *workbook, QSData *data, QWidget *parent, const char *name)
 : QDialog(parent,name,true)
 {
  setCaption( i18n("Import") );
  initDialog();
  m_workbook = workbook;

  // init the matrix list
  mlist->addColumn( i18n(" Name"), 115 );
  mlist->addColumn( i18n("RowsCols"),  65 );
  mlist->setSorting(-1);
  mlist->setAllColumnsShowFocus( true );
  m_empty_item = new QListViewItem( mlist, i18n("(Don't change)") );

  // init the channel list
  delete clist;
  m_chan_list = new KSChannelList( this );
  m_chan_list->setGeometry(25,80,225,270);
  m_chan_list->setDataObject( data );
  m_chan_list->addColumn( i18n("Name"),   100 );
  m_chan_list->addColumn( i18n("RowsCols"),  50 );
  m_chan_list->addColumn( i18n("Load"),  50 );

  // handle some signals
  connect( bload,   SIGNAL(clicked()), this, SLOT(accept()) );
  connect( bcancel, SIGNAL(clicked()), this, SLOT(reject()) );
  connect( mlist,   SIGNAL(selectionChanged()), this, SLOT(matrixSelected()) );
  connect( filename, SIGNAL(returnPressed()), this, SLOT(changedFile()) );
  connect( bbrowse,  SIGNAL(clicked()), this, SLOT(chooseFile()) );

  connect( m_chan_list, SIGNAL(selectionChanged()), this, SLOT(channelSelected()) );
  connect( m_chan_list, SIGNAL(sigShowText(const QString&)), this, SLOT(showDescription(const QString&)) );

  descriptionl->setText("Matlab's MAT 1.0 and ASCII plain text ( much like Octave's text files ) formats are supported.");
  filename->setText( m_file_name );
  refreshFileContents();
 }

//-----------------------------------------------------------//

KSImportDlg::~KSImportDlg()
 {
 }

//-----------------------------------------------------------//

void KSImportDlg::matrixSelected()
 {
  KSChannelListItem *citem = m_chan_list->selected();
  QListViewItem *mitem = mlist->selectedItem();
  if ( citem && mitem && citem->isValid() )
	 if ( mitem != m_empty_item ) citem->setText( 2, mitem->text(0) );
				 else citem->setText( 2, QString::null  );
 }

//-----------------------------------------------------------//

void KSImportDlg::chooseFile()
 {
  QString newfile = KFileDialog::getOpenFileName(QDir::homeDirPath(),
                     i18n("*|All files (*)\n*.txt|ASCII files (*.txt)\n*.mat|MATLAB 1.0 files (*.mat)"),
		     this,  i18n("Import File...") );
  if ( !newfile.isEmpty() ) {
         m_file_name = newfile;
         filename->setText( m_file_name );
         refreshFileContents();
        }
 }

//-----------------------------------------------------------//

void KSImportDlg::changedFile()
 {
  m_file_name = filename->text();
  refreshFileContents();
 }

//-----------------------------------------------------------//

void KSImportDlg::refreshFileContents()
 {
  if ( m_file_name.isEmpty() ) return;

  mlist->clear();
  m_empty_item = new QListViewItem( mlist, i18n("(Don't change)") );

  KSHeadersList *headers;
  try {
       KSMatrixIO io(m_file_name);
       headers = io.loadHeaders();
      }
  catch( KSException& e ) {
         QMessageBox::warning ( this, i18n("Error"), e.msg, QMessageBox::Ok, 0, 0 );
         return;
        }
  catch(...) {
         QMessageBox::warning ( this, i18n("Error"), i18n("Unknown error !"), QMessageBox::Ok, 0, 0 );
         return;
        }
  KSHeadersList::iterator curr = headers->end();
  KSHeadersList::iterator last = headers->begin();

  while( curr != last ) {
         curr--;
         QString size = QString::null;
         int col = (*curr)->cols();
         int row = (*curr)->rows();
         if ( row && col ) size.sprintf("%dx%d ", row, col );

         (void) new QListViewItem( mlist, (*curr)->title(), size );
        }
  delete headers;
  mlist->triggerUpdate();
 }

//-----------------------------------------------------------//

void KSImportDlg::showDescription( const QString& text )
 {
  descriptionl->setText( text );
 }

//-----------------------------------------------------------//

void KSImportDlg::channelSelected()
 {
 }


//-----------------------------------------------------------//

void KSImportDlg::loadAll()
 {
  KSMatrixIO io(m_file_name);
  KSChannelListItem *citem = m_chan_list->first();
  while ( citem ) {
         // load matrix
         if ( citem->isValid() && !citem->text(2).isEmpty() ) {
		 KSMatrix *new_matrix = io.loadMatrix( citem->text(2) );
                 if ( new_matrix ) m_workbook->execute( new KSCmdSetData(citem->dataObject(),citem->channel(),new_matrix) );
                }

         // check if there is any matrix set for the next channel
         citem = citem->next();
        }
 }

/*
                 if ( new_matrix ) {
			new_matrix->setDataObject( citem->dataObject(), citem->channel() );
			citem->dataObject()->setMatrix( citem->channel(), new_matrix );
			}
*/


//-----------------------------------------------------------//




























