/***************************************************************************
                                ksmatrixio.cpp
                             -------------------                                         
    begin                : 01-January-2000
    copyright            : (C) 2000 by Kamil Dobkowski                         
    email                : kamildobk@poczta.onet.pl                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "ksmatrixio.h"
#include "ksmatrixiohandler.h"
#include "ksmatrixmat.h"
#include "ksmatrixascii.h"

KSHandlersList KSMatrixIO::hlist;
bool KSMatrixIO::initialized = false;

//-------------------------------------------------------------------------//

KSMatrixIO::KSMatrixIO( const QString& filename, int mode  )
 {
  if ( !initialized ) {	
			 // initialize a file format list
  		 addHandler( new KSMatrixMAT()   );
       addHandler( new KSMatrixASCII() );
			 initialized = true;
			}
  if ( !filename.isNull() ) openFile( filename, mode );
 }

//-------------------------------------------------------------------------//

KSMatrixIO::~KSMatrixIO()
 {
  closeFile();
 }

//-------------------------------------------------------------------------//

void KSMatrixIO::setDefaultHandler()
 {
  KSMatrixIOHandler *dhandler;

  if ( !file.isReadable() ) { dformat = QString(); return; }
  QDataStream s( &file );

  // try to find a default handler
  try {
       dhandler = findHandler( s, file.name() );
      }
  catch ( KSException& e ) {
       dhandler= NULL; // no auto format detect
      }

  if ( dhandler ) dformat = dhandler->format();
             else dformat = QString();
 }

//-------------------------------------------------------------------------//

void KSMatrixIO::openFile( const QString& filename, int mode )
 {
  if ( filename.isNull() ) throw KSException( KSException::NullArgument );

  // close a previously opened file.
  closeFile();

  file.setName( filename );
  if( !file.open(mode) ) throw KSException(KSException::FileError);
  setDefaultHandler();

  file.resetStatus();
  file.reset();
 }

//-------------------------------------------------------------------------//

void KSMatrixIO::closeFile()
 {
  file.close();
  file.setName(QString());
  dformat = QString();
 }

//-------------------------------------------------------------------------//

KSHeadersList *KSMatrixIO::loadHeaders( const QString& format )
 {
  KSMatrixIOHandler *handler;

  QDataStream s(&file);
  if ( !format.isNull() ) handler = findHandler( format );
     else if ( !dformat.isNull() ) handler = findHandler( dformat );
                              else handler = findHandler( s, file.name() );
  file.resetStatus(); file.reset();

  return handler->headers(s);
 }

//-------------------------------------------------------------------------//

KSMatrix *KSMatrixIO::loadMatrix( const QString& matrixname, const QString& format )
 {
  KSMatrixIOHandler *handler;
  emit progress(0);
  QDataStream s(&file);
  if ( !format.isNull() ) handler = findHandler( format );
     else if ( !dformat.isNull() ) handler = findHandler( dformat );
                              else handler = findHandler( s, file.name() );

  file.resetStatus(); file.reset();
  KSMatrix *result = handler->load( s, matrixname );
  emit progress(100);
  return result;
 }

//-------------------------------------------------------------------------//

void KSMatrixIO::saveMatrix( QSMatrix *m, const QString& matrixname, const QString& format )
 {
  emit progress(0);
  QDataStream s(&file);
  KSMatrixIOHandler *handler = findHandler( format );
  handler->save( s, m, matrixname );
  emit progress(100);
 }

//-------------------------------------------------------------------------//

KSMatrixIOHandler *KSMatrixIO::findHandler( QDataStream& s, const QString&  )
 {
  KSHandlersList::iterator curr = hlist.begin();
  KSHandlersList::iterator last = hlist.end();

  s.device()->resetStatus(); s.device()->reset();
  while ( curr != last ) {
         try {
           if ( (*curr)->check(s) ) break;
          }
         catch(...) {
           s.device()->resetStatus();
           s.device()->reset();
           throw;
          }
         s.device()->resetStatus(); s.device()->reset();
         curr++;
        }

  if ( curr != last ) return (*curr);
  throw( KSException(KSException::SearchFailed) );
 }

//-------------------------------------------------------------------------//

KSMatrixIOHandler *KSMatrixIO::findHandler( const QString& format )
 {
  KSHandlersList::iterator curr = hlist.begin();
  KSHandlersList::iterator last = hlist.end();

  while ( curr != last ) {
         if ( (*curr)->format() == format ) break;
         curr++;
        }
  if ( curr != last ) return (*curr);

  throw( KSException( KSException::SearchFailed ) );
 }

//-------------------------------------------------------------------------//

void KSMatrixIO::addHandler( KSMatrixIOHandler *handler )
 {
  if ( handler ) hlist.push_back( handler );
 }

//-------------------------------------------------------------------------//

int KSMatrixIO::removeHandlers( const QString& format )
 {
  KSHandlersList::iterator curr = hlist.begin();
  KSHandlersList::iterator last = hlist.end();

  int result = 0;
  while( curr != last ) {
         if ( (*curr)->format() == format ) {
                KSHandlersList::iterator last = curr; curr++;
                hlist.erase( last ); result ++;
               } else curr ++;
        }
  return result;
 }

//-------------------------------------------------------------------------//

void KSMatrixIO::progressSlot(int percent,bool&)
 {
  emit progress(percent);
 }



























