/***************************************************************************
                                 ksmmatrixpanel.cpp
                             -------------------
    begin
    copyright            : (C) 2001 by Kamil Dobkowski
    email                : kamildbk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksmatrixpanel.h"
#include "ksplotchannels.h"
#include "part/kscommands.h"
#include "part/ksmatrix.h"
#include "part/ksglobalmatrixlist.h"
#include "part/ksdatasymbolfactory.h"
#include "widgets/qsdata.h"
#include "formula/mpformula.h"
#include <qlineedit.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qevent.h>
#include <qapplication.h>
#include <qmultilineedit.h>
#include <qmessagebox.h>
#include<iostream>

// All constructors should take QSData and channel instead just KSMatrix*
KSMatrixPanel::KSMatrixPanel( KSWorkbook *workbook, KSMatrix *matrix, QWidget *parent )
:KSMatrixPanelInterf( parent ), KSCfgPanelInterf()
 {
  m_workbook = workbook;
  m_matrix = matrix;
  m_data_object = matrix->dataObject();
  m_channel = matrix->channel();
  updateContents();
 }

//----------------------------------------------------------------------------------------//

KSMatrixPanel::~KSMatrixPanel()
 {
 }

//----------------------------------------------------------------------------------------//

void KSMatrixPanel::apply()
 {
  EType new_type = EDouble;
  if ( typeUChar->isChecked()  ) new_type = EUChar;
  else
  if ( typeShort->isChecked()  ) new_type = EShort;
  else
  if ( typeUShort->isChecked() ) new_type = EUShort;
  else
  if ( typeLong->isChecked()   ) new_type = ELong;
  else
  if ( typeFloat->isChecked()  ) new_type = EFloat;
  else
  if ( typeDouble->isChecked() ) new_type = EDouble;

  int new_rows = matrixRows->value();
  int new_cols = matrixCols->value();

  if ( isModified() ) {
	KSMatrix *new_matrix = KSMatrix::create(new_type);
        new_matrix->resize(new_rows,new_cols);
	new_matrix->setTitle( matrixName->text() );
        for( int row=0; row<new_matrix->rows(); row++ )
		for( int col=0; col<new_matrix->cols(); col++ ) {
			if ( row < m_matrix->rows() &&
			     col < m_matrix->cols()  ) {
				new_matrix->setValue( row, col, m_matrix->value(row,col) );
				}
			}
        m_matrix = new_matrix;
        m_workbook->execute( new KSCmdSetData(m_data_object,m_channel,new_matrix) );	
	}
 }



//----------------------------------------------------------------------------------------//

bool KSMatrixPanel::isModified()
 {
  EType new_type = EDouble;
  if ( typeUChar->isChecked() ) new_type = EUChar;
  else
  if ( typeShort->isChecked() ) new_type = EShort;
  else
  if ( typeUShort->isChecked() ) new_type = EUShort;
  else
  if ( typeLong->isChecked() ) new_type = ELong;
  else
  if ( typeFloat->isChecked() ) new_type = EFloat;
  else
  if ( typeDouble->isChecked() ) new_type = EDouble;

  int new_rows = matrixRows->value();
  int new_cols = matrixCols->value();

  if ( m_matrix &&
       m_matrix->title() != matrixName->text() ||
       new_rows != m_matrix->rows() ||
       new_cols != m_matrix->cols() ||
       new_type != m_matrix->type()  ) return true;

  return false;
 }

//----------------------------------------------------------------------------------------//

QWidget *KSMatrixPanel::widget()
 {
  return this;
 }

//----------------------------------------------------------------------------------------//

void KSMatrixPanel::updateContents()
 {
  matrixRows->setValue( m_matrix->rows() );
  matrixCols->setValue( m_matrix->cols() );
  matrixName->setText( m_matrix->title() );
  switch( m_matrix->type() ) {
	case EUChar: typeUChar->setChecked( true ); break;
	case EShort: typeShort->setChecked( true ); break;
	case EUShort:typeUShort->setChecked( true ); break;
	case ELong:  typeLong->setChecked( true ); break;
	case EFloat: typeFloat->setChecked( true ); break;
	case EDouble:typeDouble->setChecked( true ); break;
	}
 }




//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//




KSMatrixRefPanel::KSMatrixRefPanel( KSWorkbook *workbook, KSMatrixWorksheetCellRange *matrix, QWidget *parent )
:KSMatrixRefPanelInterf( parent ), KSCfgPanelInterf()
 {
  m_workbook = workbook;
  m_matrix = matrix;
  m_data_object = matrix->dataObject();
  m_channel = matrix->channel();
  m_chan_list = NULL;
  QApplication::postEvent( this, new QCustomEvent(QEvent::User) );
 }

//----------------------------------------------------------------------------------------//

KSMatrixRefPanel::~KSMatrixRefPanel()
 {
 }

//----------------------------------------------------------------------------------------//

void KSMatrixRefPanel::customEvent( QCustomEvent *e )
// I couldn't create m_chan_list in the constructor because the constructor is
// called in signal KSSheetList::sigDataChanged and m_chan_list connects
// to the same signal from the same object during its initialization - it seems as a little bit
// confusing to the QT.
 {
  KSMatrixRefPanelInterf::customEvent(e);
  m_chan_list = new KSChannelList( channelListPlace );
  m_chan_list->setGeometry( channelListPlace->rect() );
  m_chan_list->addColumn( i18n("Name"),   150 );
  m_chan_list->addColumn( i18n(""),  35 );
  m_chan_list->setDataObject( m_workbook->sheets(), false, 0 );
  m_chan_list->setRootIsDecorated( false );
  m_chan_list->show();
  updateContents();
 }

//----------------------------------------------------------------------------------------//

void KSMatrixRefPanel::apply()
 {
  if ( !m_chan_list ) return;

  KSChannelListItem *citem = m_chan_list->selected();
  QSData *new_object = citem ? citem->dataObject() : NULL;
  int new_channel = citem ? citem->channel() : NULL;
  int new_col_from = matrixColFrom->text().toInt();
  int new_col_to   = matrixColTo->text().toInt();
  int new_row_from = matrixRowFrom->text().toInt();
  int new_row_to   = matrixRowTo->text().toInt();
  int new_col_step = matrixColStep->text().toInt();
  int new_row_step = matrixRowStep->text().toInt();
  bool new_transposition = matrixTransposition->isChecked();
  QString new_name = matrixName->text();

  if ( new_channel < 0 ) new_object = NULL;

  if ( isModified() ) {
	KSMatrixWorksheetCellRange *new_matrix = new KSMatrixWorksheetCellRange(m_workbook);
	new_matrix->setWorksheet( m_workbook->sheets()->childIndex(new_object) );
	new_matrix->setRowFrom( new_row_from );
	new_matrix->setRowStep( new_row_step );
	new_matrix->setRowTo( new_row_to );
	new_matrix->setColFrom( new_col_from );
	new_matrix->setColStep( new_col_step );
	new_matrix->setColTo( new_col_to );
	new_matrix->setTransposition( new_transposition );	
	new_matrix->setTitle( new_name );
        m_matrix = new_matrix;
        m_workbook->execute( new KSCmdSetData(m_data_object,m_channel,new_matrix) );
       }
 }

//----------------------------------------------------------------------------------------//

QWidget *KSMatrixRefPanel::widget()
 {
  return this;
 }

//----------------------------------------------------------------------------------------//

bool KSMatrixRefPanel::isModified()
 {
  if ( !m_chan_list ) return false;

  KSChannelListItem *citem = m_chan_list->selected();
  QSData *new_object = citem ? citem->dataObject() : NULL;
  int new_channel = citem ? citem->channel() : NULL;
  int new_col_from = matrixColFrom->text().toInt();
  int new_col_to   = matrixColTo->text().toInt();
  int new_row_from = matrixRowFrom->text().toInt();
  int new_row_to   = matrixRowTo->text().toInt();
  int new_col_step = matrixColStep->text().toInt();
  int new_row_step = matrixRowStep->text().toInt();
  bool new_transposition = matrixTransposition->isChecked();
  QString new_name = matrixName->text();

  if ( new_channel < 0 ) new_object = NULL;

  if ( m_matrix->refObject() != new_object ||
       m_matrix->refChannel() != new_channel ||
       m_matrix->transposition() != new_transposition ||
       m_matrix->title() != new_name ||
       new_row_from != m_matrix->refRowFrom() ||
       new_row_to   != m_matrix->refRowTo() ||
       new_col_from != m_matrix->refColFrom() ||
       new_col_to   != m_matrix->refColTo() ||
       new_row_step != m_matrix->refRowStep() ||
       new_col_step != m_matrix->refColStep() ) return true;

  return false;
 }

//----------------------------------------------------------------------------------------//

void KSMatrixRefPanel::updateContents()
 {
  matrixName->setText( m_matrix->title() );
  matrixColFrom->setText( QString::number(m_matrix->refColFrom()) );
  matrixColTo->setText( QString::number(m_matrix->refColTo()) );
  matrixRowFrom->setText( QString::number(m_matrix->refRowFrom()) );
  matrixRowTo->setText( QString::number(m_matrix->refRowTo()) );
  matrixColStep->setText( QString::number(m_matrix->refColStep()) );
  matrixRowStep->setText( QString::number(m_matrix->refRowStep()) );
  matrixTransposition->setChecked( m_matrix->transposition() );
  m_chan_list->select( m_matrix->refObject(), m_matrix->refChannel() );
 }





//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//










KSMatrixStringPanel::KSMatrixStringPanel( KSWorkbook *workbook, KSMatrixString *matrix, QWidget *parent )
:KSMatrixStringPanelInterf( parent ), KSCfgPanelInterf()
 {
  m_workbook = workbook;
  m_matrix = matrix;
  m_data_object = matrix->dataObject();
  m_channel = matrix->channel();
  updateContents();
 }

//----------------------------------------------------------------------------------------//

KSMatrixStringPanel::~KSMatrixStringPanel()
 {
 }

//----------------------------------------------------------------------------------------//

void KSMatrixStringPanel::apply()
 {
  int new_rows = matrixRows->value();
  int new_cols = matrixCols->value();

  if ( isModified() ) {
        KSMatrixString *new_matrix = new KSMatrixString();
	new_matrix->resize(new_rows,new_cols);
        new_matrix->setTitle( matrixName->text() );
        for( int row=0; row<new_matrix->rows(); row++ )
		for( int col=0; col<new_matrix->cols(); col++ ) {
			if ( row < m_matrix->rows() &&
			     col < m_matrix->cols()  ) {
				new_matrix->setString( row, col, m_matrix->string(row,col) );
				}
			}
	m_matrix = new_matrix;
	m_workbook->execute( new KSCmdSetData(m_data_object,m_channel,new_matrix) );	
	}
	
 }

//----------------------------------------------------------------------------------------//

QWidget *KSMatrixStringPanel::widget()
 {
  return this;
 }

//----------------------------------------------------------------------------------------//

bool KSMatrixStringPanel::isModified()
 {
  int new_rows = matrixRows->value();
  int new_cols = matrixCols->value();

  if ( m_matrix->title() != matrixName->text() ||
       new_rows != m_matrix->rows() ||
       new_cols != m_matrix->cols()  ) return true;

  return false;
 }

//----------------------------------------------------------------------------------------//

void KSMatrixStringPanel::updateContents()
 {
  matrixRows->setValue( m_matrix->rows() );
  matrixCols->setValue( m_matrix->cols() );
  matrixName->setText( m_matrix->title() );
 }






//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//
//----------------------------------------------------------------------------------------//










KSMatrixFormulaPanel::KSMatrixFormulaPanel( KSWorkbook *workbook, KSMatrixFormula *matrix, QWidget *parent )
:KSMatrixFormulaPanelInterf( parent ), KSCfgPanelInterf()
 {
  m_workbook = workbook;
  m_matrix = matrix;
  m_data_object = matrix->dataObject();
  m_channel = matrix->channel();
  updateContents();
 }

//----------------------------------------------------------------------------------------//

KSMatrixFormulaPanel::~KSMatrixFormulaPanel()
 {
 }

//----------------------------------------------------------------------------------------//

void KSMatrixFormulaPanel::apply()
 {
  KSMatrixFormula *new_matrix = new KSMatrixFormula();
  new_matrix->setTitle( matrixName->text() );
  new_matrix->setFormula( matrixFormula->text() );
  new_matrix->setTransposition( m_matrix->transposition() );
  MPFactoryList factory( new KSDataSymbolFactory( m_workbook, m_matrix->dataObject() ) );
  MPFormulaError error;
  if ( !new_matrix->init( error, &factory ) ) {
	QMessageBox::critical( NULL, tr("Error"), error.message(), QMessageBox::Ok, 0 );
	}
  m_matrix = new_matrix;
  m_workbook->execute( new KSCmdSetData(m_data_object,m_channel,new_matrix) );		
 }

//----------------------------------------------------------------------------------------//

QWidget *KSMatrixFormulaPanel::widget()
 {
  return this;
 }

//----------------------------------------------------------------------------------------//

bool KSMatrixFormulaPanel::isModified()
 {
  if ( m_matrix->title()   != matrixName->text() ||
       m_matrix->formula() != matrixFormula->text() ) return true;
  return false;
 }

//----------------------------------------------------------------------------------------//

void KSMatrixFormulaPanel::updateContents()
 {
  matrixFormula->setText( m_matrix->formula() );
  matrixName->setText( m_matrix->title() );
 }

