/***************************************************************************
                                ksmatrixpanel.h
                             -------------------
    copyright            : (C) 2001 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSMATRIXPANEL_H
#define KSMATRIXPANEL_H

#include <klocale.h>
#include "ksmatrixpanelinterf.h"
#include "ksmatrixrefpanelinterf.h"
#include "ksmatrixstringpanelinterf.h"
#include "ksmatrixformulapanelinterf.h"
#include "part/ksworkbook.h"
#include"part/ksglobalmatrixlist.h"

class KSMatrix;
class KSMatrixRef;
class KSMatrixColRef;
class KSMatrixString;
class KSMatrixFormula;
class KSChannelList;

//------------------------------------------------------------------------//

/**
  * Base interface for matrix config panels. It is an interface and not a base class
  * because QtDesigner cant generate dialogs which inherits custom classes.
  */
class KSCfgPanelInterf {
  public:
   KSCfgPanelInterf() {}
   virtual ~KSCfgPanelInterf() {}
   virtual void updateContents() {}
   virtual QSMatrix *editedMatrix() { return NULL; }
   virtual QWidget *widget() = 0;
 };

//------------------------------------------------------------------------//

/**
  *@author Kamil Dobkowski
  */
class KSMatrixPanel : public KSMatrixPanelInterf, public KSCfgPanelInterf  {
    Q_OBJECT
  public:
	KSMatrixPanel( KSWorkbook *workbook, KSMatrix *matrix, QWidget *parent );
	virtual ~KSMatrixPanel();
        virtual bool isModified();
        virtual QWidget *widget();
	virtual void updateContents();
	virtual QSMatrix *editedMatrix() { return m_matrix; }
  public slots:
   	virtual void apply();	
  private:
	KSWorkbook *m_workbook;
	KSMatrix *m_matrix;
	QSData *m_data_object;
	int m_channel;
};

//------------------------------------------------------------------------//


class KSMatrixRefPanel : public KSMatrixRefPanelInterf, public KSCfgPanelInterf {
	Q_OBJECT
   public:
	 KSMatrixRefPanel( KSWorkbook *workbook, KSMatrixWorksheetCellRange *matrix, QWidget *parent );
	virtual ~KSMatrixRefPanel();
        virtual bool isModified();
        virtual QWidget *widget();
	virtual void updateContents();
	virtual QSMatrix *editedMatrix() { return m_matrix; }
  public slots:
   	virtual void apply();	
  protected:
	virtual void customEvent( QCustomEvent *e );
  private:
	KSWorkbook   *m_workbook;
	KSMatrixWorksheetCellRange  *m_matrix;
	QSData *m_data_object;
	int m_channel;
	KSChannelList *m_chan_list;
};

//------------------------------------------------------------------------//


class KSMatrixStringPanel : public KSMatrixStringPanelInterf, public KSCfgPanelInterf {
	Q_OBJECT
   public:
	KSMatrixStringPanel( KSWorkbook *workbook, KSMatrixString *matrix, QWidget *parent );
	virtual ~KSMatrixStringPanel();
        virtual bool isModified();
        virtual QWidget *widget();
	virtual void updateContents();
 	virtual QSMatrix *editedMatrix() { return m_matrix; }
  public slots:
   	virtual void apply();
  private:
	KSWorkbook *m_workbook;
	KSMatrixString  *m_matrix;
	QSData *m_data_object;
	int m_channel;
};

//------------------------------------------------------------------------//


class KSMatrixFormulaPanel : public KSMatrixFormulaPanelInterf, public KSCfgPanelInterf {
	Q_OBJECT
   public:
	KSMatrixFormulaPanel( KSWorkbook *workbook, KSMatrixFormula *matrix, QWidget *parent );
	virtual ~KSMatrixFormulaPanel();
        virtual bool isModified();
        virtual QWidget *widget();
	virtual void updateContents();
 	virtual QSMatrix *editedMatrix() { return m_matrix; }
  public slots:
   	virtual void apply();
  private:
	KSWorkbook *m_workbook;
	KSMatrixFormula *m_matrix;
	QSData *m_data_object;
	int m_channel;
};

#endif
