/***************************************************************************
                                 ksplotchannels.cpp                             
                             -------------------                                         
    begin                : 01-January-2000
    copyright            : (C) 2000 by Kamil Dobkowski                         
    email                : kamildobk@poczta.onet.pl                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include"ksplotchannels.h"
#include"part/ksmatrix.h"
#include"part/ksglobalmatrixlist.h"
#include"ksmatrixpanel.h"
#include"ksmatrixeditor.h"
#include"widgets/qsaxes2d.h"
#include"widgets/qsaxes3d.h"
#include"widgets/qscurve.h"
#include"widgets/qscontour.h"
#include"widgets/qsimage.h"
#include"widgets/qssurface.h"
#include"widgets/qsfigure.h"

#define LINE_INVISIBLE "0"
#define LINE_SOLID "1"
#define LINE_DASH "2"
#define LINE_DASHDOT "4"
#define LINE_DOT "3"
#define LINE_DASHDOTDOT "5"


#define FILL_TRANSPARENT "0"
#define FILL_SOLID "1"
#define FILL_HORIZ "9"
#define FILL_VERT "10"
#define FILL_CROSS "11"
#define FILL_BDIAG "12"
#define FILL_FDIAG "13"
#define FILL_DIAGCROSS "14"
#define FILL_HALF "6"
//-------------------------------------------------------------//
// TODO: i18n

static const char *caxis[] = {
        QT_TR_NOOP( "0. Tic marks"  ),
        QT_TR_NOOP( "1. Line styles"  ),
        QT_TR_NOOP( "2. Font styles"  ),
        QT_TR_NOOP( "3. Fill styles"  ),
       };

static const char *daxis[] = {
        QT_TR_NOOP( "<b>Labels - x axis:</b> <i></i><br>"
        " Position of user-defined tic marks on the <b>X</b> axis." ),

	QT_TR_NOOP( " Line styles column vector ( M-1 rows ) "
	"Invisible "LINE_INVISIBLE", "
	"Solid "LINE_SOLID", "
	"Dashed "LINE_DASH", "
	"Dotted "LINE_DOT", "
	"Dashed-Dotted "LINE_DASHDOT", "
	"Dashed-Dotted-Dotted "LINE_DASHDOTDOT", "),

	QT_TR_NOOP( " Font styles "),

	QT_TR_NOOP( " Fill styles column vector ( M-1 rows ) "
	"Transparent "FILL_TRANSPARENT", "
	"Solid "FILL_SOLID", "
	"Horizontal "FILL_HORIZ", "
	"Vertical "FILL_VERT", "
	"Crossed "FILL_CROSS", "
	"BDiagonal "FILL_BDIAG", "
	"FDiagonal "FILL_FDIAG", "
	"CrossDiagonal "FILL_DIAGCROSS", "
	"Half "FILL_HALF"<BR>"),	
       };

//-------------------------------------------------------------//
// TODO: i18n

static const char *caxes2d[] = {
        QT_TR_NOOP( "0. Labels - x axis"  ),
        QT_TR_NOOP( "1. Labels - y axis"  ),
        QT_TR_NOOP( "2. Labels - v axis"  ),
        QT_TR_NOOP( "3. Labels - z axis"  ),
        QT_TR_NOOP( "4. Labels - x2 axis" ),
        QT_TR_NOOP( "5. Labels - y2 axis" )
       };

static const char *daxes2d[] = {
        QT_TR_NOOP( "<b>Labels - x axis:</b> <i>( M columns )</i><br>"
        " Position of user-defined labels on the <b>X</b> axis." ),

        QT_TR_NOOP( "<b>Labels - y axis:</b> <i>( N columns )</i><br>"
        " Position of user-defined labels on the <b>Y</b> axis." ),

        QT_TR_NOOP( "<b>Labels - v axis:</b> <i>( Q columns )</i><br>"
        " Position of user-defined labels on the ( invisible ) <b>V</b> axis." ),

        QT_TR_NOOP( "<b>Labels - z axis:</b> <i>( P columns )</i><br>"
        " Position of user-defined labels on the ( invisible ) <b>Z</b> axis." ),

        QT_TR_NOOP( "<b>Labels - x axis:</b> <i>( R columns )</i><br>"
        " Position of user-defined labels on the second <b>X</b> axis." ),

        QT_TR_NOOP( "<b>Labels - y axis:</b> <i>( S columns )</i><br>"
        " Position of user-defined labels on the second <b>Y</b> axis." )
       };

//-------------------------------------------------------------//


static const char *caxes3d[] = {
        QT_TR_NOOP( "0. Labels - x axis"  ),
        QT_TR_NOOP( "1. Labels - y axis"  ),
        QT_TR_NOOP( "2. Labels - v axis"  ),
        QT_TR_NOOP( "3. Labels - z axis"  ),
        QT_TR_NOOP( "4. Labels - x2 axis" ),
        QT_TR_NOOP( "5. Labels - y2 axis" ),
        QT_TR_NOOP( "6. Labels - v2 axis" ),
        QT_TR_NOOP( "7. Labels - z2 axis" )
       };

static const char *daxes3d[] = {
        QT_TR_NOOP( "<b>Labels - x axis:</b> <i>( M columns )</i><br>"
        " Position of user-defined labels on the <b>X</b> axis." ),

        QT_TR_NOOP( "<b>Labels - y axis:</b> <i>( N columns )</i><br>"
        " Position of user-defined labels on the <b>Y</b> axis." ),

        QT_TR_NOOP( "<b>Labels - v axis:</b> <i>( O columns )</i><br>"
        " Position of user-defined labels on the ( invisible ) <b>V</b> axis." ),

        QT_TR_NOOP( "<b>Labels - z axis:</b> <i>( Q columns )</i><br>"
        " Position of user-defined labels on the <b>Z</b> axis." ),

        QT_TR_NOOP( "<b>Labels - x axis:</b> <i>( P columns )</i><br>"
        " Position of user-defined labels on the second <b>X</b> axis." ),

        QT_TR_NOOP( "<b>Labels - y axis:</b> <i>( R columns )</i><br>"
        " Position of user-defined labels on the second <b>Y</b> axis." ),

        QT_TR_NOOP( "<b>Labels - v axis:</b> <i>( S columns )</i><br>"
        " Position of user-defined labels on the ( invisible ) second <b>V</b> axis." ),

        QT_TR_NOOP( "<b>Labels - z axis:</b> <i>( T columns )</i><br>"
        " Position of user-defined labels on the second <b>Z</b> axis." )
       };

//-------------------------------------------------------------//


static const char *ccurve[] = {
        QT_TR_NOOP( "0. X Vector"         ),
        QT_TR_NOOP( "1. Y Vector"         ),
        QT_TR_NOOP( "2. DX Vector"        ),
        QT_TR_NOOP( "3. DY Vector"        ),
        QT_TR_NOOP( "4. Line Styles"      ),
        QT_TR_NOOP( "5. Fill Styles"      ),
	QT_TR_NOOP( "6. Point Styles "    ),
	QT_TR_NOOP( "7. Arrow Styles"     ),
	QT_TR_NOOP( "8. ErrorLine Styles" ),
	QT_TR_NOOP( "9. X Line Styles" ),
	QT_TR_NOOP( "10. Y Line Styles" )
       };

static const char *dcurve[] = {
	QT_TR_NOOP( " X column vector ( M rows ) "),
	QT_TR_NOOP( " Y column vector ( M rows ) "),
	QT_TR_NOOP( " DX column vector ( M rows ) "),
	QT_TR_NOOP( " DY column vector ( M rows ) "),
	QT_TR_NOOP( " Line styles column vector ( M-1 rows ) "
	"Invisible "LINE_INVISIBLE", "
	"Solid "LINE_SOLID", "
	"Dashed "LINE_DASH", "
	"Dotted "LINE_DOT", "
	"Dashed-Dotted "LINE_DASHDOT", "
	"Dashed-Dotted-Dotted "LINE_DASHDOTDOT", "),
	
	QT_TR_NOOP( " Fill styles column vector ( M-1 rows ) "
	"Transparent "FILL_TRANSPARENT", "
	"Solid "FILL_SOLID", "
	"Horizontal "FILL_HORIZ", "
	"Vertical "FILL_VERT", "
	"Crossed "FILL_CROSS", "
	"BDiagonal "FILL_BDIAG", "
	"FDiagonal "FILL_FDIAG", "
	"CrossDiagonal "FILL_DIAGCROSS", "
	"Half "FILL_HALF"<BR>"),

	QT_TR_NOOP( " Point styles column vector ( M rows ) "),
	QT_TR_NOOP( " Arrow styles column vector ( M rows ) "),
	QT_TR_NOOP( " Error line styles vector ( M rows ) "),
        QT_TR_NOOP( " X line styles vector ( M rows ) "),
        QT_TR_NOOP( " Y line styles vector ( M rows ) ")
	};
	
//-------------------------------------------------------------//



static const char *ccontour[] = {
        QT_TR_NOOP( "0. X Vector"    ),
        QT_TR_NOOP( "1. Y Vector"    ),
	QT_TR_NOOP( "2. Data"        )
       };

static const char *dcontour[] = {
	QT_TR_NOOP( "<b>Column index</b> <i>( 1 x N vector )</i><br>"
        "Contains <i>x</i> coordinates of surface points. "
        "If it is empty, a default <i>(0,1,2...)</i> vector is used. "
        "This vector must be monotone. " ),
	QT_TR_NOOP( "<b>Row index</b> <i>( M x 1 vector )</i><br>"
        "Contains <i>y</i> coordinates of surface points. "
        "If it is empty, a default <i>(0,1,2...)</i> vector is used. "
        "This vector must be monotone. " ),
	QT_TR_NOOP( "<b>Surface values</b> <i>( M x N matrix )</i><br>"
        "This matrix contains surface <i>v</i> values."
        "<b>Surface value[i,j]</b> is a <i>v</i> value at the point"
        "<b>(Column index[i], Row index[j])</b> " )
	};
/*
        QT_TR_NOOP( "3. Line Styles" ),
        QT_TR_NOOP( "4. Fill Styles" ),
        QT_TR_NOOP( "5. Font Styles" )
	QT_TR_NOOP( " Line styles: <BR>"
	"Invisible - "LINE_INVISIBLE", "
	"Solid - "LINE_SOLID", "
	"Dashed - "LINE_DASH", "
	"Dotted - "LINE_DOT", "
	"Dashed-Dotted - "LINE_DASHDOT", "
	"Dashed-Dotted-Dotted - "LINE_DASHDOTDOT", " ),
	
	 QT_TR_NOOP( " Fill styles: <BR> "
	"Transparent "FILL_TRANSPARENT", "
	"Solid "FILL_SOLID", "
	"Horizontal "FILL_HORIZ", "
	"Vertical "FILL_VERT", "
	"Crossed "FILL_CROSS", "
	"BDiagonal "FILL_BDIAG", "
	"FDiagonal "FILL_FDIAG", "
	"CrossDiagonal "FILL_DIAGCROSS", "
	"Half "FILL_HALF"<BR>" ),

         QT_TR_NOOP( " Font styles ")

*/

//-------------------------------------------------------------//



static const char *cngcontour[] = {
	QT_TR_NOOP( "0. Points - x coords" ),
	QT_TR_NOOP( "1. Points - y coords" ),
	QT_TR_NOOP( "2. Points - z coords" ),
        QT_TR_NOOP( "3. Triangles"    ),
       };

static const char *dngcontour[] = {
        QT_TR_NOOP( "<b>Points - x coords</b> <i>( N x 1 matrix )</i><br>"
	"Values at the same position in <b>x,y,z-coords</b> define coordinates of a single point. "),
        QT_TR_NOOP( "<b>Points - y coords</b> <i>( N x 1 matrix )</i><br>"
	"Values at the same position in <b>x,y,z-coords</b> define coordinates of a single point. "),
        QT_TR_NOOP( "<b>Points - z coords</b> <i>( N x 1 matrix )</i><br>"
	"Values at the same position in <b>x,y,z-coords</b> define coordinates of a single point. "),
	QT_TR_NOOP( "<b>Triangles</b> <i>( 3 x N vector )</i><br>" ),

        QT_TR_NOOP( "<b>Triangles</b> <i>( M x 3 matrix )</i><br>"
        "Each value in this matrix is an index to a row in <b>x,y,z-coords</b>. "
	"i-th row of <b>Triangles</b> describes a one triangle. "),

	};

/*
        QT_TR_NOOP( "4. Line Styles" ),
        QT_TR_NOOP( "5. Fill Styles" ),
        QT_TR_NOOP( "6. Fill Styles" )		
	
	QT_TR_NOOP( " Line styles: <BR>"
	"Invisible - "LINE_INVISIBLE", "
	"Solid - "LINE_SOLID", "
	"Dashed - "LINE_DASH", "
	"Dotted - "LINE_DOT", "
	"Dashed-Dotted - "LINE_DASHDOT", "
	"Dashed-Dotted-Dotted - "LINE_DASHDOTDOT", " ),
	
	 QT_TR_NOOP( " Fill styles: <BR> "
	"Transparent "FILL_TRANSPARENT", "
	"Solid "FILL_SOLID", "
	"Horizontal "FILL_HORIZ", "
	"Vertical "FILL_VERT", "
	"Crossed "FILL_CROSS", "
	"BDiagonal "FILL_BDIAG", "
	"FDiagonal "FILL_FDIAG", "
	"CrossDiagonal "FILL_DIAGCROSS", "
	"Half "FILL_HALF"<BR>" ),

         QT_TR_NOOP( " Font styles ")
*/
//-------------------------------------------------------------//


static const char *cimage[] = {
        QT_TR_NOOP( "0. Column index"   ),
        QT_TR_NOOP( "1. Row index"      ),
        QT_TR_NOOP( "2. Red-Index-Gray" ),
        QT_TR_NOOP( "3. Green"          ),
        QT_TR_NOOP( "4. Blue"           ),
        QT_TR_NOOP( "5. Palette "       )
       };

static const char *dimage[] = {
        QT_TR_NOOP( "<b>Column index</b> <i>( 1 x N+1 vector )</i><br>"
        "This vector may be used to form a rectangular grid, where each "
        "pixel spreads over a single mesh. All pixels in column <b>C</b> spread "
        "from <b>Column index[C]</b> to <b>Column index[C+1]</b> on "
        "the X axis. This vector must be monotone. See also <b>Row index</b>." ),

        QT_TR_NOOP( "<b>Row index</b> <i>( M+1 x 1 vector )</i><br>"
        "This vector may be used to form a rectangular grid, where each "
        "pixel spreads over a single mesh. All pixels in row <b>R</b> spread "
        "from <b>Row index[R]</b> to <b>Row index[R+1]</b> on "
        "the Y axis. This vector must be monotone. See also <b>Column index</b>." ),

        QT_TR_NOOP( "<b>Red or Index or Gray</b> <i>( M x N matrix )</i><br> "
        "Pixel values. If <b>Green</b> and <b>Blue</b> channels "
        "contains matrix of the same <i>M x N</i> size, this acts as "
        "a red channel of a color value. If a <i>Palette</i> is not "
        "empty this acts as an index to the color palette. Otherwise "
        "acts a grey value of a pixel. " ),

        QT_TR_NOOP( "<b>Green</b> <i>( M x N matrix )</i><br>"
        "If <b>Red</b> and <b>Blue</b> channels "
        "contains matrix of the same <i>M x N</i> size, this acts as "
        "a green channel of a color value. Otherwise is not used." ),

        QT_TR_NOOP( "<b>Blue</b> <i>( M x N matrix )</i><br>"
        "If <b>Red</b> and <b>Green</b> channels "
        "contains matrix of the same <i>M x N</i> size, this acts as "
        "a green channel of a color value. Otherwise is not used." ),

        QT_TR_NOOP( "<b>Palette</b> <i>( C x 3 matrix )</i><br> "
        "Color palette Each row is occupied by one color in format "
        "( red, green, blue ). Channel <b>Red-Index-Gray</b> contains "
        "indexes to this palette ( row number ) for each pixel." )

       };

//-------------------------------------------------------------//

static const char *csurface[] = {
        QT_TR_NOOP( "0. Column index"   ),
        QT_TR_NOOP( "1. Row index"      ),
        QT_TR_NOOP( "2. Surface values" ),
        QT_TR_NOOP( "3. V Values"      ),
        QT_TR_NOOP( "4. Colors - red"   ),
        QT_TR_NOOP( "5. Colors - green" ),
        QT_TR_NOOP( "6. Colors - blue"  ),
        QT_TR_NOOP( "7. Colors - alpha" )
        };

static const char *dsurface[] = {

        QT_TR_NOOP( "<b>Column index</b> <i>( 1 x N vector )</i><br>"
        "Contains <i>x</i> coordinates of surface points. "
        "If it is empty, a default <i>(0,1,2...)</i> vector is used. "
        "This vector must be monotone." ),

        QT_TR_NOOP( "<b>Row index</b> <i>( M x 1 vector )</i><br>"
        "Contains <i>y</i> coordinates of surface points. "
        "If it is empty, a default <i>(0,1,2...)</i> vector is used. "
        "This vector must be monotone." ),

        QT_TR_NOOP( "<b>Surface values</b> <i>( M x N matrix )</i><br>"
        "This matrix contains surface <i>z</i> values."
        "<b>Surface value[i,j]</b> is a <i>z</i> value at the point"
        " <b>(Column index[i], Row index[j])</b>" ),

        QT_TR_NOOP( "<b>V values</b> <i>( M x N matrix )</i><br>"
        "This matrix contains surface <i>V</i> values used for "
        "plotting 4D data ( x, y, z, v ) " ),

        QT_TR_NOOP( "<b>Colors - red</b> <i>( M x N matrix )</i><br>"
        "Custom colors for the surface. <b>red[i,j]</b> contains a color for "
        "the point at <i>(i,j)</i>. If <b>green</b> and <b>blue</b> "
        "channels contain matrices of the same size, this acts as a red "
        "channel of the color value. If not, this is a gray value of a color." ),

        QT_TR_NOOP( "<b>Colors - green</b> <i>( M x N matrix )</i><br>"
        "Green channel. See <b>Colors - red</b>." ),

        QT_TR_NOOP( "<b>Colors - blue</b> <i>( M x N matrix )</i><br>"
        "Blue channel. See <b>Colors - red</b>." ),

        QT_TR_NOOP( "<b>Colors - alpha</b> <i>( M x N matrix )</i><br>"
        "Custom alpha channel of color values. See <b>Colors - red</b>." )
        };

//-------------------------------------------------------------//

static const char *cfigure[] = {
        QT_TR_NOOP( "0. Facets - x coords"  ),
        QT_TR_NOOP( "1. Facets - y coords"  ),
        QT_TR_NOOP( "2. Facets - z coords"  ),
        QT_TR_NOOP( "3. Facets - v coords" ),
        QT_TR_NOOP( "4. Table of vertices"  ),
        QT_TR_NOOP( "5. Colors - red"       ),
        QT_TR_NOOP( "6. Colors - green"     ),
        QT_TR_NOOP( "7. Colors - blue"      ),
        QT_TR_NOOP( "8. Colors - alpha"     )
        };

static const char *dfigure[] = {
        QT_TR_NOOP( "<b>Facets - x coords</b> <i>( N x M matrix )</i><br>"
	"Values at the same position in <b>x,y,z,v-coords</b> define coordinates of a single point. "
        "i-th row of <b>x coords</b> contains <i>x</i> coordinates of vertices of "
        "the i-th facet. <i>M</i> is a number of vertices of a single "
        "facet."),

        QT_TR_NOOP( "<b>Facets - y coords</b> <i>( N x M matrix )</i><br>"
	"Values at the same position in <b>x,y,z,v-coords</b> define coordinates of a single point. "
        "i-th row of <b>y coords</b> contains <i>y</i> coordinates of vertices of "
        "the i-th facet. <i>M</i> is a number of vertices of a single "
        "facet."),

        QT_TR_NOOP( "<b>Facets - z coords</b> <i>( N x M matrix )</i><br>"
 	"Values at the same position in <b>x,y,z,v-coords</b> define coordinates of a single point. "
        "i-th row of <b>x coords</b> contains <i>z</i> coordinates of vertices of "
        "the i-th facet. <i>M</i> is a number of vertices of a single "
        "facet."),

        QT_TR_NOOP( "<b>Facets - v coords</b> <i>( N x M matrix )</i><br>"
	"Values at the same position in <b>x,y,z,v-coords</b> define coordinates of a single point. "
        "i-th row of <b>v coords</b> contains <i>v</i> coordinates of of vertices of "
        "the i-th facet. <i>M</i> is a number of vertices of a single "
        "facet. Used when drawing 4D data."),

        QT_TR_NOOP( "<b>Facets - indices</b> <i>( A x B matrix )</i><br>"
        " Each value in this matrix is an index to a row in <b>x,y,z,v-coords</b>. "
        " Those matrices have only one column in this case. i-th row of <b>indices</b> describes a one facet. "),

        QT_TR_NOOP( "<b>Colors - red</b> <i>( N x M matrix )</i><br>"
        "i-th row of <b>red</b> contains colors for vertices of i-th facet "
        "If <b>green</b> and <b>blue</b> contains a matrix of the same "
        "size this acts as a red channel of a color value. If not, this is a grey "
        "value of a color."),

        QT_TR_NOOP( "<b>Colors - green</b> <i>( N x M matrix )</i><br>"
        "Custom colors. See <b>Colors - red</b>."),

        QT_TR_NOOP( "<b>Colors - blue</b> <i>( N x M matrix )</i><br>"
        "Custom colors. See <b>Colors - red</b>."),

        QT_TR_NOOP( "<b>Colors - alpha</b> <i>( N x M matrix )</i><br>"
        "Custom colors - alpha channel. "
        "If it is empty colors have default alpha value 255."
        "See <b>Colors - red</b>." )
        };

//-------------------------------------------------------------//

static const char *dsheet =
        QT_TR_NOOP( "<b>Sheet</b><br>"
		    "Choose File/New from the main menu to set a new data. "
		    "Double click on colun header to change column data type."
		    "Click a left mouse button to show menu." );


//-------------------------------------------------------------//

KSChannelList::KSChannelList( QWidget *parent )
: QListView( parent )
  {
   m_show_all_channels = true;
   m_default_channel = -1;
   prev_selected_data = NULL;
   prev_selected_channel = -1;
   m_data = NULL;
   setSorting(-1);
   setAllColumnsShowFocus( true );
   setRootIsDecorated( true );
   setTreeStepSize( 10 );
   setMultiSelection(false);
   setVScrollBarMode( AlwaysOn );
   setHScrollBarMode( AlwaysOff );
   connect( this, SIGNAL(selectionChanged()), this, SLOT(slot_show_description()) );
  }

//-------------------------------------------------------------//

KSChannelList::~KSChannelList()
  {
  }

//-------------------------------------------------------------//

void KSChannelList::setDataObject( QSData *data, bool showAllChannels, int defaultChannel, bool showRoot )
 {
  m_show_all_channels = showAllChannels;
  m_default_channel   = defaultChannel;
  m_show_root	      = showRoot;
  if ( (QSData *)m_data != data ) {
	if ( !m_data.isNull() )  {
		disconnect( m_data, SIGNAL(sigDataChanged(QSData*,int)), this, SLOT(slot_refresh_list(QSData*,int)) );
		disconnect( m_data, SIGNAL(sigChildListChanged()), this, SLOT(slot_refresh_contents()) );
		}
	m_data = data;
        if ( data ) {
		connect( m_data, SIGNAL(sigDataChanged(QSData*,int)), this, SLOT(slot_refresh_list(QSData*,int)) );
		connect( m_data, SIGNAL(sigChildListChanged()), this, SLOT(slot_refresh_contents()) );
		}

	 slot_refresh_contents();	
	}
 }

//-------------------------------------------------------------//

void KSChannelList::select( QSData *dataObject, int channel )
 {
  KSChannelListItem *citem = first();
  while( citem ) {
	if ( citem->dataObject() == dataObject && citem->channel() == channel ) setSelected( citem, true );
	citem = citem->next();
	}
 }

//-------------------------------------------------------------//

QString KSChannelList::channelNameFactory( QSData *bobject, int num )
  {
   if ( dynamic_cast<QSAxis *>(bobject)  ) if ( num >= 0 ) return QString( caxis[num]  ); else return tr("Axis - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSAxes2D *>(bobject)  ) if ( num >= 0 ) return QString( caxes2d[num]  ); else return tr("Axes2D - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSAxes3D *>(bobject)  ) if ( num >= 0 ) return QString( caxes3d[num]  ); else return tr("Axes3D - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSCurve *>(bobject)   ) if ( num >= 0 ) return QString( ccurve[num]   ); else return tr("Curve - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSImage *>(bobject)   ) if ( num >= 0 ) return QString( cimage[num]   ); else return tr("Image - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSGriddedContour *>(bobject) ) if ( num >= 0 ) return QString( ccontour[num] ); else return tr("Contour - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSNonGriddedContour *>(bobject) ) if ( num >= 0 ) return QString( cngcontour[num] ); else return tr("NgContour - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSSurface *>(bobject) ) if ( num >= 0 ) return QString( csurface[num] ); else return tr("Surface - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<QSFigure *>(bobject)  ) if ( num >= 0 ) return QString( cfigure[num]  ); else return tr("Figure - ") + ((QSGraphicalData *)bobject)->title();
   else
   if ( dynamic_cast<KSSheetList*>(bobject) ) return tr("Sheet list");
   else
   if ( dynamic_cast<KSSheet*>(bobject) ) {
	KSSheet *m = dynamic_cast<KSSheet*>(bobject);
        return tr("Sheet : ")+m->title();
	}
   return QString();
  }

//-------------------------------------------------------------//

QString KSChannelList::channelDescFactory( QSData *bobject, int num )
  {
   if ( num >= 0 ) {
   if ( dynamic_cast<QSAxis *>(bobject)  ) return QString( daxis[num]  );
   else
   if ( dynamic_cast<QSAxes2D *>(bobject)  ) return QString( daxes2d[num]  );
   else
   if ( dynamic_cast<QSAxes3D *>(bobject)  ) return QString( daxes3d[num]  );
   else
   if ( dynamic_cast<QSCurve *>(bobject)   ) return QString( dcurve[num]   );
   else
   if ( dynamic_cast<QSGriddedContour *>(bobject) ) return QString( dcontour[num] );
   else
   if ( dynamic_cast<QSNonGriddedContour *>(bobject) ) return QString( dngcontour[num] );
   else
   if ( dynamic_cast<QSImage *>(bobject)   ) return QString( dimage[num]   );
   else
   if ( dynamic_cast<QSSurface *>(bobject) ) return QString( dsurface[num] );
   else
   if ( dynamic_cast<QSFigure *>(bobject)  ) return QString( dfigure[num]  );
   else
   if ( dynamic_cast<KSSheet *>(bobject) ) return QString( dsheet );
   }
   return QString();
  }

//-------------------------------------------------------------//

QStringList KSChannelList::channelColumnTitlesFactory( QSData *object, int channel )
 {
  QStringList result;
  if ( channel >= 0 ) {
  if ( dynamic_cast<QSAxes2D *>(object) ||
       dynamic_cast<QSAxes3D *>(object) ||
       dynamic_cast<QSAxis *>(object) && channel == 0 ) {
		result += tr("value");
		result += tr("format");
		result += tr("major");
		result += tr("angle");		
		}
   else
   if ( dynamic_cast<QSCurve*>(object) && channel == 4 ||
	dynamic_cast<QSCurve*>(object) && channel == 8 ||
	dynamic_cast<QSAxis*>(object) && channel == 1 ) {
	// line
 	result += tr("l-style");
	result += tr("l-width");
	result += tr("l-red");
        result += tr("l-green");
        result += tr("l-blue");
        result += tr("l-alpha");
	}
   else
   if ( dynamic_cast<QSCurve*>(object) && channel == 5 ||
	dynamic_cast<QSAxis*>(object) && channel == 3 ) {
	// fill
 	result += tr("f-style");
	result += tr("f-red");
        result += tr("f-green");
        result += tr("f-blue");
        result += tr("f-alpha");
	}
   if ( dynamic_cast<QSAxis*>(object) && channel == 2 ) {
	// font
	result += tr("f-family");
	result += tr("f-size");
	result += tr("f-bold");
	result += tr("f-italic");
	result += tr("f-red");
	result += tr("f-green");
	result += tr("f-blue");
	result += tr("f-alpha");
	}
   else
   if ( dynamic_cast<QSCurve*>(object) && channel == 6 ) {
	// point
 	result += tr("p-style");
	result += tr("p-fill");
	result += tr("p-size");
	result += tr("p-red");
        result += tr("p-green");
        result += tr("p-blue");
        result += tr("p-alpha");	
	}
   else
   if ( dynamic_cast<QSCurve*>(object) && channel == 7 ) {
	// arrow
	result += tr("a1-style");
	result += tr("a1-size");
	result += tr("a2-style");
	result += tr("a2-size");
	}
   else
   if ( dynamic_cast<QSCurve*>(object) ) {
	switch( channel ) {
		case 0: result+=tr("x");break;
		case 1: result+=tr("y");break;
		case 2: result+=tr("dx");break;
		case 3: result+=tr("dy");break;
		}
	}
  }
  return result;
 }

//-------------------------------------------------------------//

KSMatrixEditorInterf *KSChannelList::createEditor( KSWorkbook *workbook, QSData *dataObject, int channel, QWidget *parent )
  {
   if ( dynamic_cast<KSSheet*>(dataObject) && dataObject->matrix(channel) ) {
	return new KSMatrixSheetEditor( workbook, dynamic_cast<KSSheet*>(dataObject), parent );
	}
   else if ( dataObject->matrix(channel) ) {
   	return new KSMatrixEditor( workbook, dataObject->matrix(channel), parent );
	}
   return NULL;
  }

// if ( dynamic_cast<KSSheet*>(matrix->dataObject()) ) return new KSMatrixSheetEditor( dynamic_cast<KSSheet*>(matrix->dataObject()), parent );

//-------------------------------------------------------------//

KSCfgPanelInterf *KSChannelList::createCfgPanel( QSMatrix *matrix, KSWorkbook *workbook, QWidget *parent )
  {
   if ( dynamic_cast<KSMatrixString*>(matrix) ) {
	return new KSMatrixStringPanel(workbook,dynamic_cast<KSMatrixString*>(matrix),parent);
	}
   else if ( dynamic_cast<KSMatrixWorksheetCellRange*>(matrix) ) {
	return new KSMatrixRefPanel(workbook,dynamic_cast<KSMatrixWorksheetCellRange*>(matrix),parent);
	}
   else if ( dynamic_cast<KSMatrix*>(matrix) ) {
	return new KSMatrixPanel(workbook,dynamic_cast<KSMatrix*>(matrix),parent);
	}
   else if ( dynamic_cast<KSMatrixFormula*>(matrix) ) {
	return new KSMatrixFormulaPanel(workbook,dynamic_cast<KSMatrixFormula*>(matrix),parent);
	}
   return NULL;
  }

//-------------------------------------------------------------//

void KSChannelList::slot_refresh_contents()
  {
   // try to select the same item after refreshing the contents
   prev_selected_data    = selected() ? selected()->dataObject() : NULL;
   prev_selected_channel = selected() ? selected()->channel() : -1;

   clear();
   if ( !m_data.isNull() ) {
	 add_qsdata_object_to_list( m_data );
	 for( int child_nr=0; child_nr<m_data->childCount(); child_nr++ )
		add_qsdata_object_to_list( m_data->child(child_nr) );	
	}

   slot_refresh_list(NULL,0);
  }

//-------------------------------------------------------------//

void KSChannelList::add_qsdata_object_to_list( QSData *data )
 {
  KSChannelListItem *parentItem = NULL;
  if ( m_show_root ) {
	parentItem = new KSChannelListItem( this, data, m_default_channel, channelNameFactory(data,-1),channelDescFactory(data,-1) );
	}
  if ( m_show_all_channels )
  for( int channel=data->channelCount()-1; channel>=0; channel-- ) {
	 if ( parentItem ) {
	 	(void )new KSChannelListItem( parentItem, data, channel,
			                channelNameFactory(data,channel),
				        QString::null,
				        channelDescFactory(data,channel) );
		} else {
	 	(void )new KSChannelListItem( this, data, channel,
			                channelNameFactory(data,channel),
				        QString::null,
				        channelDescFactory(data,channel) );		
		}

	}

  if ( parentItem ) setOpen( parentItem, true );
 }



//-------------------------------------------------------------//

void KSChannelList::slot_refresh_list(QSData*object, int channel)
  {
   KSChannelListItem *current = first();
   while( current ) {
	QString size;
	if ( current->m_data &&
             current->m_channel >= 0 &&
	     current->m_data->matrix(current->m_channel) )
	     size.sprintf("%dx%d",current->m_data->matrix(current->m_channel)->rows(),
				  current->m_data->matrix(current->m_channel)->cols() );
        current->setText( 1, size );
	if ( current->dataObject() &&
	     current->dataObject() == prev_selected_data &&
	     current->channel()    == prev_selected_channel ) {
					current->setSelected( TRUE );
					setCurrentItem( current );		
					triggerUpdate();
					}
	current = current->next();
	}

   prev_selected_data = NULL;
   prev_selected_channel = -1;
   emit sigDataChanged( object, channel );
  }

//-------------------------------------------------------------//

void KSChannelList::slot_show_description()
  {
   KSChannelListItem *citem = selected();
   if ( citem && citem->dataObject() ) {
	 emit sigShowText( channelDescFactory(citem->dataObject(), citem->channel()) );
	} else {
	 emit sigShowText( QString::null );
	}
  }











