/***************************************************************************
                                  main.cpp
                             -------------------
    begin                : nie mar  5 14:43:31 CET 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>	// open, dup2
#include <fcntl.h>

#include <klocale.h>
#include <kapp.h>
#include <kssocketio.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include "kmatplotshell.h"
#include "widgets/qsgattr.h"


#define STDOUT_FD 1
#define STDERR_FD 2

KSSocketIO *kmatplot_socket = NULL;

static KCmdLineOptions options[] = {
	{ "+[file]", I18N_NOOP("*.kmp file to open"), 0 },
	{ "fd <file_descriptor>", I18N_NOOP("Do not create a socket for input commands, but use a given file descriptor instead."), 0 },
	{ 0, 0, 0},
	};

int main(int argc, char *argv[])
 {
  KAboutData *about = new KAboutData( "kmatplot",
				      I18N_NOOP("KMatplot"), 
				      "0.2.2",
				      I18N_NOOP("Math function plotter"), 
				      KAboutData::License_GPL, 
				      "(c) 2000-2001 Kamil Dobkowski" );
  about->addAuthor("Kamil Dobkowski", 0, "kamildobk@poczta.onet.pl" );
  about->addCredit("Roland Walker", 0, "walker@ncbi.nlm.nih.gov" );
  KCmdLineArgs::init( argc, argv, about );
  KCmdLineArgs::addCmdLineOptions( options );
  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
  KApplication app;

  //
  // initialize socket
  kmatplot_socket = new KSSocketIO();
  QCString option_fd = args->getOption("fd");
  if ( !option_fd.isEmpty() ) {
	// send stdout and stderr to /dev/null ...
	int null_fd = open("/dev/null",O_WRONLY);

	
	dup2( null_fd, STDOUT_FD );
	dup2( null_fd, STDERR_FD );
	// bind to open socket ( socket is opened before parent process forked )
	// see interface/common.c
	kmatplot_socket->setFileDescriptor( option_fd.toUInt() );	
	}
	
  //
  // create the main window
  KMatplotShell *shell = new KMatplotShell();
  if ( args->count() == 1 ) shell->openURL( args->url(0) );
  shell->show();
  app.setMainWidget(shell);
  
  
  int result = app.exec();
  delete kmatplot_socket;
  return result;
}  
