/***************************************************************************
                                  kmatplotpart.cpp
                             -------------------
    begin                : Sat Mar 11 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmatplotpart.h"
#include "kmatplotfactory.h"
#include "kspanelmanager.h"
#include "kscommands.h"

#include "../widgets/qsaxes2d.h"
#include "../widgets/qsaxes3d.h"
#include "../widgets/qscurve.h"
#include "../widgets/qsimage.h"
#include "../widgets/qscontour.h"
#include "../widgets/qssurface.h"
#include "../widgets/qsfigure.h"
#include "../widgets/qsclegend.h"
#include "../dialogs/ksgriddlg.h"
#include "../formula/mpformula.h"
#include "../formula/mpsymbols.h"

#include "qscobjects.h"
#include "qsctools.h"
#include "ksglobalmatrixlist.h"
#include "ksprojectxml.h"
#include "ksobjectfactory.h"

#include <qmessagebox.h>
#include <kiconloader.h>
#include <kaction.h>
#include <kstdaction.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qdom.h>
#include <qtextstream.h>
#include <qpaintdevicemetrics.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qinputdialog.h>



KMatplotPart::KMatplotPart( QWidget *parent, const char *name )
: KParts::ReadWritePart(parent,name)
 {
  // initialize formula
  MPFormula::addGlobalSymbols( new MPCommonSymFactory() );


  setInstance( KMatplotFactory::global() );
  setXMLFile( "kmatplotpart.rc" );

  //
  // File actions
  //
  (void)KStdAction::print( this, SLOT(slotFilePrint()), actionCollection() );

  m_new_axes2d = new KAction( i18n("XY Axes"), 0, this, SLOT(slotFileNewAxes2D()), actionCollection(), "new_axes2d" );
  m_new_axes3d = new KAction( i18n("XYZ Axes"), 0, this, SLOT(slotFileNewAxes3D()), actionCollection(), "new_axes3d" );

  m_new_curve     = new KAction( i18n("&XY plot"),  "curve",  0, this, SLOT(slotEditAddCurve()),   actionCollection(), "new_curve" );
  m_new_image     = new KAction( i18n("&Pixmap"),  "pixmap",  0, this, SLOT(slotEditAddImage()),   actionCollection(), "new_pixmap" );
  m_new_contour   = new KAction( i18n("C&ontour"),  "contour",  0, this, SLOT(slotEditAddGriddedContour()),   actionCollection(), "new_gridded_contour" );
  m_new_ngcontour = new KAction( i18n("&Non Gridded contour"),  "contour",  0, this, SLOT(slotEditAddNonGriddedContour()),   actionCollection(), "new_non_gridded_contour" );
  m_new_surface   = new KAction( i18n("&Surface"),"surface", 0, this, SLOT(slotEditAddSurface()), actionCollection(), "new_surface" );
  m_new_figure    = new KAction( i18n("&Figure"), "figure", 0, this, SLOT(slotEditAddFigure()),  actionCollection(), "new_figure" );

  m_DPI = new KSelectAction( i18n("Print DPI"), 0, actionCollection(), "set_print_dpi" );
  QStringList dpi_items;
  dpi_items += "72";
  dpi_items += "120";
  dpi_items += "180";
  dpi_items += "270";
  dpi_items += "360";
  m_DPI->setItems( dpi_items );
  m_DPI->setCurrentItem( 0 );
  connect(m_DPI, SIGNAL( activated(int) ), this, SLOT( slotFilePrintDPI(int) ) );

  (void )new KAction( i18n("Printer Setup..."), 0, this, SLOT(slotFilePrinterSetup()), actionCollection(), "printer_setup" );

  // Edit actiona
  m_undo = KStdAction::undo( this, SLOT( slotEditUndo() ), actionCollection(), "undo" );
  m_redo = KStdAction::redo( this, SLOT( slotEditRedo() ), actionCollection(), "redo" );
  m_copy = KStdAction::copy(this, SLOT( slotEditCopy() ), actionCollection(), "copy" );
  m_copy_all = new KAction( i18n("Copy all"), 0, this, SLOT(slotEditCopyAll()), actionCollection(), "copy_all" );
  m_paste = KStdAction::paste(this, SLOT( slotEditPaste() ), actionCollection(), "paste" );
  m_cut = KStdAction::cut(this, SLOT( slotEditCut() ), actionCollection(), "cut" );
  m_delete = new KAction( i18n("&Delete"), "editdelete", CTRL+Key_D, this, SLOT(slotEditDelete()),     actionCollection(), "delete"      );
  m_new_legend = new KAction( i18n("Legend"), 0, this, SLOT(slotInsertLegend()), actionCollection(), "new_legend" );
  m_new_xaxis  = new KAction( i18n("&X axis"), "panel_x", 0, this, SLOT(slotEditAddXAxis()), actionCollection(), "add_x_axis" );
  m_new_yaxis  = new KAction( i18n("&Y axis"), "panel_y", 0, this, SLOT(slotEditAddYAxis()), actionCollection(), "add_y_axis" );
  m_new_zaxis  = new KAction( i18n("&Z axis"), "panel_z", 0, this, SLOT(slotEditAddZAxis()), actionCollection(), "add_z_axis" );
  m_new_vaxis  = new KAction( i18n("&V axis"), "panel_v", 0, this, SLOT(slotEditAddVAxis()), actionCollection(), "add_v_axis" );

  // Arrange
  m_send_to_back   = new KAction( i18n("Send To Back"), 0, this, SLOT(slotSendToBack()), actionCollection(), "send_to_back" );
  m_bring_to_front = new KAction( i18n("Bring To Front"), 0, this, SLOT(slotBringToFront()), actionCollection(), "bring_to_front" );
  m_raise = new KAction( i18n("Raise"), 0, this, SLOT(slotRaise()), actionCollection(), "raise" );
  m_lower = new KAction( i18n("Lower"), 0, this, SLOT(slotLower()), actionCollection(), "lower" );
  m_group   = new KAction( i18n("Group"), 0, this, SLOT(slotGroup()), actionCollection(), "group" );
  m_ungroup = new KAction( i18n("Ungroup"), 0, this, SLOT(slotUngroup()), actionCollection(), "ungroup" );
 (void )new KAction( i18n("Grid ..."), 0, this, SLOT(slotGrid()), actionCollection(), "grid" );

  // Tools
  QString tool_group = "Tools";
  m_tool_zoom    = new KToggleAction( i18n("Zoom"),  "zoom", 0, actionCollection(), "tool_zoom" );
  m_tool_locate  = new KToggleAction( i18n("Locate"),  "locate", 0, actionCollection(), "tool_locate" );
  m_tool_select  = new KToggleAction( i18n("Select"), "select", 0, actionCollection(), "tool_select" );
  m_tool_label   = new KToggleAction( i18n("Label"),  "label", 0, actionCollection(), "tool_label" );
  m_tool_arrow   = new KToggleAction( i18n("Arrow"), "arrow", 0, actionCollection(), "tool_arrow" );
  m_tool_rect    = new KToggleAction( i18n("Rect"), "rectangle", 0, actionCollection(), "tool_rectangle" );

  m_tool_zoom->setExclusiveGroup(tool_group);
  m_tool_label->setExclusiveGroup(tool_group);
  m_tool_select->setExclusiveGroup(tool_group);
  m_tool_arrow->setExclusiveGroup(tool_group);
  m_tool_rect->setExclusiveGroup(tool_group);
  m_tool_locate->setExclusiveGroup(tool_group);

  connect( m_tool_zoom,    SIGNAL( toggled( bool ) ), this, SLOT( slotToolZoom( bool ) )      );
  connect( m_tool_label,   SIGNAL( toggled( bool ) ), this, SLOT( slotToolLabel( bool ) )     );
  connect( m_tool_select,  SIGNAL( toggled( bool ) ), this, SLOT( slotToolSelect( bool ) )    );
  connect( m_tool_arrow,   SIGNAL( toggled( bool ) ), this, SLOT( slotToolArrow( bool ) )     );
  connect( m_tool_rect,    SIGNAL( toggled( bool ) ), this, SLOT( slotToolRectangle( bool ) ) );
  connect( m_tool_locate,  SIGNAL( toggled( bool ) ), this, SLOT( slotToolLocate( bool ) )    );

  // Settings actions
  (void)new KToggleAction( i18n("Hide S&liders"), 0, this, SLOT(slotViewSliders()), actionCollection(), "sliders" );

  KToggleAction *aProp = new KToggleAction( i18n("Show P&roperty panel"),  BarIcon("properties",  KMatplotFactory::global()), CTRL+Key_R, actionCollection(), "properties"  );
  connect( aProp, SIGNAL( toggled( bool ) ), this, SLOT( slotSetProperties( bool ) )   );

  KToggleAction *aAutoSet = new KToggleAction( i18n("Show &Apply button"), 0, actionCollection(), "apply_properties"  );
  connect( aAutoSet, SIGNAL( toggled( bool ) ), this, SLOT( slotApplyProperties( bool ) )   );
  m_auto_apply = true;

  (void)new KToggleAction( i18n("Hide &Rulers"), 0, this, SLOT(slotViewRulers()), actionCollection(), "rulers" );


  // View actions
  KToggleAction *aPage = new KToggleAction( i18n("&Full page"),  BarIcon("fullpage",  KMatplotFactory::global()), CTRL+Key_F, actionCollection(), "full_page"  );
  connect( aPage, SIGNAL( toggled( bool ) ), this, SLOT( slotViewFullPage( bool ) )   );

  KSelectAction *aPZoom = new KSelectAction( i18n("Page zoom"), 0, actionCollection(), "page_zoom" );
  QStringList zoom_items;
  zoom_items += "50%";
  zoom_items += "75%";
  zoom_items += "100%";
  zoom_items += "150%";
  zoom_items += "200%";
  aPZoom->setItems( zoom_items );
  aPZoom->setCurrentItem( 1 );
  connect( aPZoom, SIGNAL( activated(int) ), this, SLOT( slotViewPageZoom(int) ) );

  m_panel_mgr = NULL;
  m_print_dpi = 72.0;
  m_clear_flag = true;

  m_printer = new QPrinter();
  m_printer->setFullPage(true);

  m_view = new QSPlotView( parent );
  setWidget( m_view );
  connect( m_view->selection(), SIGNAL(sigListChanged()), this, SLOT(slot_selection_changed()) );
  connect( m_view, SIGNAL(sigActiveObjectChanged()), this, SLOT(slot_selected_object_changed()) );
  connect( m_view, SIGNAL(sigActiveAxesChanged()), this, SLOT(slot_active_axes_changed()) );
  connect( m_view, SIGNAL(sigPageBarClicked()), this, SLOT(slot_show_page_popup()) );
  connect( m_view, SIGNAL(sigCurrentPageChanged()), this, SLOT(slot_curr_page_changed()) );
  connect( m_view, SIGNAL(sigSliderPressed()), this, SLOT(slot_view_slider_pressed()) );
  connect( m_view, SIGNAL(sigSliderReleased()), this, SLOT(slot_view_slider_released()) );
  connect( m_view, SIGNAL(sigScrollBarPressed()), this, SLOT(slot_view_scrollbar_pressed()) );
  connect( m_view, SIGNAL(sigScrollBarReleased()), this, SLOT(slot_view_scrollbar_released()) );
  m_curr_ranges_cmd = NULL;
  m_curr_properties_cmd = NULL;
  slot_selected_object_changed();
  slot_active_axes_changed();
  slot_curr_page_changed();
  slotViewPageZoom( 1 );

  m_workbook = new KSWorkbook( m_view );
  m_workbook->clear();
  m_workbook->pageAdd( new QSPage(m_workbook) );
  m_workbook->setPrinter( m_printer );
  connect( m_workbook->commandHistory(), SIGNAL(sigNewUndo()), this, SLOT(slot_new_undo()) );
  connect( m_workbook->commandHistory(), SIGNAL(sigNewRedo()), this, SLOT(slot_new_redo()) );
  connect( m_workbook->commandHistory(), SIGNAL(sigError(const QString&)), this, SLOT(slot_show_error(const QString&)) );
  slot_new_undo();
  slot_new_redo();
  m_view->setWorkbook( m_workbook );

  slotSetProperties( true );
  aProp->setChecked( true );
  slotViewFullPage( true );
  aPage->setChecked( true );
  slotFilePrintDPI( 0 );
  //slotViewPageZoom( 0 );

  connect( m_workbook, SIGNAL(sigWorkbookChanged()), this, SLOT(slot_set_dirty()) );
 }

  //panelmgr = NULL;
  //connect( m_view, SIGNAL(message(const QString&)), this, SLOT(slotMessage(const QString&)) );

//-----------------------------------------------------------------------------//

KMatplotPart::~KMatplotPart()
 {
  m_workbook->commandHistory()->clear();
  delete m_curr_ranges_cmd;
  delete m_curr_properties_cmd;
  delete m_printer;
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::slotEditCut()
 {
  if ( m_view->selection()->count() > 0 ) {
	KSObjectFactory factory( m_workbook );
	factory.copyQSCObjectCollectionToClipboard( m_view->selection() );
	m_workbook->execute( new KSCmdRemoveCObject( m_view->selection() ) );
	}
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::slotEditCopy()
 {
  if ( m_view->selection()->count() > 0 ) {
	KSObjectFactory factory( m_workbook );
	factory.copyQSCObjectCollectionToClipboard( m_view->selection() );
	}
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::slotEditCopyAll()
 {
  if ( m_view->selection()->count() > 0 ) {
	KSObjectFactory factory( m_workbook );
	factory.setFlags( KSObjectFactory::CopyAllData );
	factory.copyQSCObjectCollectionToClipboard( m_view->selection() );	
	}
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::slotEditPaste()
 {
  KSObjectFactory factory( m_workbook );
  QSCObjectCollection *new_objects = factory.pasteQSCObjectCollectionFromClipboard();
  if ( new_objects ) {
  	bool ok = m_workbook->execute( new KSCmdAddCObject( new_objects, m_view->activeCollection() ) );
  	if ( ok ) for( int i=0; i<new_objects->count(); i++ ) m_view->selection()->add( new_objects->object(i) );		
	delete new_objects;
	}	
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditDelete()
 {
  m_workbook->execute( new KSCmdRemoveCObject( m_view->selection() ) );
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::setClearFlag( bool clear )
// see openFile
 {
  m_clear_flag = clear;
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::clearAll()
 {
  m_workbook->clear();
  m_workbook->sheets()->clearAll();
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::refreshSettings()
 {
  double dpi = m_print_dpi;

  if ( dpi ==  72.0 ) m_DPI->setCurrentItem( 0 );
  if ( dpi == 120.0 ) m_DPI->setCurrentItem( 1 );
  if ( dpi == 180.0 ) m_DPI->setCurrentItem( 2 );
  if ( dpi == 270.0 ) m_DPI->setCurrentItem( 3 );
  if ( dpi == 360.0 ) m_DPI->setCurrentItem( 4 );
 }

//-----------------------------------------------------------------------------//

bool KMatplotPart::openFile()
 {
  if ( m_clear_flag ) clearAll();

  // use 'm_file' member
  QFile f( m_file );
  //if ( !f.open( IO_ReadOnly ) ) return false;

  KSProjectXML *xml = new KSProjectXML( this );
  bool result = xml->loadFromFile( f );
  delete xml;
  f.close();

  return result;
 }

//-----------------------------------------------------------------------------//

bool KMatplotPart::saveFile()
 {
  // use 'm_file' member
  if ( !isReadWrite() ) return false;
  QFile f( m_file );
  if ( !f.open( IO_WriteOnly ) ) return false;

  KSProjectXML *xml = new KSProjectXML( this );
  bool result = xml->saveToFile( f );
  delete xml;

  f.close();
  return result;
 }

//-----------------------------------------------------------------------------//

void KMatplotPart::setReadWrite( bool rw )
 {
  KParts::ReadWritePart::setReadWrite( rw );
 }

//-------------------------------------------------------------//

void KMatplotPart::slot_show_page_popup()
 {
  QPopupMenu *menu = new QPopupMenu( m_view );
  int add_id = menu->insertItem( tr("Add a new page") );
  int rename_id = menu->insertItem( tr("Rename") );
  int remove_id = menu->insertItem( tr("Remove") );
  menu->insertSeparator();
  int raise_id = menu->insertItem( tr("Raise") );
  int lower_id = menu->insertItem( tr("Lower") );
  menu->insertSeparator();
  int to_front_id = menu->insertItem( tr("To Front") );
  int to_back_id  = menu->insertItem( tr("To Back") );

  int result_id = menu->exec(QCursor::pos()); delete menu;
	
  if ( result_id == add_id ) {
	m_workbook->execute( new KSCmdAddPage( new QSPage(NULL), m_workbook ) );
	}
  else if ( result_id == raise_id ) {
	m_workbook->execute( new KSCmdPageOrder(KSCmdPageOrder::Raise,m_view->currentPage(),m_workbook) );	
	}
  else if ( result_id == lower_id ) {
	m_workbook->execute( new KSCmdPageOrder(KSCmdPageOrder::Lower,m_view->currentPage(),m_workbook) );
	}
  else if ( result_id == to_back_id ) {
	m_workbook->execute( new KSCmdPageOrder(KSCmdPageOrder::ToBack,m_view->currentPage(),m_workbook) );
	}
  else if ( result_id == to_front_id ) {
	m_workbook->execute( new KSCmdPageOrder(KSCmdPageOrder::ToFront,m_view->currentPage(),m_workbook) );
	}
  else if ( result_id == remove_id ) {
	m_workbook->execute( new KSCmdRemovePage(m_view->currentPage(),m_workbook) );
	}
  else if ( result_id == rename_id ) {
	bool ok_pressed = false;
  	QString new_title = QInputDialog::getText( tr("Enter a new page title"), QString::null, m_view->currentPage()->title(), &ok_pressed, m_view );
	if ( ok_pressed ) { m_view->currentPage()->setTitle( new_title ); }
	}	
 }

//-----------------------------------------------------------//

void KMatplotPart::slotGroup()
 {
  m_workbook->execute( new KSCmdCObjectGroup( m_view->selection(), m_view->selection() ) );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotUngroup()
 {
  if ( dynamic_cast<QSCGroup*>(m_view->activeObject()) )
	m_workbook->execute( new KSCmdCObjectUngroup( dynamic_cast<QSCGroup*>(m_view->activeObject()), m_view->selection() ) );
 }


//-----------------------------------------------------------//

void KMatplotPart::slotBringToFront()
 {
  m_workbook->execute( new KSCmdObjectOrder( KSCmdObjectOrder::ToFront, m_view->selection(), m_workbook ) );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotSendToBack()
 {
  m_workbook->execute( new KSCmdObjectOrder( KSCmdObjectOrder::ToBack, m_view->selection(), m_workbook ) );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotRaise()
 {
  m_workbook->execute( new KSCmdObjectOrder( KSCmdObjectOrder::Raise, m_view->selection(), m_workbook ) );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotLower()
 {
  m_workbook->execute( new KSCmdObjectOrder( KSCmdObjectOrder::Lower, m_view->selection(), m_workbook ) );
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_new_undo()
 {
  m_undo->setEnabled( m_workbook->commandHistory()->isUndoPossible() );
  m_undo->setText( tr("Undo: ")+m_workbook->commandHistory()->undoCommandTitle() );
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_new_redo()
 {
  m_redo->setEnabled( m_workbook->commandHistory()->isRedoPossible() );
  m_redo->setText( tr("Redo: ")+m_workbook->commandHistory()->redoCommandTitle() );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditUndo()
 {
  m_workbook->commandHistory()->undo();
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditRedo()
 {
  m_workbook->commandHistory()->redo();
 }

//-----------------------------------------------------------//

void KMatplotPart::slotViewSliders()
 {
  m_view->setSlidersVisible( !m_view->slidersVisible() );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotViewRulers()
 {
  m_view->setRulersVisible( !m_view->rulersVisible() );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotViewFullPage( bool enabled )
 {
  m_view->setFullPage( enabled );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotViewPageZoom( int index )
 {
  switch( index ) {
  	case 0: m_view->setZoom( 0.50 ); break;
  	case 1: m_view->setZoom( 0.75 ); break;
  	case 2: m_view->setZoom( 1.00 ); break;
  	case 3: m_view->setZoom( 1.50 ); break;
  	case 4: m_view->setZoom( 2.00 ); break;
  	}
 }

//-----------------------------------------------------------//

  //msg += i18n("Application (ksetapp): ") + QString::number(KMatplotPart::socketIO()->appNumber()) + "\n";
  //msg += i18n("Window (ksetplot): ") + QString::number(part->partId()) + "\n";
  //msg += i18n("Socket : ") + (const char *)KMatplotPart::socketIO()->socketName();
  //slotMessage( msg );

//-----------------------------------------------------------//

void KMatplotPart::slotSetProperties( bool enable )
 {
  delete m_panel_mgr;
  m_panel_mgr = NULL;

  if ( enable ) {   
    m_panel_mgr = new KSPanelManager( m_view );
    m_panel_mgr->setView( m_view );
    m_panel_mgr->setEnabled( true );
    m_panel_mgr->setAutoSetProperties( m_auto_apply );
    m_view->setConfigPanel( m_panel_mgr );
    m_panel_mgr->show();
    }
 }

//-----------------------------------------------------------//

void KMatplotPart::slotApplyProperties( bool enable )
 {
  m_auto_apply = !enable;
  if ( m_panel_mgr ) m_panel_mgr->setAutoSetProperties( m_auto_apply );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotFileNewAxes2D()
 {
  QSAxes *new_axes = new QSAxes2D();
  bool ok = m_workbook->execute( new KSCmdAddCObject( new_axes->shadowObject(), m_view->currentPage()->objects() ) );
  if ( ok ) m_view->selection()->set( new_axes->shadowObject() );
 }


//-----------------------------------------------------------//

void KMatplotPart::slotFileNewAxes3D()
 {
 QSAxes *new_axes = new QSAxes3D();
 bool ok = m_workbook->execute( new KSCmdAddCObject( new_axes->shadowObject(), m_view->currentPage()->objects() ) );
 if ( ok ) m_view->selection()->set( new_axes->shadowObject() );
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_curr_page_changed()
 {
  if ( m_view->currentPage() ) {
	m_new_axes2d->setEnabled( TRUE );
	m_new_axes3d->setEnabled( TRUE );
	} else {
 	m_new_axes2d->setEnabled( FALSE );
	m_new_axes3d->setEnabled( FALSE );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_active_axes_changed()
 {
  m_view->bindScrollBar( QSPlotView::HorizontalBar, QSAxis::XAxisType );
  m_view->bindScrollBar( QSPlotView::VerticalBar, QSAxis::YAxisType );
  if ( dynamic_cast<QSAxes2D*>(m_view->activeAxes()) && m_view->fullPage() ) {
  	m_view->bindSlider( QSPlotView::HorizontalSlider, "", 0, 0 );
  	m_view->bindSlider( QSPlotView::VerticalSlider,   "", 0, 0 );
 	m_view->bindSlider( QSPlotView::AdditionalSlider, "", 0, 0 );
  	}
  else
  if ( dynamic_cast<QSAxes3D*>(m_view->activeAxes()) && m_view->fullPage() ) {
  	m_view->bindSlider( QSPlotView::HorizontalSlider, "azimuth", 0, 359 );
  	m_view->bindSlider( QSPlotView::VerticalSlider, "elevation", -90, 90 );
  	m_view->bindSlider( QSPlotView::AdditionalSlider, "lightAzimuth", 0, 359 );
  	}
  else {
  	m_view->bindSlider( QSPlotView::HorizontalSlider, "", 0, 0 );
  	m_view->bindSlider( QSPlotView::VerticalSlider,   "", 0, 0 );
 	m_view->bindSlider( QSPlotView::AdditionalSlider, "", 0, 0 );
       }

  if ( m_view->activeAxes() ) {
	// enable actions
	m_new_curve->setEnabled( TRUE );
	m_new_image->setEnabled( TRUE );
	m_new_contour->setEnabled( TRUE );
	m_new_ngcontour->setEnabled( TRUE );
	m_new_surface->setEnabled( TRUE );
	m_new_figure->setEnabled( TRUE );
	m_new_legend->setEnabled( TRUE );
	m_new_xaxis->setEnabled( TRUE );
	m_new_yaxis->setEnabled( TRUE );
	m_new_zaxis->setEnabled( TRUE );
	m_new_vaxis->setEnabled( TRUE );
       } else {
	// disable actions
	m_new_curve->setEnabled( FALSE );
	m_new_image->setEnabled( FALSE );
	m_new_contour->setEnabled( FALSE );
	m_new_ngcontour->setEnabled( FALSE );
	m_new_surface->setEnabled( FALSE );
	m_new_figure->setEnabled( FALSE );
	m_new_legend->setEnabled( FALSE );
	m_new_xaxis->setEnabled( FALSE );
	m_new_yaxis->setEnabled( FALSE );
	m_new_zaxis->setEnabled( FALSE );
	m_new_vaxis->setEnabled( FALSE );
       }
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_selection_changed()
 {
  if ( m_view->selection()->count()>0 ) {
	m_cut->setEnabled( TRUE );
	m_copy->setEnabled( TRUE );
	m_copy_all->setEnabled( TRUE );
        m_delete->setEnabled( TRUE );
	m_bring_to_front->setEnabled( TRUE );
        m_send_to_back->setEnabled( TRUE  );
	m_raise->setEnabled( TRUE  );
	m_lower->setEnabled( TRUE  );
	} else {
	m_cut->setEnabled( FALSE );
	m_copy->setEnabled( FALSE );
	m_copy_all->setEnabled( FALSE );
        m_delete->setEnabled( FALSE );
	m_bring_to_front->setEnabled( FALSE );
        m_send_to_back->setEnabled( FALSE );
	m_raise->setEnabled( FALSE );
	m_lower->setEnabled( FALSE );
	}

  if ( m_view->selection()->count()>1 ) {
	m_group->setEnabled( TRUE );
	} else {
	m_group->setEnabled( FALSE );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_selected_object_changed()
 {
  if ( dynamic_cast<QSCGroup*>(m_view->activeObject()) && !m_view->activeObject()->isAxesShadow() ) {
	m_ungroup->setEnabled( TRUE );	
	} else {
	m_ungroup->setEnabled( FALSE );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_view_slider_pressed()
 {
  if ( m_view->activeAxes() ) {
	delete m_curr_properties_cmd;
        m_curr_properties_cmd = new KSCmdSetProperties( m_view->activeAxes() );
	// only register properties ( current value will be remembered )
	for ( int slider_nr=0; slider_nr<3; slider_nr++ ) {
        	QCString property_name = m_view->sliderProperty( (QSPlotView::SliderType )slider_nr);
		m_curr_properties_cmd->setProperty( property_name, m_view->activeAxes()->property(property_name) );
		}
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_view_slider_released()
 {
  if ( m_curr_properties_cmd ) {
	// update to new values
	for ( int slider_nr=0; slider_nr<3; slider_nr++ ) {
        	QCString property_name = m_view->sliderProperty( (QSPlotView::SliderType )slider_nr );
		m_curr_properties_cmd->setProperty( property_name, m_view->activeAxes()->property(property_name) );
		}
	m_workbook->execute( m_curr_properties_cmd );
	m_curr_properties_cmd = NULL;
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_view_scrollbar_pressed()
 {
  if ( m_view->activeAxes() ) {
	delete m_curr_ranges_cmd;
	m_curr_ranges_cmd = new KSCmdSetRanges( m_view->activeAxes() );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_view_scrollbar_released()
 {
  if ( m_curr_ranges_cmd ) {
	m_curr_ranges_cmd->commit();
	m_workbook->execute( m_curr_ranges_cmd );
	m_curr_ranges_cmd = NULL;
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_set_dirty()
 {
  if ( isReadWrite() ) setModified();
 }

//-----------------------------------------------------------//

void KMatplotPart::slot_show_error( const QString& message )
 {
  QMessageBox::critical( NULL, tr("Error"), message, QMessageBox::Ok, 0, 0 );
 }

//-----------------------------------------------------------//

bool KMatplotPart::check_axes()
 {
  if ( !m_view->activeAxes() ) {
  	QMessageBox::warning ( NULL, tr("Error"), tr("No axes selected"), QMessageBox::Ok, 0, 0 );
	return false;
	}
  return true;
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddCurve()
 {
  if ( check_axes() ) {
	m_workbook->execute( new KSCmdAddDataset(new QSCurve(m_view->activeAxes())) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::DatasetCategory, m_view->activeAxes()->plotCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddImage()
 {
  if ( check_axes() ) {
	m_workbook->execute( new KSCmdAddDataset(new QSImage(m_view->activeAxes())) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::DatasetCategory, m_view->activeAxes()->plotCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddGriddedContour()
 {
  if ( check_axes() ) {
	m_workbook->execute( new KSCmdAddDataset(new QSGriddedContour(m_view->activeAxes())) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::DatasetCategory, m_view->activeAxes()->plotCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddNonGriddedContour()
 {
  if ( check_axes() ) {
  	m_workbook->execute( new KSCmdAddDataset(new QSNonGriddedContour(m_view->activeAxes())) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::DatasetCategory, m_view->activeAxes()->plotCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddSurface()
{
 if ( check_axes() ) {
	m_workbook->execute( new KSCmdAddDataset(new QSSurface(m_view->activeAxes())) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::DatasetCategory, m_view->activeAxes()->plotCount()-1 );
	}
}

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddFigure()
 {
  if ( check_axes() ) {
  	m_workbook->execute( new KSCmdAddDataset(new QSFigure(m_view->activeAxes())) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::DatasetCategory, m_view->activeAxes()->plotCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddXAxis()
 {
  if ( check_axes() ) {
  	m_workbook->execute( new KSCmdAddAxis( new QSAxis(QSAxis::XAxisType,m_view->activeAxes()) ) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::AxisCategory, m_view->activeAxes()->axisCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddYAxis()
 {
  if ( check_axes() ) {
  	m_workbook->execute( new KSCmdAddAxis( new QSAxis(QSAxis::YAxisType,m_view->activeAxes()) ) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::AxisCategory, m_view->activeAxes()->axisCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddZAxis()
 {
  if ( check_axes() ) {
  	m_workbook->execute( new KSCmdAddAxis( new QSAxis(QSAxis::ZAxisType,m_view->activeAxes()) ) );
 	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::AxisCategory, m_view->activeAxes()->axisCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotEditAddVAxis()
 {
  if ( check_axes() ) {
  	m_workbook->execute( new KSCmdAddAxis( new QSAxis(QSAxis::VAxisType,m_view->activeAxes()) ) );
	if ( m_panel_mgr ) m_panel_mgr->selectPanel( QSAxes::AxisCategory, m_view->activeAxes()->axisCount()-1 );
	}
 }

//-----------------------------------------------------------//

void KMatplotPart::slotToolZoom( bool on )
 {
  if ( on ) m_view->setTool( new QSToolZoom() );
      else  m_view->setTool( NULL );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotToolSelect( bool on )
 {
  if ( on ) m_view->setTool( new QSToolSelect(this) );
      else  m_view->setTool( NULL );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotToolLabel( bool on )
 {
  if ( on ) m_view->setTool( new QSToolLabel() );
      else  m_view->setTool( NULL );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotToolArrow( bool on )
 {
  if ( on ) m_view->setTool( new QSToolArrow() );
      else  m_view->setTool( NULL );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotToolRectangle( bool on )
 {
  if ( on ) m_view->setTool( new QSToolRect() );
      else  m_view->setTool( NULL );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotToolLocate( bool on )
 {
  if ( on ) m_view->setTool( new QSToolLocate() );
      else  m_view->setTool( NULL );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotMessage( const QString& message )
 {
  setStatusBarText(message);
 }

//-----------------------------------------------------------//

void KMatplotPart::slotInsertLegend()
 {
  if ( check_axes() ) {
  	m_workbook->execute( new KSCmdAddCObject(new QSCLegend(m_view->activeAxes()),m_view->activeCollection()) );
  	}
 }

//-----------------------------------------------------------//

bool KMatplotPart::slotFilePrint()
 {
  bool end = false;
  while( !end )
  switch( QMessageBox::information( widget(),
                                    i18n("Print"),
                                    i18n("Print the current plot ?"),
                                    i18n("&Print"), i18n("Printer &Setup..."), i18n("&Cancel"),
                                    0, 2 ) ) {
        case 0: // Print
                end = true;
                break;
        case 1: // Page setup
                end = false;
                slotFilePrinterSetup();
                break;
        case 2: // Cancel
                end = true;
                return TRUE;
                break;
       }

  slotMessage(i18n("Printing. Please wait..."));

  QPainter paint(m_printer);
  QPaintDeviceMetrics pdm(m_printer);
  //double dpi = m_view->printerDpi();
  // set resolution of the paper
  paint.setWindow( 0,
		   0,
		   int(pdm.width()  * m_print_dpi / 72.0 + 0.5),
		   int(pdm.height() * m_print_dpi / 72.0 + 0.5) );
  m_workbook->print( &paint, m_printer, m_print_dpi );
  paint.end();
  slotMessage(i18n("Done."));

  return FALSE;
 }

//-----------------------------------------------------------//

void KMatplotPart::slotFilePrintDPI( int index )
 {
  int dpi = 72;
  switch( index ) {
  	case 0: dpi = 72;  break;
  	case 1: dpi = 120; break;
  	case 2: dpi = 180; break;
  	case 3: dpi = 270; break;
  	case 4: dpi = 360; break;
  	}
  m_print_dpi = dpi;
 }

//-----------------------------------------------------------//

void KMatplotPart::slotFilePrinterSetup()
 {
  m_printer->setMinMax( 1, m_workbook->pageCount() );
  if ( m_printer->setup(widget()) ) m_workbook->setPrinter( m_printer );
 }

//-----------------------------------------------------------//

void KMatplotPart::slotGrid()
 {
  KSGridDlg d( m_view, widget() );
  d.exec();
 }

//-------------------------------------------------------------//

void KMatplotPart::slotShowObjectMenu()
 {
  QPopupMenu *menu = new QPopupMenu( m_view );
  m_cut->plug( menu );
  m_copy->plug( menu );
  m_copy_all->plug( menu );
  m_paste->plug( menu );
  m_delete->plug( menu );
  menu->insertSeparator();

  QPopupMenu *popup_add_axis = new QPopupMenu( menu );
  m_new_xaxis->plug( popup_add_axis );
  m_new_yaxis->plug( popup_add_axis );
  m_new_zaxis->plug( popup_add_axis );
  m_new_vaxis->plug( popup_add_axis );
  menu->insertItem( tr("Add axis"), popup_add_axis );

  QPopupMenu *popup_add_dataset = new QPopupMenu( menu );
  m_new_curve->plug( popup_add_dataset );
  m_new_image->plug( popup_add_dataset );
  m_new_contour->plug( popup_add_dataset );
  m_new_ngcontour->plug( popup_add_dataset );
  m_new_surface->plug( popup_add_dataset );
  m_new_figure->plug( popup_add_dataset );
  menu->insertItem( tr("Add dataset"), popup_add_dataset );
  menu->insertSeparator();

  QPopupMenu *popup_order = new QPopupMenu( menu );
  m_bring_to_front->plug( popup_order );
  m_send_to_back->plug( popup_order );
  popup_order->insertSeparator();
  m_raise->plug( popup_order );
  m_lower->plug( popup_order );
  menu->insertItem( tr("Order"), popup_order );

  menu->exec(QCursor::pos());

 }

