/***************************************************************************
                                  kmatplotpart.h
                             -------------------
    begin                : Sat Mar 11 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMATPLOTPART_H
#define KMATPLOTPART_H


#include "ksmatrix.h"
#include "../widgets/qsgattr.h"
#include "../widgets/qsplotview.h"
#include "ksworkbook.h"
#include <kparts/part.h>

class QPrinter;
class QSCurve;
class QSCObject;
class KToggleAction;
class KSelectAction;

class QTextStream;
class QSMatrix;
class KSPanelManager;

class KSCmdSetRanges;
class KSCmdSetProperties;

/**
  *@author Kamil Dobkowski
  */
class KMatplotPart : public KParts::ReadWritePart  {

   Q_OBJECT

public:
    KMatplotPart(QWidget *parent=0, const char *name=0);
   ~KMatplotPart();

    void setClearFlag( bool clear );
    KSWorkbook *workbook() { return m_workbook; }
    QSPlotView *view() { return m_view; }
    KSPanelManager *panels() { return m_panel_mgr; }
    void refreshSettings();
    virtual void setReadWrite( bool rw );

public slots:


    void clearAll();
    void slotFileNewAxes2D();
    void slotFileNewAxes3D();
    void slotFilePrintDPI( int index );
    void slotFilePrinterSetup();
    bool slotFilePrint();

    void slotEditUndo();
    void slotEditRedo();
    void slotEditCut();
    void slotEditCopy();
    void slotEditCopyAll();
    void slotEditPaste();
    void slotEditAddCurve();
    void slotEditAddImage();
    void slotEditAddGriddedContour();
    void slotEditAddNonGriddedContour();
    void slotEditAddSurface();
    void slotEditAddFigure();
    void slotEditDelete();
    void slotEditAddXAxis();
    void slotEditAddYAxis();
    void slotEditAddZAxis();
    void slotEditAddVAxis();

    void slotBringToFront();
    void slotSendToBack();
    void slotRaise();
    void slotLower();
    void slotGrid();
    void slotGroup();
    void slotUngroup();

    void slotViewSliders();
    void slotViewRulers();
    void slotViewFullPage( bool );
    void slotViewPageZoom( int );

    void slotSetProperties( bool );
    void slotInsertLegend();
    void slotApplyProperties( bool );
    void slotToolZoom( bool );
    void slotToolSelect( bool );
    void slotToolLabel( bool );
    void slotToolArrow( bool );
    void slotToolRectangle( bool );
    void slotToolLocate( bool );
    void slotMessage( const QString& message );
    void slotShowObjectMenu();

protected:
   virtual bool openFile();
   virtual bool saveFile();

   QPrinter       *m_printer;
   QSPlotView     *m_view;
   KSWorkbook     *m_workbook;
   KSPanelManager *m_panel_mgr;

   double m_print_dpi;
   bool m_auto_apply;
   bool m_clear_flag;

   KSCmdSetRanges *m_curr_ranges_cmd;
   KSCmdSetProperties *m_curr_properties_cmd;

   KAction *m_new_axes2d;
   KAction *m_new_axes3d;
   KAction *m_new_curve;
   KAction *m_new_image;
   KAction *m_new_contour;
   KAction *m_new_ngcontour;
   KAction *m_new_surface;
   KAction *m_new_figure;
   KAction *m_new_legend;
   KAction *m_new_xaxis;
   KAction *m_new_yaxis;
   KAction *m_new_zaxis;
   KAction *m_new_vaxis;

   KAction* m_undo;
   KAction* m_redo;
   KAction* m_cut;
   KAction* m_copy;
   KAction* m_copy_all;
   KAction* m_paste;
   KAction* m_delete;
   KAction *m_bring_to_front;
   KAction *m_send_to_back;
   KAction *m_raise;
   KAction *m_lower;
   KAction *m_group;
   KAction *m_ungroup;

   KToggleAction *m_tool_zoom;
   KToggleAction *m_tool_locate;
   KToggleAction *m_tool_select;
   KToggleAction *m_tool_label;
   KToggleAction *m_tool_arrow;
   KToggleAction *m_tool_rect;
   KSelectAction *m_DPI;

private slots:
   void slot_selection_changed();
   void slot_selected_object_changed();
   void slot_active_axes_changed();
   void slot_set_dirty();
   void slot_new_undo();
   void slot_new_redo();
   void slot_show_error( const QString& messsage );
   void slot_show_page_popup();
   void slot_curr_page_changed();
   void slot_view_slider_pressed();
   void slot_view_slider_released();
   void slot_view_scrollbar_pressed();
   void slot_view_scrollbar_released();

private:
   bool check_axes();
};

#endif
