/***************************************************************************
                                  kspanelmanager.cpp
                             -------------------
    begin                : Sat Oct 7 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kspanelmanager.h"

#include "../dialogs/kscarrowpanel.h"
#include "../dialogs/ksclabelpanel.h"
#include "../dialogs/kscpospanel.h"
#include "../dialogs/kscrectpanel.h"
#include "../dialogs/ksclegendcpanel.h"
#include "../dialogs/ksaxispanel.h"
#include "../dialogs/kscubepanel.h"
#include "../dialogs/ksfigurepanel.h"
#include "../dialogs/ksframepanel.h"
#include "../dialogs/ksglpanel.h"
#include "../dialogs/ksgeneralpanel.h"
#include "../dialogs/ksimagepanel.h"
#include "../dialogs/kslightpanel.h"
#include "../dialogs/ksmeshpanel.h"
#include "../dialogs/ksseriespanel.h"
#include "../dialogs/kssurfacepanel.h"
#include "../dialogs/ksticspanel.h"
#include "../dialogs/ksviewpanel.h"
#include "../dialogs/kscontourpanel.h"
#include "../dialogs/ksgradientpanel.h"
#include "../dialogs/ksrangepanel.h"
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qobjcoll.h>
#include <klocale.h>

#include "../widgets/qsaxes2d.h"
#include "../widgets/qsaxes3d.h"
#include "../widgets/qscurve.h"
#include "../widgets/qscontour.h"
#include "../widgets/qsimage.h"
#include "../widgets/qssurface.h"
#include "../widgets/qsfigure.h"
#include "../widgets/qsclegend.h"
#include "../part/ksworkbook.h"
#include "qscobjects.h"

//-------------------------------------------------------------//

struct kspanelmanager_private_data {
         int curr_category;
	 int default_category;
	 KSPanel *curr_panel;
         QGuardedPtr<QSPlotView> curr_view;
         QGuardedPtr<QSCObject>  curr_object;
         KSWorkbook *workbook;

	 QFrame *categ_area;
	 QFrame *panel_area;
	 QFrame *main_area;
	 QFrame *help_area;
	 QLabel *info;	

	 QButtonGroup *categ_buttons;
	 QButtonGroup *panel_buttons;

         QPushButton *apply;
         QTimer      *atimer;
         QTimer      *rtimer;

	 void set_category( int id );
         void arrange_buttons( QFrame *button_area );	
	};

 //-------------------------------------------------------------//

void kspanelmanager_private_data::set_category( int id )
 {
  if ( categ_buttons->find(id) ) categ_buttons->find(id)->animateClick();
 }

//-------------------------------------------------------------//

void kspanelmanager_private_data::arrange_buttons( QFrame *button_area )
 {
  static const int buttons_in_row = 6;
  int x = button_area->contentsRect().x();
  int y = button_area->contentsRect().y();
  int w = button_area->contentsRect().width()/buttons_in_row;
  int h = 25;

  QObjectList *button_list = button_area->queryList( "QButton" );
  int button_nr = 0;
  QObjectListIt it( *button_list );
  while ( it.current() != NULL ) {
	 QButton *button = (QButton *)it.current();
	 if ( !button->isHidden() ) {
         	button->setFixedSize( w, h );
         	button->move( x+(button_nr%buttons_in_row)*w,
			      y+(button_nr/buttons_in_row)*h );
                button_nr++;
	       }
	 ++it;
	}

  button_area->setFixedHeight( button_area->childrenRect().size().height()+button_area->frameWidth() );
  delete button_list;
 }

//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//
//-------------------------------------------------------------//



KSPanelManager::KSPanelManager(QWidget *parent, const char *name )
: QFrame(parent,name)
 {
  // init panel manager
  is_enabled  = false;
  pr_reading  = false;
  pr_applying = false;
  pr_autoset  = true;
  resize( 160, 355 );
  setMinimumWidth( 160 );
  setMaximumWidth( 160 );
  setMinimumHeight( 1 );
  setFont(QFont("helvetica", 10));
  d = new kspanelmanager_private_data();

  // timers
  // instead of calling readProperties directly use rather rtimer->start( 250, TRUE );
  // instead of calling applyProperties directly use rather atimer->start( 250, TRUE );
  d->rtimer = new QTimer( this );
  d->atimer = new QTimer( this );
  connect( d->rtimer, SIGNAL(timeout()), this, SLOT(readProperties()) );
  connect( d->atimer, SIGNAL(timeout()), this, SLOT(applyProperties()) );

  // place for buttons, panels, help
  QVBoxLayout *l = new QVBoxLayout( this, 0, 5 );
  //l->setResizeMode( QLayout::FreeResize );
  d->categ_area = new QFrame( this );
  d->panel_area = new QFrame( this );
  d->main_area = new QFrame( this );
  d->help_area = new QFrame( this );

  d->info = new QLabel( d->help_area );
  d->info->setFont(QFont("helvetica", 9));
  d->info->setMargin( 3 );
  d->info->setAlignment( AlignTop | AlignLeft | WordBreak | DontClip );

  //d->categ_area->setFixedSize( 140, 30 );
  //d->panel_area->setFixedSize( 140, 25 );
  d->main_area->setFixedSize( 160, 330 );
  d->categ_area->setFrameStyle( StyledPanel | Sunken );
  d->panel_area->setFrameStyle( StyledPanel | Sunken );
  d->main_area->setFrameStyle( StyledPanel | Sunken );
  d->help_area->setFrameStyle( StyledPanel | Sunken );

  l->addWidget( d->categ_area );
  l->addWidget( d->panel_area );
  l->addWidget( d->main_area );
  l->addWidget( d->help_area, 1 );

  d->info->move( d->help_area->frameWidth(), d->help_area->frameWidth() );
  d->info->resize( 160, 1000 );

  // layouts for button boxes
  d->categ_buttons = new QButtonGroup( this ); d->categ_buttons->hide();
  d->panel_buttons = new QButtonGroup( this ); d->panel_buttons->hide();
  d->categ_buttons->setExclusive( true );
  d->panel_buttons->setExclusive( true );
  d->categ_buttons->hide();
  d->panel_buttons->hide();
  connect( d->categ_buttons, SIGNAL(pressed(int)), this, SLOT(categorySelected(int)) );
  connect( d->panel_buttons, SIGNAL(pressed(int)), this, SLOT(panelSelected(int)) );

  // current state
  d->curr_view   = NULL;
  d->curr_object = NULL;
  d->curr_panel  = NULL;
  d->curr_category = -1;

  // apply button
  d->apply  = new QPushButton( i18n("Apply"), d->main_area );
  d->apply->move( 45, 305 );
  d->apply->setFixedSize( 90, 25 );
  d->apply->setDefault(true);
  connect( d->apply, SIGNAL(clicked()), this, SLOT(applyProperties()) );


  setAutoSetProperties( true );
 }

//-------------------------------------------------------------//

KSPanelManager::~KSPanelManager()
 {
  // check if there is applyProperties waiting to be called
  if ( d->atimer->isActive() ) applyProperties();
  delete d;
 }

//-------------------------------------------------------------//

void KSPanelManager::setView( QSPlotView *view )
 {
  if ( d->curr_view ) {
         disconnect( d->curr_view, SIGNAL(sigActiveObjectChanged() ), this, SLOT(updatePanels()) );
	 disconnect( d->curr_view, SIGNAL(sigActiveAxesChanged() ), this, SLOT(updatePanels()) );
         disconnect( d->curr_view, SIGNAL(sigActiveAxesDatasetsChanged()), this, SLOT(updatePanels()) );
         disconnect( d->curr_view, SIGNAL(message( const QString&)), this, SLOT(setInfo(const QString&)) );
        }
  d->curr_view = view;
  d->workbook = dynamic_cast<KSWorkbook*>(view->workbook());
  if ( d->curr_view ) {
         connect( d->curr_view, SIGNAL(sigActiveObjectChanged() ), this, SLOT(updatePanels()) );
	 connect( d->curr_view, SIGNAL(sigActiveAxesChanged() ), this, SLOT(updatePanels()) );
         connect( d->curr_view, SIGNAL(sigActiveAxesDatasetsChanged()), this, SLOT(updatePanels()) );
         connect( d->curr_view, SIGNAL(message( const QString&)), this, SLOT(setInfo(const QString&)) );
        }

  updatePanels();
 }

//-------------------------------------------------------------//

QSPlotView *KSPanelManager::view() const
 {
  return (QSPlotView *)d->curr_view;
 }

//-------------------------------------------------------------//

void KSPanelManager::setInfo( const QString& text )
// Info area under config panel
 {
  d->info->setText( text );
 }

//-------------------------------------------------------------//

void KSPanelManager::setEnabled( bool enabled )
 {
  is_enabled = enabled;
  updatePanels();
 }

//-------------------------------------------------------------//

void KSPanelManager::setAutoSetProperties( bool enabled )
 {
  pr_autoset = enabled;
  if ( pr_autoset ) d->apply->hide();
  	       else d->apply->show();
 }

//-------------------------------------------------------------//

void KSPanelManager::updatePanels()
// called when plot changes or active object changes
 {
  if ( !d->curr_object.isNull() ) disconnect( d->curr_object, SIGNAL(sigUpdate()), this, SLOT(updateContents()) );
  if ( is_enabled ) {
		d->curr_object = get_curr_object();
                panelSelected( -1 );
		panel_button_factory();
                category_button_factory();
		d->set_category( d->default_category );
		}
  if ( !d->curr_object.isNull() ) connect( d->curr_object, SIGNAL(sigUpdate()), this, SLOT(updateContents()) );
 }

//-------------------------------------------------------------//

void KSPanelManager::categorySelected( int category_id )
 {
  d->curr_category = category_id;
  KSPanelButton *default_button = NULL;
  // show only those buttons, which belong to the current category
  int button_count = d->panel_buttons->count();
  for( int button_nr=0; button_nr<button_count; button_nr++ ) {
	KSPanelButton *button = panel_button( button_nr );
	if ( button && button->category() == category_id ) button->show();
					              else button->hide();
	if ( button && button->category() == category_id &&
	     default_button == NULL ) default_button = button;
	}

  d->arrange_buttons( d->panel_area );
  if ( default_button ) default_button->animateClick(); else panelSelected( -1 );
 }

//-------------------------------------------------------------//

void KSPanelManager::panelSelected( int button_id )
// called when new panel is selected
 {
  // check if contents of the currently removed panel changed
  if ( d->atimer->isActive() ) { d->atimer->stop(); applyProperties(); }

  // delete a previous panel	
  delete d->curr_panel; d->curr_panel = NULL;

  // create a new panel
  if ( is_enabled && panel_button(button_id) ) d->curr_panel = panel_button(button_id)->createPanel( d->main_area );

  // watch if its properties changed
  if ( d->curr_panel ) {
	d->curr_panel->move( 5, 5 );
        d->curr_panel->show();
	readProperties();
        connect( d->curr_panel, SIGNAL(propertiesChanged()), this, SLOT(updatePlot()) );
        connect( d->curr_panel, SIGNAL(message(const QString&)), this, SLOT(setInfo(const QString&)) );
        }
 }

//-------------------------------------------------------------//

void KSPanelManager::selectPanel( int category, int element )
// show panel of the dataset
 {
  if ( get_curr_object() )
  if ( category >= 0 || element >= 0 ) {
  	for( int button_nr=0; button_nr<d->panel_buttons->count(); button_nr++ ) {
		KSPanelButton *button = dynamic_cast<KSPanelButton*>(panel_button(button_nr));
		if ( button && button->isButtonFor( get_curr_object(), category, element ) ) {
			d->set_category( button->category() );
			button->animateClick();
			}
		}
	}
 }

//-------------------------------------------------------------//

void KSPanelManager::applyProperties()
 {
  pr_applying = true;
  if ( d->curr_panel ) d->curr_panel->applyProperties();
  pr_applying = false;
 }

//-------------------------------------------------------------//

void KSPanelManager::readProperties()
 {
  pr_reading = true;
  if ( d->curr_panel ) d->curr_panel->readProperties();
  pr_reading = false;
 }

//-------------------------------------------------------------//

void KSPanelManager::updateContents()
 {
  if ( is_enabled && !pr_autoset || !pr_applying ) d->rtimer->start( 250, TRUE );
 }

//-------------------------------------------------------------//

void KSPanelManager::updatePlot()
 {
  if ( is_enabled && pr_autoset && !pr_reading ) d->atimer->start( 1000, TRUE );
 }

//-------------------------------------------------------------//

QSCObject *KSPanelManager::get_curr_object()
 {
  if ( d->curr_view )
	if ( d->curr_view->activeObject() ) return d->curr_view->activeObject();
  return NULL;
 }

//-------------------------------------------------------------//

void KSPanelManager::add_button( KSPanelButton *button )
 {
  d->panel_buttons->insert( button );
 }

//-------------------------------------------------------------//

KSPanelButton *KSPanelManager::panel_button( int button_id )
 {
  return dynamic_cast<KSPanelButton*>(d->panel_buttons->find(button_id));
 }

//-------------------------------------------------------------//

void KSPanelManager::category_button_factory()
 {
  QButtonGroup *button_group = d->categ_buttons;

   // delete all buttons
  static const int category_max = 10;
  int button_count = button_group->count();
  for( int button_nr=0; button_nr<category_max; button_nr++ ) {
	QButton *button = button_group->find( button_nr );
	button_group->remove( button );
	delete button;
	}

  // add category buttons
  d->default_category = -1;
  button_count = d->panel_buttons->count();
  for( int button_nr=0; button_nr<button_count; button_nr++ )
       if ( panel_button(button_nr) ) {
	 KSPanelButton::ButtonCategory category = panel_button(button_nr)->category();
	 if ( !button_group->find(category) ) {
		KSPanelButton *new_category_button = KSPanelButton::categoryButton( category, d->categ_area );
		button_group->insert( new_category_button, category );
		new_category_button->show();
		}
	 if ( d->default_category == -1 ) d->default_category = category;
	}

  d->arrange_buttons( d->categ_area );
 }

//-------------------------------------------------------------//

void KSPanelManager::panel_button_factory()
 {
  QSCObject *curr_object = d->curr_object;
  QButtonGroup *button_group = d->panel_buttons;

  // delete all buttons
  int button_count = button_group->count();
  for( int button_nr=button_count-1; button_nr>=0; button_nr-- ) {
	QButton *button = button_group->find( button_nr );
	button_group->remove( button );
	delete button;
	}

  // fill button group with new buttons
  if ( curr_object && curr_object->isAxesShadow() ) {
	QSAxes *axes = curr_object->parentAxes();
	add_button( new KSPanelButtonRange(d->panel_area,d->workbook,axes) );
	add_button( new KSPanelButtonGeneral(d->panel_area,d->workbook,axes) );
	if ( dynamic_cast<QSAxes2D*>(axes) ) {
                QSAxes2D *axes2 = dynamic_cast<QSAxes2D*>(axes);
		add_button( new KSPanelButtonFrame(d->panel_area,d->workbook,axes2) );
		}
 	if ( dynamic_cast<QSAxes3D*>(axes) ) {
		QSAxes3D *axes3 = dynamic_cast<QSAxes3D*>(axes);
		add_button( new	KSPanelButtonCube(d->panel_area,d->workbook,axes3) );
		add_button( new KSPanelButtonView(d->panel_area,d->workbook,axes3) );	
		add_button( new KSPanelButtonLight(d->panel_area,d->workbook,axes3) );
		add_button( new KSPanelButtonGL(d->panel_area,d->workbook,axes3) );
		}
	for( int axis=0; axis<axes->axisCount(); axis++ ) {
		add_button( new KSPanelButtonAxis(d->panel_area,d->workbook,axes->axis(axis)) );
		add_button( new KSPanelButtonGrid(d->panel_area,d->workbook,axes->axis(axis)) );
		}
	for( int dataset=0; dataset<axes->plotCount(); dataset++ ) {
                if ( dynamic_cast<QSCurve*>(axes->plot(dataset)) ) {
			QSCurve *curve = dynamic_cast<QSCurve*>(axes->plot(dataset));
			add_button( new KSPanelButtonCurve(d->panel_area,d->workbook,curve) );
			}
                if ( dynamic_cast<QSContour*>(axes->plot(dataset)) ) {
                        QSContour *contour = dynamic_cast<QSContour*>(axes->plot(dataset));
			add_button( new KSPanelButtonContour(d->panel_area,d->workbook,contour) );
			add_button( new KSPanelButtonGradient(d->panel_area,d->workbook,contour) );
			}
                if ( dynamic_cast<QSImage*>(axes->plot(dataset)) ) {
			QSImage *image = dynamic_cast<QSImage*>(axes->plot(dataset));
			add_button( new KSPanelButtonImage(d->panel_area,d->workbook,image) );
			add_button( new KSPanelButtonGradient(d->panel_area,d->workbook,image) );			
			}
		if ( dynamic_cast<QSSurface*>(axes->plot(dataset)) ) {
			QSSurface *surface = dynamic_cast<QSSurface*>(axes->plot(dataset));
			add_button( new KSPanelButtonSurface(d->panel_area,d->workbook,surface) );
			add_button( new KSPanelButtonMesh(d->panel_area,d->workbook,surface) );
			}
		if ( dynamic_cast<QSFigure*>(axes->plot(dataset)) ) {
			QSFigure *figure = dynamic_cast<QSFigure*>(axes->plot(dataset));
			add_button( new KSPanelButtonFigure(d->panel_area,d->workbook,figure) );
			add_button( new KSPanelButtonMesh(d->panel_area,d->workbook,figure) );			
			}
		}
	}
  else
  if ( dynamic_cast<QSCLegend*>(curr_object) ) {
        QSCLegend *l = dynamic_cast<QSCLegend*>(curr_object);
	add_button( new KSPanelButtonCLegend(d->panel_area,d->workbook,l) );
	add_button( new KSPanelButtonCLabel(d->panel_area,d->workbook,l) );
	add_button( new KSPanelButtonCPos(d->panel_area,d->workbook,l,"posX","posY","posZ","coordX","coordY","coordZ") );
       }
  else
  if ( dynamic_cast<QSCArrow*>(curr_object) ) {
        QSCArrow *a = dynamic_cast<QSCArrow*>(curr_object);
	add_button( new KSPanelButtonCArrow(d->panel_area,d->workbook,a) );
	add_button( new KSPanelButtonCPos(d->panel_area,d->workbook,a,"originX","originY","originZ","originCoordX","originCoordY","originCoordZ") );
	add_button( new KSPanelButtonCPos(d->panel_area,d->workbook,a,"endX","endY","endZ","endCoordX","endCoordY","endCoordZ") );
        }
  else
  if ( dynamic_cast<QSCLabel*>(curr_object) ) {
        QSCLabel *l = dynamic_cast<QSCLabel*>(curr_object);
	add_button( new KSPanelButtonCLabel(d->panel_area,d->workbook,l) );
	add_button( new KSPanelButtonCPos(d->panel_area,d->workbook,l,"posX","posY","posZ","coordX","coordY","coordZ") );
        }
  else
  if ( dynamic_cast<QSCRect*>(curr_object) ) {
        QSCRect *r = dynamic_cast<QSCRect*>(curr_object);
	add_button( new KSPanelButtonCRect(d->panel_area,d->workbook,r) );
	add_button( new KSPanelButtonCPos(d->panel_area,d->workbook,r,"originX","originY","originZ","originCoordX","originCoordY","originCoordZ") );
	add_button( new KSPanelButtonCPos(d->panel_area,d->workbook,r,"endX","endY","endZ","endCoordX","endCoordY","endCoordZ") );
	}

  // receive help messages when cursor is over a button
  for( int button_nr=0; button_nr<d->panel_buttons->count(); button_nr++ )
     if ( panel_button(button_nr) )
	 connect( panel_button(button_nr), SIGNAL(message(const QString&)), this, SLOT(setInfo(const QString&)) );
 }

