/***************************************************************************
                                  kspanelmanager.h
                             -------------------
    begin                : Sat Oct 7 2000
    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@friko.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSPANELMANAGER_H
#define KSPANELMANAGER_H

#include <qwidget.h>
#include <qframe.h>
#include <qguardedptr.h>
#include <qstringlist.h>
#include <qevent.h>

#include "../widgets/qscobject.h"
#include "../widgets/qsplotview.h"
#include "../widgets/qsaxes.h"

class KSPanel;
class KSPanelButton;
struct kspanelmanager_private_data;

/**
  *@author Kamil Dobkowski
  */
class KSPanelManager : public QFrame  {

        Q_OBJECT

   public:
        /**
          * Constructor
          */
	KSPanelManager(QWidget *parent=0, const char *name=0);
	/**
	  * Destructor
	  */
	~KSPanelManager();
	/**
	  * Sets a new view to use with this panel manager.
	  */
	void setView( QSPlotView *view );
	/**
	  * Enables this panel manager to manage panels.
	  */
	void setEnabled( bool enabled );
	/**
	  * Apply properties to plot immediately after the controls change.
	  * Hide apply button
	  */
	void setAutoSetProperties( bool enabled );
	/**
	  * Returns enabled setting
	  */
	bool enabled() const { return is_enabled; }
	/**
	  *
	  */
	bool autoSetProperties() const { return pr_autoset; }
	/**
	  * Returns the current view
	  */
	QSPlotView *view() const;

  public slots:
 	/**
	  * Sets a short info app number and window number.
	  */
	void setInfo( const QString& info );
	void selectPanel( int category, int element );

   private:

        QSCObject *get_curr_object();
        void category_button_factory();
	void panel_button_factory();
	KSPanelButton *panel_button( int button_id );
        void add_button( KSPanelButton *button );

        bool is_enabled;
	bool pr_autoset;
	bool pr_reading;
	bool pr_applying;
	struct kspanelmanager_private_data *d;

   private slots:

        void updatePanels();
        void updatePlot();
        void updateContents();
        void categorySelected( int category_id );
        void panelSelected( int panel_id );
        void applyProperties();
        void readProperties();

};

#endif
