/***************************************************************************
                                 qsmatrix.h                               
                             -------------------                                         
    begin                : 01-January-2000
    copyright            : (C) 2000 by Kamil Dobkowski                         
    email                : kamildobk@poczta.onet.pl                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef QSMATRIX_H
#define QSMATRIX_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include<qstring.h>
#include<qobject.h>
#include"qsserializable.h"

class QSData;
/**
  * All matrix classes should inherit from this class.
  *
  */
class QSMatrix : public QObject, public QSSerializable {
	Q_OBJECT
		Q_PROPERTY( QString title READ title WRITE setTitle )
	public:
	  /**
            * Constructor
            */
	 QSMatrix();
	  /**
            * Destructor. Emits 'sigDeleting()'
            */
	 virtual ~QSMatrix();
	/**
	  * Returns a value at position ( row, col ) in this matrix,
	  * converted to 'double'. This method must be redefined.
	  */
	virtual double value( int row, int col ) = 0;
	/**
	  * Returns string at position ( row, col ).
	  * Default implementation calls value() and
	  * converts the result to string.
	  */
	virtual QString string( int row, int col ) { return QString::number(value(row,col),'g',9); }
	/**
	  * Returns the number of rows in this matrix.
	  */
	virtual int rows() const = 0;
	/**
	  * Return the number of columns in this matrix.
	  */
	virtual int cols() const = 0;
	 /**
           * Resizes matrix ( preserves old data ). This should be reimplemented for
	   * each matrix type.Default implementation does nothing
           */
        virtual bool resize( int rows, int cols );
	/**
	  * Does transposition. This should be reimplemented. Default implementation does nothing.
	  */
	virtual bool transpose() { return false; }
	/**
	  * Copy range.'dst' can be the same matrix.
	  */	
	void copyRange( int row, int col, QSMatrix *src, int srcStartRow, int srcStartCol, int srcEndRow, int srcEndCol );
	 /**
           * Sets a new name.
           */
	 void setTitle( const QString& title );
         /**
           * Returns a name
           */
	 QString title() const { return m_title; }
	 /**
           * Sets a data object. It is called atomaticallyby QSData object, so
           * there is no need to call it by hand.
           */
         virtual void setDataObject( QSData *data, int channel );
         /**
           * Returns data object
           */
	 QSData *dataObject() const { return m_data_object; }
	 /**
           *  Returns channel
           */
	 int channel() const { return m_channel; }
	 /**
           *  This should be reimplemented to set a new value if editable() is TRUE.
	   *  Doesn't call dataChanging() and dataChanged.
           */
         virtual void setValue( int row, int col, double value );
	 /**
           * Set string
           */
	 virtual void setString( int row, int col, const QString& string );
         /**
	   * If true editor will be painted in blue.
           */
	 virtual bool isReference() const;
         /**
           * If is editable using 'setValue()' this function should return true.
	   * Returns false by default.
           */
	 virtual bool isEditable() const;
	 /**
           * Returns if this matrix holds elements as string raher than matrix.
           */
	 virtual bool isString() const;	 	
	 /**
           * Calls QSData::dataChanging()
           */
	 virtual void dataChanging();
	 /**
           * Calls QSData::dataChanged()
           */
	 virtual void dataChanged();
	/**
          * Loads all properties
          */
	virtual void loadStateFromStream( QDataStream& stream, QSObjectFactory *factory );
	/**
          * Saves all properties
          */
	virtual void saveStateToStream( QDataStream& stream, QSObjectFactory *factory );

	protected:	
	 QString m_title;
	 QSData *m_data_object;
	 int m_channel;

      	 QSMatrix( const QSMatrix& );
      	 void operator=(const QSMatrix& );
	
	};
#endif








