/***************************************************************************
                                 qssurface.h
                             -------------------
    begin                : 01-January-2000

    copyright            : (C) 2000 by Kamil Dobkowski
    email                : kamildobk@poczta.onet.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __QSSURFACE_H
#define __QSSURFACE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include"qsplot.h"
#include"qsaxes.h"



/**
  * Base class for 3d surfaces.
  * @author  Kamil Dobkowski
  * @version 0.0.1
  */
class QSSurface : public QSPlot3D
   {
    Q_OBJECT
    public:
       	enum Channels   {
			  XVector=0,
       	                  YVector=1,
			  ZData = 2,
			  VData  =3,
       	                  ColorsR=4,
       	                  ColorsG=5,
       	                  ColorsB=6,
       	                  ColorsA=7 };
       	/**
         * Constructor.
	 */
       QSSurface(QSAxes* parent, const char * name=0);
	/**
          * Destructor.
	  */
       virtual ~QSSurface();
       virtual QString posInfo( QSPt2f& pos );
       virtual QSPt2f legendItemSize( QSDrv *drv );
       virtual void drawLegendItem( const QSPt2f& pos, QSDrv *drv );
	virtual void loadStateFromStream( QDataStream& stream, QSObjectFactory *factory );
	virtual void saveStateToStream( QDataStream& stream, QSObjectFactory *factory );
        virtual ColumnType columnType( int channel, int column ) const;
	virtual QString channelVariable( int channel ) const;

   protected:
      virtual void dataChanged( int channel = -1 );
      virtual void allocRuntimeData();
      virtual void freeRuntimeData();
      virtual bool getAxisRange( QSAxis *axis, double& min, double& max );
      virtual bool start();
      virtual bool step();
      virtual void end();

    private:
	struct surface_runtime_data;
	struct surface_runtime_data *d;
        bool   m_minmax_z_valid;
	bool   m_minmax_v_valid;
        double m_max_z;
        double m_min_z;
	double m_min_v;
	double m_max_v;

        void init_loops();	
        void alloc_buffers();
        void free_buffers();
	void init_colors();
	void prepare_quarter();
       	void calculate_normals( int i, int j );
     	QSGFill vertex_color( int i, int j );
     	QSPt3f vertex_normal();
   };

#endif


















































